/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.app;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import fr.upemlv.lgtools.distagger.AnneConfiguration;
import fr.upemlv.lgtools.distagger.Preprocessor;
import fr.upemlv.lgtools.tagging.LGTaggerConfig;
import fr.upemlv.lgtools.tagging.LGTaggerConstants;
import fr.upemlv.lgtools.tagging.LGTaggerParameters;
import fr.upemlv.lgtools.tagging.impl.LGTagger;
import fr.upemlv.lgtools.tagging.impl.LGTaggerFactory;
import fr.upemlv.lgtools.tagging.impl.Utils;
import fr.upemlv.lgtools.tagging.lex.Dico;
import fr.upemlv.lgtools.tagging.lex.InternalTagset;
import fr.upemlv.lgtools.tagging.lex.LookupMatch;
import fr.upemlv.lgtools.tagging.lex.Tag;
import fr.upemlv.lgtools.tagging.lex.Tagset;
import fr.upemlv.lgtools.tagging.nlp.UnitexProcess;
import fr.upemlv.lgtools.text.FullTaggedText;
import fr.upemlv.lgtools.text.LightRawText;
import fr.upemlv.lgtools.text.RawText;
import fr.upemlv.lgtools.text.Sentence;
import fr.upemlv.lgtools.text.TaggedElement;
import fr.upemlv.lgtools.text.TaggedText;
import fr.upemlv.lgtools.text.TaggedTextFactory;

/**
 * Program for jadt2012 experiments
 * 
 * @author Matthieu Constant
 *
 */

public class Jadt2012 {
	private static Pattern DISF_PATT = Pattern.compile("^EMPTY\\(([^)]+)\\)$");
	private static JadtScore startingDisScore = new JadtScore("STARTING");
	private static JadtDistrib startingDisDistrib = new JadtDistrib("STARTING TYPE");
	private static JadtScore stdScore = new JadtScore("STD");
	private static JadtScore disInDicMweScore = new JadtScore("DIS IN DIC MWE");
	
	//private static JadtScore mweScore = new JadtScore("MWE");
	private static JadtScore dicMweScore = new JadtScore("DIC MWE");
	//private static JadtScore notDicMweScore = new JadtScore("NOT DIC MWE");
	//private static JadtMultiScores lengthScores = new JadtMultiScores("length");
	private static JadtDistrib dicMweLengthScores = new JadtDistrib("dic mwe length");
	private static JadtDistrib disDicMweLengthScores = new JadtDistrib("disf dic mwe length");
	//private static JadtMultiScores notDicLengthScores = new JadtMultiScores("not dic length");
	//private static JadtMultiScores typeScores = new JadtMultiScores("type");
	private static JadtDistrib dicTypeScores = new JadtDistrib("dic mwe cat");
	private static JadtDistrib disDicTypeScores = new JadtDistrib("disfl dic mwe cat");
	//private static JadtMultiScores notDicTypeScores = new JadtMultiScores("not dic type");
	private static JadtDistrib stdDist = new JadtDistrib("STD DIST");
	//private static JadtDistrib mweDist = new JadtDistrib("MWE DIST");
	private static JadtDistrib dicMweDist = new JadtDistrib("DIC MWE DIST");
	//private static JadtDistrib notDicMweDist = new JadtDistrib("NOT DIC MWE DIST");
	/*private static JadtScore startingDisfScore = new JadtScore("STARTING DISF IN MWE");
	private static JadtScore internalDisfScore = new JadtScore("INTERNAL DISF IN MWE");
	private static JadtScore bothDisfScore = new JadtScore("BOTH DISF IN MWE");
	*/
	private static JadtScore dicStartingDisfScore = new JadtScore("DIC STARTING DISF IN MWE");
	private static JadtScore dicInternalDisfScore = new JadtScore("DIC INTERNAL DISF IN MWE");
	private static JadtScore dicBothDisfScore = new JadtScore("DIC BOTH DISF IN MWE");
	/*private static JadtScore notDicStartingDisfScore = new JadtScore("NOT DIC STARTING DISF IN MWE");
	private static JadtScore notDicInternalDisfScore = new JadtScore("NOT DIC INTERNAL DISF IN MWE");
	private static JadtScore notDicBothDisfScore = new JadtScore("NOT DIC BOTH DISF IN MWE");
	*/
	private static LookupMatch longestMatch(List<LookupMatch> matches){
		LookupMatch lm = null;
		int lastPos = -1;
		for(LookupMatch m:matches){
			if(m.getEndPosition() > lastPos){
				lastPos = m.getEndPosition();
				lm = m;
			}
		}
		return lm;
	}
	
	private static boolean isDisfluency(String unit){
		return DISF_PATT.matcher(unit).matches();
	}
	
	private static TaggedText filterMW(TaggedText text,List<String> resources,Tagset tagset) throws IOException, InterruptedException{
		UnitexProcess p = Utils.createUnitexProcess();
		String filename = "/tmp/mine";
		RawText t =  new LightRawText(text);
		t.saveForUnitex(filename, "UTF-8");
		Dico dic = p.getTextDico(filename, "French", resources, "dlc", "UTF-8", tagset);
		System.err.println(resources);
		TaggedText tt = new FullTaggedText(tagset);
		
		Iterator<Sentence<TaggedElement>> it = text.iterator();
		for(Sentence<String> sentence:t){
			Sentence<TaggedElement> ts = new Sentence<TaggedElement>();
			Sentence<TaggedElement> old = it.next();
			//System.err.println(sentence.getSequence());
			tt.addSentence(ts);
			int pos = 0;
			Iterator<List<LookupMatch>> lit = dic.lookup(sentence.getSequence()).iterator(); 
			while(lit.hasNext()){
				LookupMatch m = longestMatch(lit.next());
				//System.err.println(m);
				if((m != null) && (m.getEndPosition() - m.getStartPosition() != 0)){
					for(int i = 0 ; i <= (m.getEndPosition() - m.getStartPosition());i++){
					   ts.add(new TaggedElement("*"+old.get(pos).getLexicalValue(), old.get(pos).getTag()));
					   //System.err.println(new TaggedElement(pos+"=>*"+old.get(pos).getLexicalValue(), old.get(pos).getTag()));
					   //System.err.println(m);
					   pos++;
					   if(i < (m.getEndPosition() - m.getStartPosition())){
						   lit.next();
					   }
					}
					
				}
				else{					
					//System.err.println(pos+"=>"+old.get(pos));
					ts.add(old.get(pos));
					pos++;
				}
				
				
			}		
		}
		
		return tt;
	}
	
	private static TaggedText postprocess(SpokenText text,TaggedText tt){
		Tagset tagset = new InternalTagset(tt.getTagset());
		TaggedText res = new FullTaggedText(tagset);
		
		Iterator<List<Integer>> dit = text.getDisfluencies().iterator();
		Iterator<Sentence<TaggedElement>> stit = tt.iterator();
		for(Sentence<String> s:text.getText()){
			Sentence<TaggedElement> sentence = new Sentence<TaggedElement>();
			res.addSentence(sentence);
			Iterator<TaggedElement> tit = stit.next().iterator();
			Iterator<Integer> it = dit.next().iterator();
			int curr = it.hasNext()?it.next():-1;
			int pos = 0;
			//System.err.println("+++++++++++++++++++++++++");
			for(String w:s){
				
				//System.err.println("****"+w);
				//System.err.println(curr+"##"+pos);
				//System.err.println(pos+"==>"+s.get(pos));
				
				if(curr  == pos){					
					if(!s.get(pos).startsWith("@")){
						Tag t = new Tag(InternalTagset.EMPTY,tagset);
						String[] tab = w.split("\\$");
						TaggedElement e = new TaggedElement("EMPTY("+tab[1]+")", t);
						sentence.add(e);
						//System.err.println(e);						
					}
					curr = it.hasNext()?it.next():-1;
				}
				else{
					
					TaggedElement e = tit.next();
					//System.err.println(e+"---"+w);
					sentence.add(e);
				}
				pos++;
			}
		}
		return res;
	}
	
	
	private static boolean hasToBeMerged(TaggedElement e1,TaggedElement e2){
		if(e2 == null) return false;
		if(e2.getLexicalValue().contains(LGTaggerConstants.TOKEN_SEPARATOR)){
			//e2 is multiword
			if(e1.getLexicalValue().startsWith("EMPTY(") && e1.getLexicalValue().endsWith(")")){
			  //e1 is disfluency
				return true;
				/*
				if(!e1.getLexicalValue().equals("EMPTY(frag)") && !e1.getLexicalValue().equals("EMPTY(euh)")){
					//e1 is not a fragment and is not a "euh"
					return true;
				}*/
			}
			
		}
		return false;
	}
	
	private static List<Integer> getWordIndexesInMW(String w){
		int pos = 0;
		List<Integer> list = new LinkedList<Integer>();
		for(String t:getTokens(w)){
			if(!isDisfluency(t)){
				list.add(pos);
			}
			pos++;
		}
		return list;
	}
	
	private static int countStartingDisfluency(String w){
		List<Integer> list = getWordIndexesInMW(w);
		if(list.isEmpty()){
			return 0;
		}
		return list.get(0);
	}
	
	private static int countInternalDisfluency(String w){
		List<Integer> list = getWordIndexesInMW(w);
		if(list.isEmpty()){
			return 0;
		}
		int cnt = 0;
		int pos = list.get(0);
		for(int i:list){
			if(i - cnt != pos){
				cnt++;
			}
			pos++;
		}
		return cnt;
	}
	
	
	private static String[] getTokens(String w){
		return w.split(LGTaggerConstants.TOKEN_SEPARATOR);
	}
	
	private static int wordSize(String w){
		return getTokens(w).length;
	}
	
	private static int cleanedWordSize(String w){
		return wordSize(w) - getDisfluenciesCnt(w);
	}
	
	private static int getDisfluenciesCnt(String w){
		return getDisfluencies(w).size();
	}
	
	private static boolean isInDico(String word){
		for(String t:getTokens(word)){
		   	if(!isDisfluency(t) &&!t.startsWith("*")){
		   			return false;
		   	}
		   
		}
		return true;
	}
	
	private static boolean isMultiword(String word){
		return getTokens(word).length > 1;
	}
	
	
	
	
	private static List<String> getDisfluencies(String w){
		List<String> list = new LinkedList<String>();				
		for(String t:getTokens(w)){
		   	Matcher m = DISF_PATT.matcher(t);
		   	if(m.matches()){
		   		list.add(m.group(1));
		   	}
		}
		//if(list.size() > 1) System.err.println(list);
		return list;
	}
	
	
	
	private static void addInDistrib(JadtDistrib dist,List<String> subnames){
		for(String n:subnames){
			dist.add(n);
		}
	}
	
	private static void fillInternalStartingDisfScores(boolean hasStarting,boolean hasInternal,JadtScore starting,JadtScore internal, JadtScore both){
		if(hasStarting && hasInternal){
			both.inc(1, 1);
			starting.inc(0,1);
			internal.inc(0,1);
		}
		else{
			if(hasInternal){
				both.inc(0, 1);
				starting.inc(0,1);
				internal.inc(1,1);
			}
			else{
				if(hasStarting){
					both.inc(0, 1);
					starting.inc(1,1);
					internal.inc(0,1);
				}
				else{
					both.inc(0, 1);
					starting.inc(0,1);
					internal.inc(0,1);
				}
			}
		}
	}
	
	private static void count(TaggedText t){
		
		
		for(Sentence<TaggedElement> s:t){
			for(TaggedElement e:s){
				String w = e.getLexicalValue();
				String tag = e.getTag().getValue();
				int wordSize = wordSize(w);
				int disSize = getDisfluenciesCnt(w);
				List<String> disf = getDisfluencies(w);
				addInDistrib(stdDist, disf);
				stdScore.inc(disSize, wordSize);
				if(isMultiword(w)){
					int startingCnt = countStartingDisfluency(w);
					int internalCnt = countInternalDisfluency(w);
					boolean hasStarting = startingCnt != 0; 
					boolean hasInternal = internalCnt != 0;
					boolean hasDisfluency = (disSize != 0);
					//fillInternalStartingDisfScores(hasStarting,hasInternal,startingDisfScore,internalDisfScore, bothDisfScore);
					
					int cleanedWordSize = cleanedWordSize(w);
					//System.err.println(w);
					//mweScore.inc(disSize, wordSize);
					//lengthScores.increment(""+cleanedWordSize, disSize, wordSize);
					//typeScores.increment(tag, disSize, wordSize);
					//addInDistrib(mweDist, disf);
					
					if(isInDico(w)){
						if(hasInternal || hasStarting){
							startingDisScore.inc(startingCnt, internalCnt+startingCnt);
							if(hasStarting){
								addInDistrib(startingDisDistrib, Arrays.asList(getDisfluencies(w).get(0)));								
							}							
						}
						dicMweScore.inc(disSize, wordSize);
						dicMweLengthScores.add(""+cleanedWordSize);
						dicTypeScores.add(tag);
						if(hasDisfluency){
							disDicMweLengthScores.add(""+cleanedWordSize);
							disDicTypeScores.add(tag);
							disInDicMweScore.inc(disSize,1);
							//System.err.println(tag+"=>"+w);
						}
						//dicLengthScores.increment(""+cleanedWordSize, disSize, wordSize);
						//dicTypeScores.increment(tag, disSize, wordSize);
						addInDistrib(dicMweDist, disf);
						fillInternalStartingDisfScores(hasStarting,hasInternal,dicStartingDisfScore,dicInternalDisfScore, dicBothDisfScore);
					}
					else{
						//notDicMweScore.inc(disSize, wordSize);
						//notDicLengthScores.increment(""+cleanedWordSize, disSize, wordSize);
						//notDicTypeScores.increment(tag, disSize, wordSize);
						//addInDistrib(notDicMweDist, disf);
						//fillInternalStartingDisfScores(hasStarting,hasInternal,notDicStartingDisfScore,notDicInternalDisfScore, notDicBothDisfScore);
					}
				}
			}
		}
		stdScore.print();
		startingDisScore.print();
		startingDisDistrib.print();
		//mweScore.print();
		dicMweScore.print();
		disInDicMweScore.print();		
		//notDicMweScore.print();
		//lengthScores.print();
		dicMweLengthScores.print();
		disDicMweLengthScores.print();
		//notDicLengthScores.print();
		//typeScores.print();
		dicTypeScores.print();
		disDicTypeScores.print();
		//notDicTypeScores.print();
		stdDist.print();
		//mweDist.print();
		dicMweDist.print();
		//notDicMweDist.print();
		//internalDisfScore.print();
		//startingDisfScore.print();
		//bothDisfScore.print();
		dicInternalDisfScore.print();
		dicStartingDisfScore.print();
		dicBothDisfScore.print();
		//notDicInternalDisfScore.print();
		//notDicStartingDisfScore.print();
		//notDicBothDisfScore.print();
	}
	
	
	private static TaggedText adjust(TaggedText tt,Tagset tagset){
		TaggedText res = new FullTaggedText(tagset);
		for(Sentence<TaggedElement> s:tt){
			Sentence<TaggedElement> sres = new Sentence<TaggedElement>();
			res.addSentence(sres);
			for(int pos = 0 ; pos < s.size() ; pos++){
				TaggedElement e1 = s.get(pos);
				TaggedElement e2 = (pos+1 == s.size())?null:s.get(pos+1);
				if(hasToBeMerged(e1,e2)){
					e1 = new TaggedElement(e1.getLexicalValue()+LGTaggerConstants.TOKEN_SEPARATOR+e2.getLexicalValue(), e2.getTag());
					pos++;
				}
				sres.add(e1);
			}
		}
		
		return res;
	}
	
	
	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
		String DDIR = LGTaggerConstants.LGTAGGER_PATH + File.separator + "French" + File.separator+"spoken";
		AnneConfiguration dconfig = new AnneConfiguration();
		dconfig.setClasses(new File(DDIR+File.separator+"autocorrections.txt"));
		dconfig.setEuh(new File(DDIR+File.separator+"euh.txt"));
		dconfig.setInserts(new File(DDIR+File.separator+"insertions.txt"));
		dconfig.setMeta(new File(DDIR+File.separator+"meta.txt"));
		dconfig.setOutDir(new File("/tmp"));
		//File text = new File("021_C.trs.val");
		//File text = new File("/mnt/libre/data/refs/corpus/oral/corpusADok-corr.txt");
		File text = new File("/home/ladl/mconstan/data/refs/corpus/oral/corpusADok-corr.txt");
		//File text = new File("/tmp/tmpor.txt");
		
		dconfig.add(text);
		
		Preprocessor prep = new Preprocessor(dconfig);
		prep.preprocess(text);
		prep.write(dconfig.getOutDir()+File.separator+"res.txt");
		SpokenText t = new SpokenText("/tmp/res.txt.txt");
		//SpokenText t = new SpokenText("/tmp/tmp.txt");
		//System.err.println(t.getCleanText());

		//setting tagger
		LGTaggerParameters parameters = new LGTaggerParameters();
		parameters.setDecomposedText(true);
		String path = LGTaggerConstants.LGTAGGER_PATH;
		String SEP = File.separator;
		//String modelPath = path+SEP+"French"+SEP+"models"+SEP+"default-big-mwe";
		String modelPath = path+SEP+"French"+SEP+"models"+SEP+"oral-mwe";
		LGTaggerConfig config = new LGTaggerConfig(path+SEP+"French"+SEP+"config"+SEP+"default-mwe.cfg", false);
		parameters.setModel(modelPath);		
		LGTagger tagger = LGTaggerFactory.createStandardSingleModelTagger(parameters);		
		TaggedText tt = tagger.perform(t.getCleanText());
		Tagset tagset = new InternalTagset(config.getTagset());
		tt = filterMW(tt,config.getResources(),tagset);
		TaggedTextFactory.recomposeTaggedText(tt, false,false).save("/tmp/tagged.txt");		
		
		tt = TaggedTextFactory.recomposeTaggedText(postprocess(t, tt), false,true);
		tt = adjust(tt,tagset);
		new LightRawText(tt).saveAsIs("/tmp/result.txt");
		count(tt);
	}

}
