/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.app;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.LinkedList;
import java.util.List;

import fr.upemlv.lgtools.distagger.SimpleTagWriter;
import fr.upemlv.lgtools.text.FullRawText;
import fr.upemlv.lgtools.text.RawText;
import fr.upemlv.lgtools.text.Sentence;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class SpokenText {	
    private final RawText text = new FullRawText();
    private final RawText cleanText = new FullRawText();
    private final List<List<Integer>> disfluencies = new LinkedList<List<Integer>>();
    
    public SpokenText(String filename) throws IOException{
    	loadText(filename);
    }
    
    private void loadText(String filename) throws IOException{
    	BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(filename), "UTF-8"));
    	String line;
    	while((line = reader.readLine()) != null){
    		String[] words = line.split(" ");
    		List<String> sentence = new LinkedList<String>();
    		List<String> cleanSentence = new LinkedList<String>();
    		sentence.add("!");
    		cleanSentence.add("!");
    		List<Integer> disf = new LinkedList<Integer>();
    		if(words.length == 0){
    			//System.err.println("ICIII");
    			continue;
    		}
    		int pos = 1;
    		for(String w:words){
    			sentence.add(w);
    			if(!w.contains(SimpleTagWriter.DISFLUENCY_MARKER)){
    				cleanSentence.add(w);
    			}
    			else{
    				disf.add(pos);
    			}
    			pos++;
    		}
    		text.addSentence(new Sentence<String>(sentence));
    		cleanText.addSentence(new Sentence<String>(cleanSentence));
    		disfluencies.add(disf);
    	}
    	reader.close();
    }
    
    
    
    /**
	 * @return the text
	 */
	public RawText getText() {
		return text;
	}

	/**
	 * @return the cleanText
	 */
	public RawText getCleanText() {
		return cleanText;
	}

	/**
	 * @return the disfluencies
	 */
	public List<List<Integer>> getDisfluencies() {
		return disfluencies;
	}
	  
}
