/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;


public class AnneTagWriterForElag implements AnneTagWriter {

	private final AnneTagWriter tagWriterInDisfluency = new AnneTagWriter(){

		public String write(Word w) {
			return w.getForm();
		}

		public String write(Meta m) {
			return m.getValue();
		}

		public String write(Disfluency dis) {
			return null;
		}

		public String write(EndOverlapping eover) {
			if(!eover.getOrigin().equals("")){
				return " {-|,.IGN+overEnd}";
			}		
			return null;
		}

		public String write(BackRefOverlapping brover) {
			return null;
		}

		public String write(RefOverlapping rover) {
			String res = "|-";
			if(!rover.isInternal()){
				for(int i:rover.getRefs()){
					res+="@"+i;
				}
			}
			return res;
		}

		public String write(DisfluencySeq ds) {
			return null;
		}
		
	};
	
	
	public String write(Word w) {
		return " "+w.getValue();
	}

	public String write(Meta m) {
		String s = m.getForm();
		String val = s;
		if(val == null) return null;
	   s = "{"+val+",.IGN+"+m.getType()+"}";
		if(val.equals("//") || val.equals("(silence)")){
			s+= " {S}\n";
		}		
		return " "+s;
	}

	public String write(Disfluency dis) {
		String rm = "";
		for(AnneTag t:dis){
			rm += " "+t.getValue();
		}
		rm = rm.substring(1, rm.length());
		return " {"+rm+",.IGN+"+dis.getType()+"}";
	}

	public String write(EndOverlapping eover) {
		if(!eover.getOrigin().equals("")){
			return " {-|,.IGN+overEnd}";
		}		
		return null;
	}

	public String write(BackRefOverlapping brover) {
		return null;				
	}

	public String write(RefOverlapping rover) {
		String s = "";
		if(!rover.isInternal()){
			for (int i:rover.getRefs()){		
				s += " @"+i;
				}
			s = s.substring(1,s.length());
		}
		return " {|-"+s+",.IGN+over}";
	}
/*
	private boolean isFragOrEuh(Disfluency d){
		return d.getType().equals("euh")|| d.getType().equals("frag");
	}
	
	private String getDisfluencyAnnotation(Disfluency d){
		return "{"+d.getValue()+",.IGN+"+d.getType()+"}";
	}
	*/
	public String write(DisfluencySeq ds) {
		//particular case
/*		if(ds.getSeq().size() == 2 && isFragOrEuh(ds.getSeq().get(0))&& isFragOrEuh(ds.getSeq().get(1))){
			return " "+getDisfluencyAnnotation(ds.getSeq().get(0))+" "+getDisfluencyAnnotation(ds.getSeq().get(1));
		}*/

		String res="";
		for(AnneTag t:ds){
			 String s = t.write(tagWriterInDisfluency);
			 if(s != null){
				 res += " "+s;
			 }
		}
		res = res.substring(1, res.length());
		return " {"+res+",.IGN+"+ds.getType()+"}"; 
	}

}
