/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class DefaultTagWriter implements AnneTagXMLWriter {
   private Document document;
	
	public DefaultTagWriter(Document doc){
	   this.document = doc;	
	}
	
	public Element write(Word w) {
		Element e = document.createElement("word");
		e.setAttribute("pos", ""+w.getPos());
		e.setAttribute("value", w.getForm());
		return e;
	}

	public Element write(Meta m) {
		Element e = document.createElement("meta");
		e.setAttribute("pos", ""+m.getPos());
		e.setAttribute("type", m.getType());
		e.setAttribute("form", m.getForm());
		return e;
	}

	public Element write(Disfluency dis) {
		Element e = document.createElement("disf");
		e.setAttribute("pos", ""+dis.getPos());
		e.setAttribute("type",dis.getType().getType());
		Element rm = document.createElement("rm");
		for(AnneTag tag:dis.getRm()){
			rm.appendChild(tag.toXMLElement(this));
		}
		e.appendChild(rm);
		if(dis.getRr() == null) return e;
		Element rr = document.createElement("rr");		
		for(AnneTag tag:dis.getRr()){
			Element wordref = document.createElement("wordref");
			//rr.appendChild(tag.toXMLElement(this));
			wordref.setAttribute("value", ""+tag.getPos());
			rr.appendChild(wordref);
		}
		e.appendChild(rr);
		return e;
	}

	public Element write(EndOverlapping eover) {
		Element e = document.createElement("end_over");
		e.setAttribute("pos", ""+eover.getPos());
		return e;
	}

	public Element write(BackRefOverlapping brover) {
		Element e = document.createElement("over");
		e.setAttribute("pos", ""+brover.getPos());
		e.setAttribute("type", "backref");
		e.setAttribute("backref",""+brover.getBackRef());
		return e;
	}

	public Element write(RefOverlapping rover) {
		Element e = document.createElement("over");
		e.setAttribute("pos", ""+rover.getPos());
		e.setAttribute("type", "ref");
		for(int r:rover.getRefs()){
			Element c = document.createElement("ref");
			c.setAttribute("value", ""+r);
			e.appendChild(c);
		}
		return e;
	}

	public Element write(DisfluencySeq ds) {
		Element e = document.createElement("difluencySeq");
		e.setAttribute("pos", ""+ds.getPos());
		for(Disfluency d:ds.getSeq()){
			e.appendChild(d.toXMLElement(this));
		}		
		return e;
	}

}
