/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Element;


public class Disfluency implements AnneTag,Iterable<AnneTag> {
	private int pos;
	private Operation type;
	private final Slot parentSlot;
	private List<AnneTag> rm;
	private List<AnneTag>rr;

	public Disfluency(int pos,List<AnneTag> rm,List<AnneTag> rr,Slot parentSlot){
	    this(pos,null,rm,rr,parentSlot);	
	}
	
	public Disfluency(int pos, Operation type, List<AnneTag> rm,List<AnneTag> rr,Slot parentSlot){
		this.pos = pos;
		this.rm = rm;
		this.rr = rr;
		this.type = type;
		this.parentSlot = parentSlot;
	}
	
	
	public Slot getParentSlot() {
		return parentSlot;
	}

	public Operation getType() {
		return type;
	}

	public boolean isUsable() {
		return false;
	}

	public boolean isLeaf() {
		return false;
	}

	public String getValue() {
		return null;
	}
    
	public List<AnneTag> getRm() {
		return rm;
	}

	public List<AnneTag> getRr() {
		return rr;
	}

	public int getPos(){
		return pos;
	}	
	
	public String toString(){
		String res = "[dis "+getPos()+" ; "+type+" ; rm=";
		for(AnneTag t:getRm()){
			res +=""+t.getPos()+";";
		}
		res += " rr=";
		if(getRr() != null){
		  for(AnneTag t:getRr()){
			 res +=""+t.getPos()+";";
		  }
		}
		return res+"]";
	}

	public Element toXMLElement(AnneTagXMLWriter tagWriter) {
		return tagWriter.write(this);
	}
	
	public String write(AnneTagWriter tagWriter) {
		return tagWriter.write(this);
	}

	public int getLastPos() {
		//System.out.println("===="+(rm.get(rm.size() - 1).getLastPos()));
		return rm.get(rm.size() - 1).getLastPos();
	}

	public int getRrStartingPos(){
		if(rr == null || rr.size() == 0){
			return getLastPos()+1;
		}
		return rr.get(0).getPos();
	}
	
	public Iterator<AnneTag> iterator() {		
		return new DisfluencyIterator(this);
	}
	
}
