/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.util.ArrayList;
import java.util.List;

public class DisfluencyMatcher {
	private StopWords stopWords;
	private WordClasses wordClasses;
	private EuhClass euhClass;	
	private Operation currentOperation= Operation.REP;
	private boolean modified = true;

	public DisfluencyMatcher(StopWords stopWords,WordClasses wordClasses,EuhClass euhClass){
		this.stopWords = stopWords;
		this.wordClasses = wordClasses;
		this.euhClass = euhClass;
	}

	private void initCurrentOperation(){
		currentOperation = Operation.NONE;
	}

	private void setCurrentOperation(Operation op){
		switch(currentOperation){
		case REP: 
			currentOperation=op;
			if(op == Operation.FRAG){ 
				currentOperation = Operation.REPFRAG;
			}
			break;
		case EUH: break;
		case CORR: 
			if(op == Operation.FRAG){ 
				currentOperation = Operation.CORRFRAG;
			}
			break;
		default:
			currentOperation = op;
			break;    	     
		}
	}

	private Operation getCurrentOperation(){
		return currentOperation;
	}

	private  List<Disfluency> createAnnotationList(int size){
		List<Disfluency> list = new ArrayList<Disfluency>();
		for(int i = 0 ; i < size ; i++){
			list.add(null);
		}
		return list;
	}

	private boolean match(List<AnneTag> tags,List<Integer> t,int start1,int start2,int length){
		for(int i = 0; i <length ; i++){
			String w1 = tags.get(t.get(start1 + i)).getValue();
			String w2 = tags.get(t.get(start2 + i)).getValue();
			if(!w1.equals(w2)){
				if(!wordClasses.equivalentWords(w1, w2)){
					if(!isFragment(w1)){
						return false;  
					}
					else{
						w1=w1.substring(0,w1.length()-1);
						if(w2.startsWith(w1)){
							setCurrentOperation(Operation.FRAGREP);
						}
						else{
						    setCurrentOperation(Operation.FRAG);
						}
					}
				}
				else{
					setCurrentOperation(Operation.CORR);
				}
			}
			else{
				setCurrentOperation(Operation.REP);
			}

		}
		return true;
	}

	private boolean isFragment(String w){
		return w.endsWith("/");
	}

	private boolean isEuh(String w){
		return euhClass.contains(w);
	}
	
	private boolean insertHasFragment(List<AnneTag> tags,List<Integer> t,int start,int length){
		for(int i=0;i < length ; i++){
			if(isFragment(tags.get(t.get(start+i)).getValue())){
				return true;
			}
		}
		return false;
	}

	private int findNextDisfluency(int pos, List<Integer> t, List<Disfluency> annot,List<AnneTag> tags,Slot slot){
		int i = 1;
		initCurrentOperation();
		String w = tags.get(t.get(pos)).getValue();
		//System.out.println(w);
		if(isEuh(w)){
			setCurrentOperation(Operation.EUH);		   
			int position = t.get(pos);
			annot.set(position, DisfluencyFactory.createEuhDisfluency(position, tags,slot));
			t.remove(pos);
			return pos;
		}
		if(pos == t.size() - 1 && isFragment(w)){
			setCurrentOperation(Operation.FRAG);
			//pos, getCurrentOperation(),t,tags,pos+i+j,pos+2*i+j
			annot.set(t.get(pos), DisfluencyFactory.createDisfluency(pos, getCurrentOperation(),t,tags,pos+1,pos+1,slot));
			t.remove(pos);
			return pos;
		}
		while( i <=  (t.size()-pos)/2){
			int j = 0;
			///TODO
//			dealing with insertions of stopwords
			do{
				if(match(tags,t,pos,pos+i+j,i)){
					if(insertHasFragment(tags,t,pos+i,j)){
						setCurrentOperation(Operation.FRAG);
					}
					annot.set(t.get(pos), DisfluencyFactory.createDisfluency(pos, getCurrentOperation(),t,tags,pos+i+j,pos+2*i+j,slot));
					//System.out.println(annot.get(t.get(pos)).toString());
					for(int k = pos+i+j-1; k>=pos; k--){				   
						t.remove(k);
					}					
					return pos;
				}
				j++;
				w = tags.get(t.get(pos+i+j-1)).getValue();
			}while((i <= (t.size() - pos - j)/2) && ((stopWords.isStopWord(w))|| (isFragment(w))));
			/*while((i <= (t.size() - pos - j)/2) && (stopWords.isStopWord(w))); // fragments processed in match*/
			if(isFragment(w)){				
				setCurrentOperation(Operation.FRAG);
			}
			i++;   
		}

		return pos + 1;
	}
/*
	private void updateSlot(Slot slot,List<Integer> t,List<AnneTag> tags, List<Disfluency> annot){
		    List<AnneTag> list = new ArrayList<AnneTag>();
            int i = 0;
            for(AnneTag tag:tags){
            	if(!(tag instanceof Word)){
            		list.add(tag);
            	}
            	else{
            		int pos = tag.getPos();
            		if(annot.get(pos) != null){
            			list.add(annot.get(pos));
            		}
            		else{
            			if((i < t.size()) && (t.get(i) == pos)){
            				list.add(tags.get(pos));
            				i++;
            			}
            		}
            	}
            }
            slot.setList(list);
	}
	*/
	
	private int getNextDisfluency(int pos,List<Disfluency> annot){
		while((pos < annot.size()) && (annot.get(pos) == null)){
			pos++;
		}
		if(pos >= annot.size()) return -1;
		return pos;
	}
	
	
	private DisfluencySeq getNextDisfluencySeq(int pos,Slot slot,List<Disfluency> annot){
		//System.err.println(pos);
		int i = getNextDisfluency(pos,annot);
		if(i == -1){
			return null;
		}		
		DisfluencySeq ds = new DisfluencySeq(slot);
		Disfluency current = annot.get(i);
		ds.add(current);
		//System.err.println("FOUND "+current);
		i = current.getLastPos() + 1;
		//System.err.println("I="+i);
		while((i = getNextDisfluency(i, annot)) != -1){			
			Disfluency next = annot.get(i);
			//System.err.println("FOUND2 "+next);
			List<Integer> positions = slot.getInvertedWordPositions();
			//System.err.println(positions);
			//System.err.println("C "+positions.get(current.getLastPos()));
			//System.err.println("N "+positions.get(next.getPos()));
			if(positions.get(current.getLastPos()) + 1 != positions.get(next.getPos())){
				break;
			}			
			ds.add(next);
			//System.err.println("ADD =>"+ds);
			current = next;
			i = current.getLastPos()+1;
		}		
		return ds;
	}
	
	
	private void updateSlot2(Slot slot,List<Disfluency> annot){
       //System.out.println(slot.getTags());
       //System.out.println(slot.getWordPositions());
       //System.out.println(annot);
       //System.out.println(slot.getInvertedWordPositions());
       int i = 0;
      while(i < annot.size()){
    	  DisfluencySeq ds = getNextDisfluencySeq(i, slot, annot);
    	  if(ds == null){
    		  break;
    	  }
    	  slot.addDisfluency(ds);
    	  i = ds.getLastPos() + 1;
      } 
       
}
	
	public void annotate(Slot slot){
		List<Integer> t = slot.getWordPositions();
		List<AnneTag> tags = slot.getTags();
		List<Disfluency> annot = createAnnotationList(tags.size());
		//System.out.println("++++"+slot);
		do{    		
			modified = false;
			int i = 0;
			while(i < t.size()){
				int prev = t.size();
				//System.out.println("pos="+i);
				i = findNextDisfluency(i,t,annot,tags,slot);
				if(prev != t.size()){
					modified = true;
				}
			}
		}while(modified);
		//updateSlot(slot,t,tags,annot);
		updateSlot2(slot,annot);
		//System.out.println(slot.getDisfluencies());
	}

}
