/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Element;

public class DisfluencySeq implements AnneTag,Iterable<AnneTag> {
	final private List<Disfluency> seq = new ArrayList<Disfluency>();
	final private Slot parentSlot;
	
	public DisfluencySeq(Slot parentSlot){
		this.parentSlot = parentSlot;
	}
	
	public Slot getParentSlot() {
		return parentSlot;
	}



	public Operation getType(){
		return seq.get(seq.size() - 1).getType(); //on prend la dernière disfluence?
		//A changer : on prend la dernière reconnue par le processus itératif?
	}
	
	public List<Disfluency> getSeq() {
		return seq;
	}

	public boolean add(Disfluency e) {
		return seq.add(e);
	}

	public int getLastPos() {
		if(seq.isEmpty() ){
			return 0;
		}
		return seq.get(seq.size() - 1).getLastPos();
	}

	public int getRrStartingPos(){
		if(seq.isEmpty() ){
			return 0;
		}		
		Disfluency d = seq.get(seq.size() - 1);		
		if(d.getRr() == null || d.getRr().isEmpty()){
			return getLastPos()+1;
		}
		return d.getRr().get(0).getPos();
	}
	
	public int getRrEndingPos(){
		if(seq.isEmpty() ){
			return 0;
		}
		Disfluency d = seq.get(seq.size() - 1);
		if(d.getRr() == null || d.getRr().isEmpty()){
			return getLastPos();
		}
		return d.getRr().get(d.getRr().size() -1).getLastPos();
	}
	
	public boolean isRepairIncludedIn(DisfluencySeq seq){
		return (getRrStartingPos()>=seq.getRrStartingPos() && getRrEndingPos()<=seq.getRrEndingPos());
	}
	
	public int getPos() {
		if(seq.isEmpty() ){
			return 0;
		}
		return seq.get(0).getPos();
	}

	public String getValue() {
		return null;
	}

	public boolean isUsable() {
		return false;
	}

	public Element toXMLElement(AnneTagXMLWriter tagWriter) {
		return tagWriter.write(this);
	}

	public String write(AnneTagWriter tagWriter) {
		return tagWriter.write(this);
	}

	public String toString(){
		String res="{";
		for(Disfluency d:seq){
			res+=d.toString()+";";
		}
		return res+"}";
	}

	public Iterator<AnneTag> iterator() {
		return new DisfluencySeqIterator(this);
	}
	
	public Iterator<AnneTag> getRrIterator(){
		return new DisfluencyRrIterator(this);
	}
	
}
