/**
 * DisTagger
 * 
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 **/

package fr.upemlv.lgtools.distagger;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import org.jdom.JDOMException;

/**
 * 
 * Main class for Distagger
 * 
 * @author Matthieu Constant
 *
 */

public class DistaggerMain {
	private static AnneConfiguration config;
	public static final String CURRENT_DIR = new File(".").getAbsolutePath();
	
	
	private static void addFileComponent(JPanel panel, int y,JLabel label,JTextField textfield,JButton button,JButton clear){
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(5,5,5,5);
		c.gridy = y;
		c.gridwidth = 2;
		c.gridx = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		//c.anchor = GridBagConstraints.WEST;
		
		panel.add(label,c);
		c.gridy = y + 1;
		c.gridwidth = 1;
		//c.anchor = GridBagConstraints.CENTER;
		panel.add(textfield,c);
		c.fill = GridBagConstraints.NONE;
		if(clear != null){
			c.gridx = 1;
			c.anchor = GridBagConstraints.WEST;
			panel.add(clear,c);
		}
		c.gridx = 2;
		c.gridwidth = 1;
		
	    c.anchor = GridBagConstraints.EAST;
		panel.add(button,c);
		
	}
	
	private static void createAndAddFileComponent(int y,final JPanel p,final GUIOperations op,String label){
		createAndAddFileComponent(y, p, op, label,false);
	}
	
  private static void createAndAddFileComponent(int y,final JPanel p,final GUIOperations op,String label,final boolean dir){
	  JButton b = new JButton("...");
	  JButton clear = null;
	  final JTextField tf = new JTextField();
	  tf.setEnabled(false);
	  if(dir){
		clear = new JButton("Clear");  
	  }
	  
	  config.addConfigListener(new ConfigListener(){

		  private String getFilename(File f){
			  if(f == null) return "";
			  return f.getAbsolutePath();
		  }
		  
		  private void update(){			
			  tf.setText(getFilename(op.getFile(config)));  
		  }
		  
		public void configChanged() {
				update();		
		}

		public void configCleaned() {
			tf.setText("");
		}

		public void configLoaded() {
			update();
		}

		public void configSaved() {
		}
		  
	  });
	b.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser(CURRENT_DIR);
				if(config.getConfig() != null){
					fc.setCurrentDirectory(config.getConfig().getParentFile());
				}
				if(dir){
					fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				}
				else{
					fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
				}				
				int returnVal = fc.showOpenDialog(p);
				if(returnVal == JFileChooser.APPROVE_OPTION){
				    File f = fc.getSelectedFile();				
				   if(f == null) return;
				   op.setFile(config, f);
				}
			}
			
		});
	if(clear != null){
	    clear.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				DistaggerMain.config.setOutDir(null);
				
			}
	    	
	    });
	}
		addFileComponent(p, y, new JLabel(label), tf, b,clear);
       
  }
  	
	
	
	public static JPanel createMainPanel(){
		final JPanel p = new JPanel();
		int Y = 0;
		p.setLayout(new GridBagLayout());
		Dimension d = new Dimension(150,200);
							   
		createAndAddFileComponent(0,p,GUIOperations.EUH,"Euh class:");
		createAndAddFileComponent(2,p,GUIOperations.CLASSES,"Equivalence classes for self-correction:");
		createAndAddFileComponent(4,p,GUIOperations.INSERTS,"Inserts in disfluencies:");
		createAndAddFileComponent(6,p,GUIOperations.META,"Meta annotation:");
		createAndAddFileComponent(8,p,GUIOperations.OUT,"Output directory:",true);
		Y=10;
				
		final AnneListAdapter model = new AnneListAdapter(config);			
		final JList list = new JList(model);
		list.setCellRenderer(new DefaultListCellRenderer(){

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				File f = (File)value;
				setText(f.getName() + " ["+f.getParent()+"]");
				return this;
			}
			
			
		});
		JScrollPane scp = new JScrollPane(list);
		//config.add(new File("/home/mat/research/experiments/"));
		scp.setMinimumSize(d);
		JButton add = new JButton("Add...");
		add.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser(CURRENT_DIR);
				int returnVal = fc.showOpenDialog(p);
				if(returnVal == JFileChooser.APPROVE_OPTION){
				    File f = fc.getSelectedFile();
				    if(f == null) return;
				    config.add(f);
				}
			}

		});
		JButton remove = new JButton("Remove");
		remove.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {

			   if(list.getSelectedIndices().length == 0) return;
			   int[] tab = list.getSelectedIndices();
			   for(int i = tab.length - 1 ; i >= 0  ; i--){
				       config.remove(tab[i]);
			   }
			}
			
		});
		
		
		JButton preprocess = new JButton("Preprocess");
		preprocess.addActionListener(new ActionListener(){
			private boolean areValidArguments(){
				if(config.getClasses() == null){
					return false;
				}
				if(config.getEuh() == null){
					return false;
				}
				if(config.getInserts() == null){
					return false;
				}
				if(config.getMeta() == null){
					return false;
				}
				return true;
			}
			
			private String getOutputFileName(File f,String suffix){				
				File pref = config.getOutDir();		
				String s = "";
				if(pref == null){
					s = f.getAbsolutePath(); 
				}
				else{
					s = pref.getAbsolutePath()+"/"+f.getName();			
				}
				int i = s.lastIndexOf('.');
				if(i != - 1){
					s = s.substring(0,i);
				}
				return s+ suffix;
			}
			
			public void actionPerformed(ActionEvent e) {
				Preprocessor preproc = new Preprocessor(config);
				if(! areValidArguments()){
					JOptionPane.showMessageDialog(p, "Missing classes, insertions or/and euh files!");
					return;
				}				
				for(File f:config.getTexts()){
					try{						
				       preproc.preprocess(f);
				       preproc.write(getOutputFileName(f, "_norm"));
				       }
					catch(IOException ex){
						JOptionPane.showMessageDialog(p, ex.getMessage());
					}
					catch(AnneException ex){
						JOptionPane.showMessageDialog(p, ex.getMessage());
					} catch (JDOMException ex) {
						JOptionPane.showMessageDialog(p, ex.getMessage());
					}
				}
			}
			
		});

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(5,5,5,5);
		c.gridx = 0;
		c.gridy = Y;
		c.gridheight = 4;
		c.gridwidth = 1;
		p.add(scp,c);		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridheight = 1;
		c.gridx = 1;		
		c.gridwidth = 2;
		p.add(add,c);
		
		c.gridy = Y+1;
		p.add(remove,c);
		
		c.gridy = Y+3;
        c.anchor = GridBagConstraints.SOUTH;		
		p.add(preprocess,c);
		return p;		
	}

	
	
	
	private static JMenuBar createMenu(final JFrame frame){
		JMenuBar jmb = new JMenuBar();
		
		JMenu config = new JMenu("Configuration");
		JMenuItem newConfig = new JMenuItem("New");
		newConfig.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				DistaggerMain.config.clear();
			}
			
		});
		JMenuItem open = new JMenuItem("Open...");
		open.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser(CURRENT_DIR);
				fc.showOpenDialog(frame);
				File f = fc.getSelectedFile();
				if(f == null) return;
				try{
				DistaggerMain.config.load(f.getAbsolutePath());
				}
				catch(Exception ex){
					JOptionPane.showMessageDialog(frame, ex.getMessage());
				}
			}			
		});
		JMenuItem save = new JMenuItem("Save");
		save.addActionListener(new SaveActionListener(DistaggerMain.config,false,frame));
		JMenuItem saveAs = new JMenuItem("SaveAs...");
		saveAs.addActionListener(new SaveActionListener(DistaggerMain.config,true,frame));
		config.add(newConfig);
		config.add(open);
		config.add(save);
		config.add(saveAs);
		jmb.add(config);
		
		JMenu help = new JMenu("Help");
		help.setEnabled(false);
		jmb.add(help);
		
		return jmb;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		final JFrame frame = new JFrame("New");
		config = new AnneConfiguration();
		//config.add(new File("/home/mat/data/texts/french/oral/blaNB1l-brut.txt"));		
		config.addConfigListener(new TitleConfigListener(frame,config));
		frame.setJMenuBar(createMenu(frame));
		frame.setContentPane(createMainPanel());
		frame.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e) {
				if(!config.isToBeSaved()) return;
				int answer = JOptionPane.showConfirmDialog(frame, "Do you want to save configuration before closing?","Saving confirmation",JOptionPane.YES_NO_OPTION);	
				if(answer == 0){
					new SaveActionListener(config,false,frame).actionPerformed(null);
				}
			}			
		});
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
		frame.pack();
	}

}
