/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class OverlappingExtractor {
	private static Pattern speaker = Pattern.compile("^ *([a-zA-Z0-9_]+)");
	private String overlappingStart = "\\|-";
	private String overlappingEnd = "-\\|";
	private String speakerDelimiter = "<[^>]*>";
	private String outsideSpeakerDelimiter = "(^[^<]*<)|(>[^<]*<)|(>[^<]*$)";
	private int previousSlot = -1;
	private int currentSlot = 0;
	private boolean inMultiLineOverlapping = false;
	//private String overlappingEndDelimiter = " {1} ";
	
	
	private String getBackRefDelimiter(int id){
		return " @01#"+id+"@ ";
	}
	
	private String getUniqueRefDelimiter(int id,boolean internal){
		char c = internal?'0':'1';
		return " @00"+c+"#"+id+"@ ";
	}
	
	private String getRefDelimiter(int start, int end,boolean internal){
		char c = internal?'0':'1';
		String res = "@00"+c;//";//+id+"}";
		for(int i = start ; i <= end ; i++){
			res += "#"+( i + 1);			
		}		
		return res + "@";
	}
	
	private String getEndDelimiter(String s){
		//return overlappingEndDelimiter;
		return "@1"+s+"@";
	}
	
	private String getBackRefOverlapping(String over,int id,String val){
		return getBackRefDelimiter(id)+over+getEndDelimiter(val);
	}
	
	private String getRefOverlapping(String over,int start,int end,boolean internal){
		return getRefDelimiter(start,end,internal)+over+getEndDelimiter("-|");
	}
	
	private String getUniqueRefOverlapping(String over,int id,boolean internal){
		return getUniqueRefDelimiter(id,internal)+over+getEndDelimiter("");
	}
	
	
	private String getSlotSpeaker(String line){
		Matcher m = speaker.matcher(line);
		if(m.find()){
			return m.group(1);
		}
		return null;
	}
	

	private void testError(int n) throws AnneException{
		if(n > 2){
			throw new AnneException("Too many terminating overlapping segment markers");
		}
	}
	
	private Slot createSpeakingSlot(String id,Transcription transcription){
		return new Slot(id,transcription);
	}
	
	/*
	private String extractOverlapping(String s,Transcription transcription,List<String> currentOverlappings) throws AnneException{
		return extractOverlapping(s, false,transcription,currentOverlappings);
	}
	
	
	
	private String extractOverlapping(String s,boolean type,Transcription transcription,List<String> currentOverlappings) throws AnneException{
		String[] fragments = s.split(speakerDelimiter);
		String[] speakers = s.split(outsideSpeakerDelimiter);
		int nf = fragments.length;
		int ns = speakers.length;		
		if(ns != nf){
			throw new AnneException("Problem in the overlapping fragment formatting");
		}		
		if(nf == 1){
			if(type != inMultiLineOverlapping){
				throw new AnneException("Incoherent overlapping fragment segmentation! type="+type+"!="+inMultiLineOverlapping);
			}
			if(inMultiLineOverlapping){
				inMultiLineOverlapping = false;
			}
			else{
				inMultiLineOverlapping = true;
			}
			//printOverlapping(fragments[0]);			
			return getSlotDelimiter(true,!inMultiLineOverlapping)+" "+fragments[0]+" "+getSlotDelimiter(false,!inMultiLineOverlapping);
		}		
		for(int i=0 ; i < fragments.length ; i++){
			if(i != 0){
				Slot slot = createSpeakingSlot(speakers[i]);
		    	//processSlot(fragments[i], sl);
		    	currentOverlappings.add("==="+fragments[i]);
			}
			//System.out.print("++"+fragments[i]);
			//System.out.println("=>"+speakers[i]);
		}
		return getSlotDelimiter(true,false)+fragments[0]+getSlotDelimiter(false,false);
	
	}
	*/
	
	private String extractInMiddleOverlappings(String s,Transcription transcription,List<String> currentOverlappings) throws AnneException{
		String[] fragments = s.split(speakerDelimiter);
		String[] speakers = s.split(outsideSpeakerDelimiter);
		int nf = fragments.length;
		int ns = speakers.length;
		if(nf != ns){
			throw new AnneException("Format problem");
		}
		if(nf <= 1 ){
			throw new AnneException("In middle Overlapping should not be empty!");
		}
		String res=fragments[0];
		int current = currentOverlappings.size();
		for(int i=1 ; i < fragments.length ; i++){
				Slot slot = createSpeakingSlot(speakers[i],transcription);
		    	currentOverlappings.add(getBackRefOverlapping(fragments[i],currentSlot,""));
		    	transcription.add(slot);
		}
		
		return getRefOverlapping(res, currentSlot+current,currentSlot+currentOverlappings.size() - 1,true);
	}
	
	
	
	private String extractInMultilineBackRefOverlapping(String s) throws AnneException{
		if(!inMultiLineOverlapping){
			throw new AnneException("Incoherent overlapping fragment segmentation: not in a multiline overlapping");
		}    		
		String res = getBackRefOverlapping(s,previousSlot,"-|");
		inMultiLineOverlapping = false;
		return res;
	}
	
	private String extractInMultilineRefOverlapping(String s,int nextSlot) throws AnneException{
		if(inMultiLineOverlapping){
			throw new AnneException("Incoherent overlapping fragment segmentation: already in a multiline overlapping");
		}    		
		String res = getUniqueRefOverlapping(s,nextSlot,false);
		inMultiLineOverlapping = true;
		return res;
	}
	
	
	private String processFirstPart(String s) throws AnneException{
    	String[] tab = s.split(overlappingEnd);
    	String res="";
    	int n = tab.length;
    	testError(n);
    	if(n == 1){
    	if(s.endsWith("-|")){
    		res = extractInMultilineBackRefOverlapping(tab[0]);
    		}
    		else{
    		    res = tab[0];
    		}
    	}
    	if(n == 2){
    		res = extractInMultilineBackRefOverlapping(tab[0]);    		
    		res += tab[1];
    	}
    	return res;
    }
	
	private String processPartInMiddle(String s,Transcription transcription,List<String> currentOverlappings)throws AnneException{
		if(inMultiLineOverlapping) throw new AnneException("Already in an overlapping sgement!\n-->"+s);
		String[] tab = s.split(overlappingEnd);
		String res="";
		int n = tab.length;
    	testError(n);
    	if(n == 1){
    		throw new AnneException("Overlapping segment in middle should be terminated");
    	}
    	if(n == 2){
    		res = extractInMiddleOverlappings(tab[0],transcription,currentOverlappings);
    		res += tab[1];
    	}
    	return res;
	}
	
	
	private String processFinalPart(String s, Transcription transcription,List<String> currentOverlappings) throws AnneException{
		String[] tab = s.split(overlappingEnd);
		int n = tab.length;
		String res ="";
    	testError(n);
    	if(n == 1){  
    		if(!s.endsWith("-|")){
    		   res = extractInMultilineRefOverlapping(tab[0],currentSlot+currentOverlappings.size() + 1);
    		}
    		else{
    			res= extractInMiddleOverlappings(tab[0],transcription,currentOverlappings);
    		}
    	}
    	if(n == 2){
    		res = extractInMiddleOverlappings(tab[0],transcription,currentOverlappings);
    		res += tab[1];
    	}
    	return res;
	}
	
	
	
	private List<String> extractOverlappings(String line,Transcription transcription) throws AnneException{
		List<String> currentOverlappings = new ArrayList<String>();
	    String[] tab = line.split(overlappingStart);
	    String res = "";	 
	    if(tab.length == 0) return currentOverlappings;
	    //System.out.println(line);
	    try{
	    res += processFirstPart(tab[0]);
	    for(int i = 1 ; i < tab.length - 1 ; i++){
	    	res += processPartInMiddle(tab[i],transcription,currentOverlappings);
	    }
	    if(tab.length > 1){
	    	res += processFinalPart(tab[tab.length - 1],transcription,currentOverlappings);
	    }	    
	    currentOverlappings.add(0,res);
	    }
	    catch(AnneException e){
	    	throw new AnneException(e.getMessage() + "\n"+ line);
	    }
	    return currentOverlappings;
	}
	
	
	
	
	public List<String> extract(String line, Transcription transcription)throws AnneException{
		Slot mainSlot = createSpeakingSlot(getSlotSpeaker(line),transcription);		
		transcription.add(mainSlot);
		//System.out.println(line);
		//System.out.println(mainSlot);
		int i = mainSlot.getSpeaker().length();
		List<String> slots = extractOverlappings(line.substring(i + 1),transcription);
		//slots.add(line);
		
		previousSlot = currentSlot;
		currentSlot = currentSlot + slots.size(); 
		return slots;
	}
	
}
