/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */
package fr.upemlv.lgtools.distagger;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jdom.JDOMException;

public class Preprocessor {
	private static Pattern transcriptionBegin = Pattern.compile("<deb id=\"([^\"]+)\">");
	private static Pattern transcriptionEnd = Pattern.compile("<fin id=\"([^\"]+)\">");
	public static String TRANSCRIBER_EXTENSION = "trs";
	private AnneTranscriptions currentTranscriptions;
	private AnneConfiguration config;
	
	public Preprocessor(AnneConfiguration config){
		this.config = config;
	}
	
	private String transcriptionStarting(String s){
		Matcher m = transcriptionBegin.matcher(s);
		if(m.find()){
			return m.group(1);
		}
		return null;
	}
	
	private String transcriptionEnding(String s){
		Matcher m = transcriptionEnd.matcher(s);
		if(m.find()){
			return m.group(1);
		}
		return null;
	}
	
	public void preprocess(File f) throws IOException,AnneException, JDOMException{
		String suf="";
		if(f.getCanonicalPath().endsWith(TRANSCRIBER_EXTENSION)){
		    TranscriptionFormatter.transcriberToValibel(f.getAbsolutePath());
			//System.exit(0);
		    //return;
		    suf = ".val";
		}
		String filename = f.getAbsolutePath()+suf;
		System.out.println("Preprocessing "+filename+"...");
		BufferedReader input = new BufferedReader(new InputStreamReader(new FileInputStream(filename),"UTF-8"));		
		currentTranscriptions = new AnneTranscriptions();
		OverlappingExtractor extractor = new OverlappingExtractor();
		StopWords stopWords = new StopWords(config.getInserts().getAbsolutePath());
		WordClasses wordClasses = new WordClasses(config.getClasses().getAbsolutePath());
		MetaAnnotation metaAnnotation = new MetaAnnotation(config.getMeta().getAbsolutePath());
		EuhClass euhClass = new EuhClass(config.getEuh().getAbsolutePath());
		DisfluencyMatcher disMatcher = new DisfluencyMatcher(stopWords,wordClasses,euhClass);
		SlotTokenizer tokenizer = new SlotTokenizer(metaAnnotation);
		boolean inTrans = false;
		String line;
		int i = 0;
		int wordCount = 0;
		while((line = input.readLine()) != null){
			if(line.isEmpty()){
				throw new AnneException("There is an empty turn!");				
			}
			String s = transcriptionStarting(line);
			if(s == null){
				String s2 = transcriptionEnding(line);
				if(s2 == null){
					if(!inTrans){ 
						throw new AnneException("The transcription did not start with appropriate tag");
					}
					Transcription currentTranscription = currentTranscriptions.getCurrentTranscription();
					List<String> slots = extractor.extract(line,currentTranscription);
					//System.err.println("***");
					for(String sl:slots){
						//System.err.println(sl);
						//System.err.println(i);
						//System.err.println(currentTranscription.size());
					   Slot slot = currentTranscription.getSlot(i);
					   slot.setId(i);
					   tokenizer.tokenize(sl,slot);
					  // System.out.println(""+i+": "+slot);
					  disMatcher.annotate(slot);					   
					   //System.out.println(""+i+": "+slot);
					   i++;
					   wordCount += slot.getWordPositions().size();
					}
				}
				else{
					inTrans = false;
					i=0;
				}
			}
			else{
				if(inTrans){
					throw new AnneException("A transcription cannot start before another one stops");
				}
				currentTranscriptions.add(new Transcription(s));
				inTrans = true;
			}
		}		
		input.close();
		//System.out.println(stopWords.isStopWord("oui"));
		System.out.println("trabscription word count:"+wordCount);
	}	
	
	
	/**
	 * @param filename
	 */
	public void write(String filename){
		if(filename.endsWith(TRANSCRIBER_EXTENSION)){
			filename += ".val";
		}
		/*System.out.println("Writing in "+filename+".xml ...");
		TranscriptionsXMLWriter writer = new TranscriptionsXMLWriter(currentTranscriptions);
		writer.write(filename+".xml");
		System.out.println("Filtering and writing in "+filename+".txt ...");
		TranscriptionsFilter filter = new TranscriptionsFilter(currentTranscriptions);
		filter.write(filename+".txt");
		*/
		System.err.println("Annotating and writing in "+filename+".snt ...");
		TranscriptionsFilterForElag elagFilter = new TranscriptionsFilterForElag(currentTranscriptions);
		elagFilter.write(filename+".snt");
		
		/*System.out.println("Annotating and writing in "+filename+".rha ...");
		RhapsodyTranscriptionFilter rhapFilter = new RhapsodyTranscriptionFilter(currentTranscriptions);
		rhapFilter.write(filename+".rha");*/
		System.err.println("Annotating and writing in "+filename+".txt ...");
		SimpleTranscriptionWriter simpleWriter = new SimpleTranscriptionWriter(currentTranscriptions);		
		simpleWriter.write(filename+".txt");
		System.err.println("Done.");
	}
	
	
}
