/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */
package fr.upemlv.lgtools.distagger;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;

public class RefOverlapping implements AnneTag {
	private List<Integer> refs = new ArrayList<Integer>();
	private int pos;
	private boolean internal;
	
	public RefOverlapping(int pos,boolean internal){
		this.pos = pos;
		this.internal = internal;
	}
	
	public boolean addRef(Integer e) {
		return refs.add(e);
	}
	  
	public List<Integer> getRefs(){
		return refs;
	}
	
	public String toString(){
		String s =  "[over ref";
		for(Integer ref:refs){
			s+=":"+ref;
		}		
		return s + "]";
	}


	public boolean isUsable() {
		return false;
	}


	public String getValue() {
		return "|-";
	}


	public int getPos(){
		return pos;
	}
	
	public boolean isInternal(){
		return internal;
	}
	
	public Element toXMLElement(AnneTagXMLWriter tagWriter) {
		return tagWriter.write(this);
	}
   
	public String write(AnneTagWriter tagWriter) {
		return tagWriter.write(this);
	}

	public int getLastPos() {
		return pos;
	}
	
}
