/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.util.Iterator;

public class RhapsodieTranscriptionIterator implements Iterator<Slot> {
   final private Transcription transcription;
   private int count=0;
	
	public RhapsodieTranscriptionIterator(Transcription t){
	  this.transcription = t;   
   }
	
	public boolean hasNext() {
		return count < transcription.size();
	}
	private  int getNextCount(Slot slot){
		int max = -1;
		slot.setIterator(new DefaultSlotIterator(slot));
		for(AnneTag t:slot){
			if(t instanceof RefOverlapping){
				RefOverlapping over = (RefOverlapping)t;
				if(!over.isInternal()){
					for(Integer ref:over.getRefs()){
						max = (ref>max?ref:max);
					}
				}
			}
		}
		return (max!=-1?max:slot.getId())+1;
	}
    
	public Slot next() {
		Slot slot = transcription.getSlot(count);
		count = getNextCount(slot);
		return slot;
	}

	public void remove() {
	
	}

}
