/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.util.Iterator;
import java.util.List;

public class RhapsodyAnneTagWriter implements AnneTagWriter{
   private RhapsodyInsideDisfluencyTagWriter insideTagWriter;
   private List<Integer> inInternalOverlapping=null;
   private Transcription current;
   
   public List<Integer> getInInternalOverlapping(){
	   return inInternalOverlapping;
   }
   
   public void setInInternalOverlapping(List<Integer> list){
	   this.inInternalOverlapping = list;
   }
   
   public String write(Word w) {
	   //System.err.println(w+" "+inInternalOverlapping);
		return  " "+w.getForm().replace('/', '~');
	}

	public String write(Meta m) {
		return null;
	}
   
	
	
	public void setTranscription(Transcription t){
		current = t;
	}
	
	public static  String getSimpleDisfluencyValue(Disfluency dis,AnneTagWriter writer){
		String s="";
		for(AnneTag t:dis){
			s+= t.write(writer);
		}
		return s.substring(1, s.length());
	}
	
	public String write(Disfluency dis) {
		switch(dis.getType()){
		case EUH:
			return "\""+getSimpleDisfluencyValue(dis,this)+"\"";
		case FRAG:
			//System.err.println("----------");
			//System.err.println(dis.getRm());
			//System.err.println(dis.getRr());
			return getSimpleDisfluencyValue(dis,this);
		default:
			String res = "";
			for(AnneTag t:dis){
				String s=t.write(this);
				if(s != null){
					res+=s;
				}
			}
			return res.substring(1, res.length())+"|"; 
		}
	}

	public String write(EndOverlapping eover) {
	if(eover.getOrigin().equals("")){
			return "";
	}		
		//return null;
		String s = "";		
		//System.err.println("CCCCCCCCCCCCCCCCCCCCCCCCC");
		if(getInInternalOverlapping() != null){
			//System.err.println("CCCCCCCCCCCCCCCCCCCCCCCCC");
			s += "";
			//System.out.println(eover);
			for(int i:getInInternalOverlapping()){
				//System.err.println("ICI "+i);				
				Slot slot = current.getSlot(i);
			//	System.out.println(slot);
				s+="\n($"+slot.getSpeaker();
				slot.setIterator(new SlotIteratorWithDisfluency(slot,DisfluencyLimit.ENTIRE)); 
				for(AnneTag t:slot){
					//System.out.println(t);
					if(t instanceof EndOverlapping){
						break;
					}
					String tmp = t.write(this);
					if(tmp != null){
						s+= " "+tmp;
					}
				}
				s+=")";
			}			
		}
		s += "\n-$\n";
		setInInternalOverlapping(null);
		return s;
	}

	public String write(BackRefOverlapping brover) {
		return null;
	}

	public String write(RefOverlapping rover) {
		setInInternalOverlapping(null);
		if(!rover.isInternal()){
			setInInternalOverlapping(rover.getRefs());
			//System.err.println(inInternalOverlapping);
		}
		return "\n$-\n";
	}

	public String write(DisfluencySeq ds) {
		List<Disfluency> seq = ds.getSeq();
//		TRAITER LES EUH ET FRAG NON COMPLETE
		//if(seq.size() == 1 && seq.get(0).getRr() == null){
		if(ds.getType() == Operation.EUH || ds.getType() == Operation.FRAG){
			return " "+seq.get(0).write(this);			
		}
		
		
		
		String s = " {";
		//System.err.println("============");
		insideTagWriter = new RhapsodyInsideDisfluencyTagWriter(this,ds);
		for(Disfluency d:ds.getSeq()){	
			//System.err.println("###"+d);
			s+=d.write(insideTagWriter);
		}
		Iterator<AnneTag> it = ds.getRrIterator();
		String rr = "";
		while(it.hasNext()){			
			AnneTag t = it.next();
			//System.err.println("%%%"+t);
			rr += t.write(insideTagWriter);
		}
		if(rr.length() > 0){
		    rr = rr.substring(1, rr.length());
		}
		
		s += rr+"}";
		//System.err.println("************");
		return s;
	}

}
