/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.util.List;

public class RhapsodyInsideDisfluencyTagWriter extends RhapsodyAnneTagWriter  {
    private final RhapsodyAnneTagWriter tagWriter;
    //private List<Integer> inInternalOverlapping=null;
    private final DisfluencySeq disfluency;
	
    public RhapsodyInsideDisfluencyTagWriter(RhapsodyAnneTagWriter tagWriter,DisfluencySeq disfluency){
		super();
    	this.tagWriter = tagWriter;
		this.disfluency = disfluency;
	}
	
    public void setInInternalOverlapping(List<Integer> list){
    	tagWriter.setInInternalOverlapping(list);
    }
    
    public List<Integer> getInInternalOverlapping(){
 	   return tagWriter.getInInternalOverlapping();
    }    

    @Override
	public String write(EndOverlapping eover) {
		/*if(eover.getOrigin().equals("")){
			return "";
	   }*/
		Transcription current = disfluency.getParentSlot().getParentTranscription();
		String s = "";		
		//System.err.println("CCCCCCCCCCCCCCCCCCCCCCCCC");
		if(getInInternalOverlapping() != null){
			//System.err.println("CCCCCCCCCCCCCCCCCCCCCCCCC");
			s += "|} ";
			for(int i:getInInternalOverlapping()){
				//System.err.println("ICI "+i);				
				Slot slot = current.getSlot(i);
				s+="\n($"+slot.getSpeaker();
				slot.setIterator(new SlotIteratorWithDisfluency(slot,DisfluencyLimit.ENTIRE)); 
				for(AnneTag t:slot){
					String tmp = t.write(tagWriter);
					if(tmp != null){
						s+= " "+tmp;
					}
				}
				s+=")";				
			}			
		}	
		//System.err.println("CCCCCCCCCCCCCCCCCCCCCCCCC");
		s += "\n-$\n{";
		setInInternalOverlapping(null);
		return s;
	}
   
	@Override
	public String write(BackRefOverlapping brover) {
		return null;
	}

	@Override
	public String write(RefOverlapping rover) {
		setInInternalOverlapping(null);
		if(!rover.isInternal()){
			setInInternalOverlapping(rover.getRefs());
		}
		return "\n$-\n";
	}
}
