/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;

public class RhapsodyTranscriptionFilter {
	private AnneTranscriptions transcriptions;
	
	private RhapsodyAnneTagWriter tagWriter = new RhapsodyAnneTagWriter();
	public RhapsodyTranscriptionFilter(AnneTranscriptions transcriptions){
		this.transcriptions = transcriptions;
	}
	
	private void writeSlot(Slot slot,Writer writer) throws IOException{
		writer.write("$"+slot.getSpeaker());
		slot.setIterator(new SlotIteratorWithDisfluency(slot,DisfluencyLimit.ENTIRE));
		System.err.println(slot);
		for(AnneTag t:slot){
			//System.err.println(t);			
			String s = t.write(tagWriter);
			if(s != null){
				writer.write(s);
			}
		}
		writer.write("\n");
	}
	

	private void writeTranscription(Transcription t,Writer writer) throws IOException{
		tagWriter.setTranscription(t);
		//t.setIterator(new DefaultTranscriptionIterator(t));
		t.setIterator(new RhapsodieTranscriptionIterator(t));
		for(Slot slot:t){
			//System.err.println(slot.getId());
			writeSlot(slot,writer);			
		}
	}
	
	
	public void write(String filename){
		try{
			//System.err.println("HERE");
			
			Writer writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),"UTF-8"));			
			for(Transcription t:transcriptions.getTranscriptions()){            	
				writeTranscription(t,writer);            	
			}
			writer.close();
		}
		catch(IOException e){
			System.err.println(e);
			System.exit(1);
		}
	}

}
