/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


public class Slot implements Iterable<AnneTag>{

	  private List<AnneTag>tags = new ArrayList<AnneTag>();
	  private final List<DisfluencySeq> disfluencies = new ArrayList<DisfluencySeq>();
	  private String speaker;
	  private int id;
	  private Iterator<AnneTag> iterator = new DefaultSlotIterator(this);
	  final private Transcription parent;
	  
	public Slot(String speaker,Transcription parent) {
		super();
		this.speaker = speaker;
		this.parent = parent;
	}

	public Transcription getParentTranscription(){
		return this.parent;
	}
	
	public String getSpeaker() {
		return speaker;
	}

	public void setId(int id){
		this.id = id;
	}
	
	public int getId(){
		return this.id;
	}
	
	public List<AnneTag> getTags() {
		return tags;
	}

   public List<Integer> getInvertedWordPositions(){
	   List<Integer> res = new ArrayList<Integer>();
	   for(int i= 0 ;  i < getTags().size(); i++){
		   res.add(-1);
	   }
	   int cnt = 0;
	   for(int pos:getWordPositions()){
		   res.set(pos, cnt);
		   cnt++;
	   }
	   return res;
   }
	
	public List<Integer> getWordPositions() {
		List<Integer> res = new ArrayList<Integer>(); 
		for(AnneTag t:getTags()){
			if(t.isUsable()){
				res.add(t.getPos());
			}
		}
		return res;
	}
	
	public void setList(List<AnneTag> tags){
		this.tags = tags;
	}
	
	public int size(){
		return tags.size();
	}
		
	public AnneTag get(int index){
		return tags.get(index);
	}
	
	public boolean add(AnneTag e) {
		return tags.add(e);
	}
	  
	
	public List<DisfluencySeq> getDisfluencies() {
		return disfluencies;
	}
	
	public List<DisfluencySeq> getInvertedDisfluencies() {
		List<DisfluencySeq> list = new ArrayList<DisfluencySeq>();
		for(int i = 0 ; i < tags.size() ; i++){
			list.add(null);
		}
		for(DisfluencySeq d:getDisfluencies()){
			list.set(d.getPos(),d);
		}
		return list;
	}

	public boolean addDisfluency(DisfluencySeq d){
		return disfluencies.add(d);
	}
	
	public String toString(){
		String res = getSpeaker()+">";
		for(AnneTag t:getTags()){
		    res +=t;
		}
		return res;
	}

	public void setIterator(Iterator<AnneTag> iterator){
		this.iterator = iterator;
	}
	
	public Iterator<AnneTag> iterator() {
		return iterator;
	}
	 
}
