/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class SlotTokenizer {
	private Pattern tokenPattern = Pattern.compile("(\\([^)]+\\))|(@[^@]+@)|(\\{[^}]+\\})|([^ @{(]+)");
	private final  MetaAnnotation metaAnnotation;
	
	public SlotTokenizer(MetaAnnotation metaAnnotation){
		this.metaAnnotation = metaAnnotation;
	}
	
	private Word createWord(String w,int pos){		
		return new Word(w,pos);
	}
	
	private Meta createMeta(String form,String type,int pos){
		return new Meta(form,type,pos);
	}
	
	private AnneTag createOverlapping(String content, int pos){
		//System.err.println(content);
		char start = content.charAt(1);
		if(start == '1'){
			String origin = "";			
			if(content.length() > 3){
				origin = content.substring(2,content.length() - 1);
			}
			return new EndOverlapping(pos,origin);
		}
		start = content.charAt(2);
		if(start == '1'){ 			
			return new BackRefOverlapping(Integer.parseInt(content.substring(4, content.length()-1)),pos);
		}
		String[] ids = content.substring(1, content.length() - 1).split("#");
		boolean internal = true;
		if(content.charAt(3) == '0'){
			internal = false;
		}
		RefOverlapping o = new RefOverlapping(pos,internal);
		
		for(int i = 1 ; i < ids.length ; i++){
			o.addRef(Integer.parseInt(ids[i]));
		}
		return o;
	}
	
	
	public void tokenize(String text, Slot slot){
		//System.err.println(text);
		text = text.replaceAll("'", "' ");
		text = text.replaceAll("-", " - ");
		text = text.replaceAll("  ", " ");		
		Matcher m = tokenPattern.matcher(text);
		int pos = 0;
		while(m.find()){
			String g1 = m.group(1);
			String g2 = m.group(2);
			String g3 = m.group(3);
			String g4 = m.group(4);
			if(g1 != null && !g1.equals("")){
				slot.add(createMeta(metaAnnotation.getAnnotation(g1),g1,pos));	
			}
			else{
				if(g2 != null && ! g2.equals("")){
					slot.add(createOverlapping( g2,pos));
				}
				else{
					if(g3 != null && ! g3.equals("")){						
						String s = g3;
						s = s.substring(1,s.length()-1);
						String tab[] = s.split(",");
						g4=tab[0];
					}
				    if(g4.equals("/") || g4.equals("//")||g4.equals("?")){
						slot.add(createMeta(metaAnnotation.getAnnotation(g4),g4,pos));
					}
					else{
					   slot.add(createWord(g4,pos));
					}
				}
			}
			pos++;
		}		
	}
	
}
