/**
 * 
 */
package fr.upemlv.lgtools.distagger;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * @author Matthieu Constant
 *
 */
public class TranscriberTurnInfo {
		private final List<String> speakers;
		private final Map<String,String> transcripts;
		
		public TranscriberTurnInfo(List<String> speakers){
			this.speakers = new ArrayList<String>(speakers);
			this.transcripts = new HashMap<String, String>();
			for(String name:this.speakers){
				this.transcripts.put(name,"");
			}
		}
		
		public TranscriberTurnInfo(){
			this.speakers = new ArrayList<String>();
			this.transcripts = new HashMap<String, String>();			
		}
		
		
		public TranscriberTurnInfo(String speakers){
			this.speakers = new ArrayList<String>(Arrays.asList(speakers.split(" ")));
			this.transcripts = new HashMap<String, String>();
			for(String name:this.speakers){
				this.transcripts.put(name,"");
			}
		}
		
		
		public List<String> getSpeakers(){
			return speakers;
		}
		
		public void addText(int nSpeaker,String text){
			transcripts.put(speakers.get(nSpeaker),  (transcripts.get(speakers.get(nSpeaker)) + " "+text).trim().replaceAll("- ","/ "));
		}
		
		public void addText(String speaker,String text){
			transcripts.put(speaker,  (transcripts.get(speaker) + " "+text.trim()));
		}
		
		
		
		public void clean(){
			List<String> toRemove = new LinkedList<String>();
			for(int i = 0 ; i < speakers.size() ; i++){
				String speaker = speakers.get(i);
				String tmp = transcripts.get(speaker);
				if(tmp.isEmpty()){
					toRemove.add(speaker);
				}
			}
			for(String s: toRemove){
			     speakers.remove(s);
			     transcripts.remove(s);
			}
		}
		
		
		@Override
		public String toString(){
			String res = "";
			for(int i = 0 ; i < speakers.size() ; i++){
				String tmp = transcripts.get(speakers.get(i));				
				res += speakers.get(i)+" "+tmp.trim()+"\n";			
			}
			return res;
		}
		
		public String toString(String speaker){
			String res = "";
			String tmp = transcripts.get(speaker);				
			res += speaker+" "+tmp.trim()+"\n";						
			return res;
		}
		
		
		
		public boolean contains(String speaker){
			return transcripts.containsKey(speaker);
		}
		
		public String getText(String speaker){
			return transcripts.get(speaker).trim();
		}
		
		public String getFirstSpeaker(){
			if(speakers.isEmpty()){ 
				return null;
			}
			return speakers.get(0);
		}
		
		public String getNotSpeaker(String speaker){
			for(String sp:speakers){
				if(!sp.equals(speaker)){
					return sp;
				}
			}
			return null;
		}
		
		public boolean withOverlapping(){
			return speakers.size() > 1;
		}
		
		public boolean isEmpty(){
			return speakers.size() == 0;
		}
		
	
		public int speakerCount(){
			return speakers.size();
		}
		
}
