/**
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 **/

package fr.upemlv.lgtools.distagger;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Text;
import org.jdom.input.SAXBuilder;



/**
 * @author Matthieu Constant
 *
 */
public class TranscriptionFormatter {	
	
	//private static writeTurn
	
	private static TranscriberTurnInfo transcriberToValibelTurn(Element turn){
		
		String speakers = turn.getAttributeValue("speaker");
		if(speakers == null){
			return null;
		}
		TranscriberTurnInfo res = new TranscriberTurnInfo(speakers);
		
		int n = 0;
		for(Object node:turn.getContent()){
			if(node instanceof Element){
				Element e = (Element)node;				    			
				if(e.getName().equals("Who")){
					n = Integer.parseInt(e.getAttributeValue("nb")) - 1;
					res.addText(n,  e.getTextTrim());					
				}
				
			}						
			if(node instanceof Text){
				Text text = (Text)node;
				if(!text.getTextTrim().isEmpty()){
					res.addText(n,  text.getTextTrim());
				}
			}			
		}
		res.clean();		
		return res;
		
	}
	
	
	private static boolean isDefaultProcess(TranscriberTurnInfo prev,TranscriberTurnInfo cur,TranscriberTurnInfo next){
		int p = prev.speakerCount();
		int c = cur.speakerCount();
		int n = next.speakerCount();
		if(p == 1 && c == 2 && n == 1){
			String sp1 = prev.getFirstSpeaker();
			String sp2 = next.getFirstSpeaker();
			if((!sp1.equals(sp2)) && (cur.contains(sp1)) && (cur.contains(sp2))){
			  return false;
			}
		}
		return true;
		
	}
	
	private static List<TranscriberTurnInfo> processTurn(TranscriberTurnInfo prev,TranscriberTurnInfo cur,TranscriberTurnInfo next,Writer writer) throws IOException{
		List<TranscriberTurnInfo> defaultRes = new ArrayList<TranscriberTurnInfo>();
		defaultRes.add(cur);
		defaultRes.add(next);
				
		if(prev.isEmpty()){
			return defaultRes;
		}
		if(isDefaultProcess(prev, cur,next)){
			writer.write(prev.toString());
			return defaultRes;
		}
		String speaker1 = prev.getFirstSpeaker();
		String speaker2 = next.getFirstSpeaker();
		TranscriberTurnInfo t = new TranscriberTurnInfo(speaker1+" "+speaker2);
		t.addText(speaker1, prev.getText(speaker1) + " |- " + cur.getText(speaker1));
		t.addText(speaker2, cur.getText(speaker2) + " -| " + next.getText(speaker2));
		writer.write(t.toString());
		List<TranscriberTurnInfo> res = new ArrayList<TranscriberTurnInfo>();		
		res.add(new TranscriberTurnInfo());
		res.add(new TranscriberTurnInfo());
		
		return res;
	}
	
	public static void transcriberToValibel(String filename) throws JDOMException, IOException{
		
		SAXBuilder builder = new SAXBuilder(false);
	    Writer writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename+".val"),"UTF-8"));
		Document d = builder.build(new File(filename));
		Element root = d.getRootElement();
		root = root.getChild("Episode");
		List<TranscriberTurnInfo> trans = new LinkedList<TranscriberTurnInfo>();     
		for(Object o:root.getChildren("Section")){
	    	Element s = (Element)o;
	    	for(Object t:s.getChildren("Turn")){	    			    		
	    		TranscriberTurnInfo turn = transcriberToValibelTurn((Element)t);
	    		if(turn != null && !turn.isEmpty()){
	    			trans.add(turn);
	    		}	    	
	    	}	    		    
	    }
		writer.write("<deb id=\""+filename+"\">\n");
		TranscriberTurnInfo prev = new TranscriberTurnInfo();
		TranscriberTurnInfo cur = new TranscriberTurnInfo();
		for(TranscriberTurnInfo next:trans){
			List<TranscriberTurnInfo> list = processTurn(prev,cur,next,writer);
			prev = list.get(0);
			cur = list.get(1);
		}
		if(!prev.isEmpty()){
		  writer.write(prev.toString());
		}
		if(!cur.isEmpty()){
			  writer.write(cur.toString());
		}
		writer.write("<fin id=\""+filename+"\">\n");
		writer.close();		
	
	}
	
}
