/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Iterator;

public class TranscriptionsFilter {
	private AnneTranscriptions transcriptions;
	private AnneTagWriter tagWriter;
		
	public TranscriptionsFilter(AnneTranscriptions transcriptions){
		this.transcriptions = transcriptions;
		this.tagWriter = new AnneTagWriter(){

			public String write(Word w) {			
				return " "+w.getValue();
			}

			public String write(Meta m) {
				String val = m.getForm();
				if(val == null) return null;
				if(val.equals("//") || val.equals("(silence)")){
					return " {S}\n";
				}
				return null;
			}

			public String write(Disfluency dis) {				
				return null;
			}

			public String write(EndOverlapping eover) {		
				return null;
			}

			public String write(BackRefOverlapping brover) {
				return null;
			}

			public String write(RefOverlapping rover) {
				return null;
			}

			public String write(DisfluencySeq ds) {
				String s = "";
				Iterator<AnneTag> it = ds.getRrIterator();
				while(it.hasNext()){
					s += it.next().write(this);
				}
				return s;
			}
			
		};
	}

	private void writeSlot(Slot slot,Writer writer) throws IOException{
		slot.setIterator(new SlotIteratorWithDisfluency(slot,DisfluencyLimit.ENTIRE));
		for(AnneTag tag:slot){
			String s = tag.write(tagWriter);
			if(s != null){
				writer.write(s);
			}			
		}
	}
	
	private void writeTranscription(Transcription t,Writer writer) throws IOException{
		
		for(Slot slot:t){
			writeSlot(slot,writer);
			writer.write(" {S}\n");
		}
	}
	
	public void write(String filename){
		try{
			Writer writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),"UTF-8"));
            for(Transcription t:transcriptions.getTranscriptions()){
            	writeTranscription(t,writer);
            }
            writer.close();
		}
		catch(IOException e){
			System.err.println(e);
			System.exit(1);
		}
	}

}
