/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;

public class TranscriptionsFilterForElag {
	private AnneTranscriptions transcriptions;
	private AnneTagWriterForElag tagWriter = new AnneTagWriterForElag();;
	public TranscriptionsFilterForElag(AnneTranscriptions transcriptions){
		this.transcriptions = transcriptions; 
	}

	
	private void writeSlot(Slot slot,Writer writer) throws IOException{
		writer.write("{S}{#"+slot.getId()+",.IGN+slot} {"+slot.getSpeaker()+",.IGN+speaker}");
		slot.setIterator(new SlotIteratorWithDisfluency(slot,DisfluencyLimit.PART));
		for(AnneTag tag:slot){
			String s = tag.write(tagWriter);
			if(s != null){
				writer.write(s);
			}			
		}		
		writer.write(" {S}\n");
	}

	private void writeTranscription(Transcription t,Writer writer) throws IOException{
		writer.write("{S}{<deb id="+t.getId()+">,.IGN+ENR} {S}\n");
		for(Slot slot:t.getSlots()){
		
			writeSlot(slot,writer);			
		}
		writer.write("{S}{<fin id="+t.getId()+">,.IGN+ENR} {S}\n");
	}

	public void write(String filename){
		try{
			Writer writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),"UTF-8"));
			for(Transcription t:transcriptions.getTranscriptions()){            	
				writeTranscription(t,writer);            	
			}
			writer.close();
		}
		catch(IOException e){
			System.err.println(e);
			System.exit(1);
		}
	}


}
