/*
 * DisTagger
 *Author: Matthieu Constant
 *
 * Copyright (C) 2010 Université Paris-Est, LIGM and CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.distagger;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class TranscriptionsXMLWriter {
	private AnneTranscriptions transcriptions;
	private Document document;
	private AnneTagXMLWriter tagWriter;

	public TranscriptionsXMLWriter(AnneTranscriptions trans){
		try {
			this.transcriptions = trans;
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder parser = factory.newDocumentBuilder();
			document = parser.newDocument();
			this.tagWriter = new DefaultTagWriter(document);
		}
		catch(ParserConfigurationException e){
			System.err.println(e);
			System.exit(1);
		}
	}

	private void addTag(AnneTag tag,Element slot){
		slot.appendChild(tag.toXMLElement(tagWriter));    	  
	}


	private void addSlot(Slot slot,Element tr){
		Element s = document.createElement("slot");
		s.setAttribute("id",""+slot.getId());
		s.setAttribute("speaker",slot.getSpeaker());
		for(AnneTag tag:slot.getTags()){
			addTag(tag,s);
		}
		tr.appendChild(s);
	}

	private void addTranscription(Transcription tr,Element trans){
		Element t = document.createElement("transcription");
		t.setAttribute("id", tr.getId());
		for(Slot slot:tr.getSlots()){
			addSlot(slot,t);    		  
		}    	  
		trans.appendChild(t);
	}

	public void write(String filename) {
		try{
		Element trans = document.createElement("transcriptions");
		for(Transcription tr:transcriptions.getTranscriptions()){
			addTranscription(tr,trans);
		}

		document.appendChild(trans);
		TransformerFactory transFactory = TransformerFactory.newInstance();
		Transformer transformer = transFactory.newTransformer();
		Source src = new DOMSource(document);
		Result dest = new StreamResult(new File(filename));		
		transformer.transform(src,dest);
		}
		catch(TransformerConfigurationException e){
			System.err.println(e);
			System.exit(1);
		}
		catch(TransformerException e){
			System.err.println(e);
			System.exit(1);
		}
	}


}
