/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import edu.berkeley.nlp.PCFGLA.CoarseToFineMaxRuleParser;
import edu.berkeley.nlp.PCFGLA.CoarseToFineNBestParser;
import edu.berkeley.nlp.PCFGLA.Grammar;
import edu.berkeley.nlp.PCFGLA.Lexicon;
import edu.berkeley.nlp.PCFGLA.ParserData;
import edu.berkeley.nlp.PCFGLA.TreeAnnotations;
import edu.berkeley.nlp.PCFGLA.BerkeleyParser.Options;
import edu.berkeley.nlp.ling.Tree;
import edu.berkeley.nlp.util.Numberer;
import edu.berkeley.nlp.util.OptionParser;
import fr.upemlv.lgtools.text.Sentence;

/**
 * BKY parser
 * 
 * @author Matthieu Constant
 *
 */

public class BKYParser {
	private final String grammarPath;
	private final CoarseToFineMaxRuleParser parser;

	
	/**
	 * @param grammarPath
	 */
	public BKYParser(String grammarPath) {
		this.grammarPath = grammarPath;
		this.parser = loadParser();
	}
	
	/* This function results from some copy/paste from BerkeleyParser.main(). */
	
	private CoarseToFineMaxRuleParser loadParser(){
		
		String[] args = new String[]{"-gr",grammarPath};
		OptionParser optParser = new OptionParser(Options.class);
		Options opts = (Options) optParser.parse(args, true);
		String inFileName = opts.grFileName;
	    ParserData pData = ParserData.Load(inFileName);
	    if (pData==null) {
	      System.out.println("Failed to load grammar from file"+inFileName+".");
	      System.exit(1);
	    }
	    Grammar grammar = pData.getGrammar();
	    Lexicon lexicon = pData.getLexicon();
	    Numberer.setNumberers(pData.getNumbs());
		
	    //return new CoarseToFineMaxRuleParser(grammar, lexicon, 1.0,-1,opts.viterbi,opts.substates,opts.scores, opts.accurate, false, true, true);
	    return new CoarseToFineNBestParser(grammar, lexicon, 5,1.0,-1,opts.viterbi,opts.substates,opts.scores, opts.accurate, false, false, true);
	    //parser.binarization = pData.getBinarization();
	    
	}
	
	
	
	
	/**
	 * 
	 * @param sentence tokenized sentence
	 * @return the parse tree resulting from the parsing of the sentence
	 */
	
	public List<ParseTree> parse(Sentence<String> sentence,int kbest){
		List<ParseTree> trees = new LinkedList<ParseTree>(); 
		for(Tree<String> t:parser.getKBestConstrainedParses(sentence.getSequence(), null, kbest)){
			trees.add(new ParseTree(t));
		}
		return trees;
	}	
	
	public ParseTree parse(Sentence<String> sentence){
		List<ParseTree> parses = parse(sentence,1); 
		return parses.isEmpty()?null:parses.get(0);
	}
	
	public List<ParseTree> parse(String sentence,int kbest){
		return parse(new Sentence<String>(Arrays.asList(sentence.split(" "))),kbest);
	}
	
	public ParseTree parse(String sentence){
		List<ParseTree> parses = parse(sentence,1); 
		return parses.isEmpty()?null:parses.get(0);
	}
	
}
