/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;

import fr.upemlv.lgtools.utils.LGToolsProcess;

/**
 * 
 * Simple BKY parser wrapper
 * 
 * @author Matthieu Constant
 *
 */

public class BKYParserWrapper {
	private final String dirPath;
	
	/**
	 * 
	 * @param dirPath directory path where grammar is stored
	 */
	public BKYParserWrapper(String dirPath){
		this.dirPath = dirPath;
		File dir = new File(dirPath);
		if(!dir.exists()){
			System.err.println("Create directory "+dir);
			dir.mkdirs();
		}	   	
	}
	
	//used for main grammar
	//
	
	public void grammarTrain(String grammarName,final String treebankPath,final int cycleCount) throws IOException, InterruptedException{
		final String grammarPath = dirPath+File.separator+grammarName+".gr";
		
		String[] command = new String[]{"sh",ParsingConstants.BKY_GRAMMAR_TRAINER,grammarPath,treebankPath,""+cycleCount};
		
		LGToolsProcess.exec(command);
				
	}
	
	
	/**
	 * Warning: this method calls the main method from Berkeley Parser and exit
	 * @param grammarName
	 * @param inputFile
	 * @param outputFile
	 * @throws InterruptedException 
	 * @throws IOException 
	 */
	
	
	public void parse(String grammarName,String inputFile,String outputFile) throws IOException, InterruptedException{
		String grammarPath = dirPath+File.separator+grammarName+".gr";
				
		String[] command = new String[]{"sh",ParsingConstants.BKY_PARSER,grammarPath,inputFile,outputFile};
		System.err.println(Arrays.asList(command));
		
		LGToolsProcess.exec(command);
	}
	

}
