/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing;

import java.io.File;

import fr.upemlv.lgtools.text.RawText;
import fr.upemlv.lgtools.text.Sentence;
import fr.upemlv.lgtools.utils.FileUtilities;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class BaselineParser extends MatParser {
    private static final String FTB = "ftb";
    private static final String GRAMMAR = "basegrammar";
    private BKYParser parser;
	
    public BaselineParser(ParsingConfig config,int itLearn){
		super(config,itLearn);
		loadParser();
	}
    
	public BaselineParser(ParsingConfig config){
		this(config,-1);
	}
	
	private void loadParser(){
		String grammarPath = getConfig().getOutputDir()+File.separator+GRAMMAR+".gr";
		if(new File(grammarPath).exists()){
		  parser = new BKYParser(grammarPath);
		}		
	}
	
	@Override
	public void learn(Treebank trainTreebank, Treebank validtreebank,
			Treebank testtreebank) throws Exception {
		String dirpath = getConfig().getOutputDir()+File.separator+FTB;
		FileUtilities.createDir(dirpath);
		String path = dirpath+File.separator+FTB;
		trainTreebank.save(path+"_1.mrg");
		validtreebank.save(path+"_2.mrg");
		testtreebank.save(path+"_3.mrg");
		BKYParserWrapper pw = new BKYParserWrapper(getConfig().getOutputDir());
		pw.grammarTrain(GRAMMAR, dirpath, 2);
		loadParser();
	}

	@Override
	public ParseTree parse(Sentence<String> sentence, SentenceItems items) {		
		return parser.parse(sentence);
	}
	
	public Treebank parse(RawText text){
		return parse(text,null);
	}

}
