/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.JSAPResult;

import fr.upemlv.lgtools.tagging.LGTaggerMain;

/**
 * Program that trains a PCFG by including MWE information
 */

public class LGParserTrain {
	private static String ITERATION_NUMBER = "iter";
   private static String TRAIN_CORPUS = "train";
   private static String VALIDATE_CORPUS = "validate";
   private static String CONFIG = "config";
   private static String PARSER_TYPE = "parser";
   private static String SIMPLE_PARSER = "simple";
   private static String CRF_PARSER = "crf";
   private static String BASELINE_PARSER = "baseline";
   
   
	
	private static  JSAP createJSAP() throws JSAPException{
		JSAP jsap = new JSAP();

		FlaggedOption opt;
		
		opt = new FlaggedOption(TRAIN_CORPUS)
		.setStringParser(JSAP.STRING_PARSER)
		.setRequired(true)
		.setShortFlag('t')
		.setRequired(true)
		.setLongFlag(TRAIN_CORPUS);
		opt.setHelp("Path of the training corpus");
		jsap.registerParameter(opt);

			opt = new FlaggedOption(VALIDATE_CORPUS)
			.setStringParser(JSAP.STRING_PARSER)		
			.setShortFlag('v')
			.setLongFlag(VALIDATE_CORPUS)
			.setRequired(true)
			;
			opt.setHelp("Path of validation corpus");
			jsap.registerParameter(opt);

			opt = new FlaggedOption(CONFIG)
			.setStringParser(JSAP.STRING_PARSER)		
			.setShortFlag('c')
			.setLongFlag(CONFIG)
			.setRequired(true)
			;
			opt.setHelp("Path of configuration file");
			jsap.registerParameter(opt);

			opt = new FlaggedOption(PARSER_TYPE)
			.setStringParser(JSAP.STRING_PARSER)        
			.setShortFlag('p')
			.setLongFlag(PARSER_TYPE);
			opt.setHelp("Parser type: 'baseline', 'simple' or 'crf'");
			opt.setRequired(false);
			opt.setDefault("simple");
			jsap.registerParameter(opt);
		
			
			opt = new FlaggedOption(ITERATION_NUMBER)
			.setStringParser(JSAP.INTEGER_PARSER)		
			.setShortFlag('i')
			.setLongFlag(ITERATION_NUMBER)
			.setDefault("5")
			.setRequired(false)
			;
			opt.setHelp("Number of iterations during PCFG learning");
			jsap.registerParameter(opt);
			
			return jsap;
	}
	
	private static void usage(JSAP jsap){
		System.err.print("Usage: ");
		System.err.print("java "+LGTaggerMain.class.getCanonicalName()+" ");
		System.err.println(jsap.getUsage());
		System.err.print(jsap.getHelp());
		System.exit(1);
	}
	
	
	public static void main(String[] argv) throws Exception{
		JSAP jsap = createJSAP();
		JSAPResult arguments = jsap.parse(argv);
		if(!arguments.success()){
			usage(jsap);
		}
		String trainingCorpusPath = arguments.getString(TRAIN_CORPUS);
		String validationCorpusPath = arguments.getString(VALIDATE_CORPUS);
		String configPath = arguments.getString(CONFIG);
		String parserType = arguments.getString(PARSER_TYPE);
		int itCnt = arguments.getInt(ITERATION_NUMBER);
		
		Treebank trainTreebank = new Treebank(trainingCorpusPath);
		Treebank devTreebank = new Treebank(validationCorpusPath);
		Treebank testTreebank = new Treebank(validationCorpusPath);		
		ParsingConfig config = new ParsingConfig(configPath);
		
		
		MatParser parser = null;
		if(parserType.equals(SIMPLE_PARSER)){
			parser = new SimpleMweMatParser(config,itCnt);
		}
		else{
			if(parserType.equals(CRF_PARSER)){
				parser = new MweMatParser(config, itCnt);
			}
			else{
			   if(parserType.equals(BASELINE_PARSER)){
				  parser = new BaselineParser(config,itCnt); 
			   }
			   else{
				   usage(jsap);
			   }
			}
		}
		parser.learn(trainTreebank, devTreebank, testTreebank);
	}	
}
