/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing;

import java.util.Iterator;

import fr.upemlv.lgtools.text.RawText;
import fr.upemlv.lgtools.text.Sentence;

/**
 * 
 * @author Matthieu Constant
 *
 */

public abstract class MatParser {
    private final ParsingConfig config;
	private Treebank currentIntermediateParses = null;
    private final boolean withMweAnnotation;
    private final int iterLearn; 
    
    
    protected MatParser(ParsingConfig config,int iterLearn,boolean withMweAnnotation){
		this.config = config;
		this.withMweAnnotation = withMweAnnotation;
		this.iterLearn = iterLearn;
	}
    
    
    protected MatParser(ParsingConfig config,int iterLearn){
		this(config,iterLearn,false);
	}
    
    
	protected MatParser(ParsingConfig config){
		this(config,-1,false);
	}
	
	
	/**
	 * For parsing
	 * @param config
	 * @param withMweAnnotation
	 */
	protected MatParser(ParsingConfig config,boolean withMweAnnotation){
		this(config,-1,withMweAnnotation);
	}
	
	public abstract void learn(Treebank trainTreebank,Treebank validtreebank,Treebank testtreebank) throws Exception;
	public abstract ParseTree parse(Sentence<String> sentence,SentenceItems items);
	
	protected int getIterNumber(){
		return iterLearn;
	}
	
	protected boolean isWithMweAnnotation(){
		return withMweAnnotation;
	}
	
	public Treebank parse(RawText text,TextItems items){
		Iterator<SentenceItems> sitems  = items == null ? null :items.iterator();
		Treebank treebank = new Treebank();
		if(withMweAnnotation){
			currentIntermediateParses = new Treebank();
		}
		int cnt = 0;
		for(Sentence<String> sentence:text){
			if(cnt%100 == 0 && cnt != 0){
				System.err.println(cnt +" sentences parsed.");
			}
			treebank.addTree(parse(sentence,sitems == null ? null:sitems.next()));
			cnt++;
		}
		return treebank;	
	}

	/**
	 * @return the config
	 */
	public ParsingConfig getConfig() {
		return config;
	}
	
	public Treebank getIntermediateParses(){
		return currentIntermediateParses;
	}

	public void instanciateIntermediateParses(){
		currentIntermediateParses = new Treebank();
	}
	
	protected void addIntermediateParse(ParseTree tree){
		//System.err.println(tree);
		currentIntermediateParses.addTree(tree);
	}
}
