/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class PreprocessedItem {
	private final int startPosition;
	private final int endPosition;
	private final String preterminal; //for grammar
	private final String terminal; //for grammar
	private final String partOfSpeech; //for MWE recognizer
	
	/**
	 * @param startPosition
	 * @param endPosition
	 * @param preterminal preterminal in grammar
	 * @param terminal terminal in grammar
	 * @param partOfSpeech partOfSpeech for MWE recognizer
	 */
	public PreprocessedItem(int startPosition, int endPosition,
			String terminal, String preterminal,String partOfSpeech) {
		this.startPosition = startPosition;
		this.endPosition = endPosition;
		this.preterminal = preterminal;
		this.terminal = terminal;
		this.partOfSpeech = partOfSpeech;
	}
	
	
	/**
	 * @return the startPosition
	 */	
	public int getStartPosition() {
		return startPosition;
	}
	
	/**
	 * @return the endPosition
	 */
	public int getEndPosition() {
		return endPosition;
	}
	
	/**
	 * @return the preterminal
	 */
	public String getPreterminal() {
		return preterminal;
	}
	
	/**
	 * @return the terminal
	 */
	public String getTerminal() {
		return terminal;
	}


	/**
	 * @return the partOfSpeech
	 */
	public String getPartOfSpeech() {
		return partOfSpeech;
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "PreprocessedItem [endPosition=" + endPosition
				+ ", partOfSpeech=" + partOfSpeech + ", preterminal="
				+ preterminal + ", startPosition=" + startPosition
				+ ", terminal=" + terminal + "]";
	}


	
	
}
