/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class TextItems implements Iterable<SentenceItems>{    
    private List<SentenceItems> sentences = new LinkedList<SentenceItems>();
	
	public TextItems() {}
    
	public TextItems(String filename) throws IOException {
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(filename), "UTF-8"));
		String line;
		while((line = reader.readLine()) != null){
			SentenceItems items = new SentenceItems();
			if(line.isEmpty()){
				sentences.add(null);
				continue;
			}
			String[] tab = line.split(";");
			for(String i:tab){
				String[] tab2 = i.split("#");
				PreprocessedItem item = new PreprocessedItem(Integer.parseInt(tab2[0]), Integer.parseInt(tab2[1]), tab2[3], tab2[2], tab2[4]);
				items.addItem(item);
			}
			sentences.add(items);
		}
		reader.close();
	}
	
	
	public void addSentence(SentenceItems items){
		sentences.add(items);
	}


	@Override
	public Iterator<SentenceItems> iterator() {
		return sentences.iterator();
	}
 
	
	public void save(String filename) throws IOException{
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),"UTF-8"));
		for(SentenceItems s:this){
			String tmp = "";
			for(PreprocessedItem item:s){
			   	tmp += ";"+item.getStartPosition()+"#"+item.getEndPosition()+"#"+item.getPreterminal()+"#"+item.getTerminal()+"#"+item.getPartOfSpeech();
			}
			if(!tmp.isEmpty()){
			  tmp = tmp.substring(1);
			}
			writer.write(tmp+"\n");
		}
		writer.close();
	}
	
}
