/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing;

import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import edu.berkeley.nlp.PCFGLA.TreeAnnotations;
import edu.berkeley.nlp.ling.Tree;
import edu.berkeley.nlp.ling.Trees;
import fr.upemlv.lgtools.tagging.lex.Tagset;
import fr.upemlv.lgtools.text.LightRawText;
import fr.upemlv.lgtools.text.LightTaggedText;
import fr.upemlv.lgtools.text.RawText;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class Treebank implements Iterable<ParseTree>{
	final Collection<ParseTree> trees = new LinkedList<ParseTree>();
	
	/**
	 * Create an empty treebank
	 */
	public Treebank(){
	  	
	}
	
	
	/**
	 * Load a tree bank from a file
	 * @param treebankPath treebank filepath
	 * @throws UnsupportedEncodingException
	 * @throws FileNotFoundException
	 */
	public Treebank(String treebankPath) throws UnsupportedEncodingException, FileNotFoundException{
		Trees.PennTreeReader reader = new Trees.PennTreeReader(new InputStreamReader(
				new FileInputStream(treebankPath), "UTF-8"));
		while(reader.hasNext()){
			trees.add(new ParseTree(reader.next()));
		}
	}

	/**
	 * add a new tree in treebank
	 *  
	 * @param tree tree
	 */
	
	public void addTree(ParseTree tree){
	   this.trees.add(tree);	
	}
	
	
	public void addTrees(List<ParseTree> trees){
		   this.trees.addAll(trees);	
	}

	
	@Override
	public Iterator<ParseTree> iterator() {
		return trees.iterator();
	}

	public RawText toRawText(Tagset tagset){
	   return new LightRawText(new LightTaggedText(tagset, this)); 	
	}
	
	
	public void save(String filename) throws IOException{
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),"UTF-8"));
		for(ParseTree tree:trees){
			writeParseTree(tree, writer);
		}
		writer.close();
	} 
	
	/**
	 * write a tree with a Writer
	 * @param tree
	 * @param writer
	 * @throws IOException
	 */
	
	public static void writeParseTree(ParseTree tree,Writer writer) throws IOException{
		if(tree == null){
			writer.write("(())\n");
			return;
		}		
		Tree<String> node = TreeAnnotations.unAnnotateTree(tree.getTree());
		writer.write("( "+node.getChildren().get(0)+" )\n");
	}
	
}
