/**
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing.mwe;

import java.util.Iterator;
import java.util.Set;

import fr.upemlv.lgtools.parsing.ItemFilter;
import fr.upemlv.lgtools.parsing.ParseTree;
import fr.upemlv.lgtools.parsing.PreprocessedItem;
import fr.upemlv.lgtools.parsing.SentenceItems;
import fr.upemlv.lgtools.parsing.TextItems;
import fr.upemlv.lgtools.parsing.Treebank;
import fr.upemlv.lgtools.tagging.lex.Tagset;
import fr.upemlv.lgtools.text.LightRawText;
import fr.upemlv.lgtools.text.LightTaggedText;
import fr.upemlv.lgtools.text.RawText;

public class FilteredGoldMweRecognizer extends GoldMWERecognizer {
	private final Set<String> allowedCats;
	/**
	 * @param tagset
	 */
	public FilteredGoldMweRecognizer(Tagset tagset,AutomaticMweRecognizer recognizer) {
		super(tagset,recognizer);
		allowedCats = null;
	}
	
	public FilteredGoldMweRecognizer(Tagset tagset,AutomaticMweRecognizer recognizer,Set<String> allowedCats) {
		super(tagset,recognizer);
		this.allowedCats = allowedCats;
	}

	public static SentenceItems filterItems(SentenceItems items,ParseTree tree,Set<String> allowedCats){
		ItemFilter filter = new ItemFilter(allowedCats.isEmpty()?null:allowedCats);
		SentenceItems res = new SentenceItems();
		for(PreprocessedItem item:items){
			///System.err.println(allowedCats);
		  //System.err.println(item);
		  item = filter.apply(item, tree);
		  if(item != null){
			//System.err.println("==>"+item);
		    res.addItem(item);	
		  }
		}
		
		return res;
	}
	
	@Override
	public TextItems apply(RawText text,Treebank goldTreebank) throws Exception {
		TextItems list = recognizer.apply(text);
		TextItems res = new TextItems();
		Iterator<ParseTree> it = goldTreebank.iterator();
		for(SentenceItems items:list){
			res.addSentence(filterItems(items, it.next(),allowedCats));
		}
		
		return res;
	}
	
	
	
	@Override
	public TextItems apply(Treebank treebank) throws Exception {		
		return apply(new LightRawText(new LightTaggedText(tagset, treebank)),treebank);
	}

}
