/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing.mwe;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.List;

import fr.upemlv.lgtools.parsing.ParseTree;
import fr.upemlv.lgtools.parsing.Treebank;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class GoldMweSubtreeGetter extends SubtreeGetter {
    private final String dirPath;
    private final HashMap<String, ParseTree> map = new HashMap<String, ParseTree>();
    
	
	/**
	 * @param dirPath
	 * @throws FileNotFoundException 
	 * @throws UnsupportedEncodingException 
	 */
	public GoldMweSubtreeGetter(String dirPath) throws UnsupportedEncodingException, FileNotFoundException {
		this.dirPath = dirPath;
		fillMap();
	}

	private void fillMap() throws UnsupportedEncodingException, FileNotFoundException{
		File dir = new File(this.dirPath);
		if(!dir.isDirectory()){
			throw new RuntimeException(dir + " should be a directory!");
		}
		for(String f:dir.list(new FilenameFilter() {			
			@Override
			public boolean accept(File dir, String name) {
				return name.endsWith(".mrg");
			}
		})){
			fillMap(new Treebank(dir.getAbsolutePath()+File.separator+f));
			
		}		
	}


	private void fillMap(Treebank treebank){
		for(ParseTree tree:treebank){
			ParseTree t = new ParseTree(tree.getTree().getChildren().get(0));
			map.put(getItemKey(t.getTerminalSymbols(), t.getTree().getLabel()),t);
		}
	}
	

	private static String getItemKey(List<String> symbols,String preterminal){
	   preterminal = preterminal.replaceAll("-", "");
	   //character - is a special symbol for a node label
	   return preterminal+"::"+symbols.toString();	
	}
	
	@Override
	public ParseTree get(List<String> item, String nodeLabel) {

		ParseTree res = map.get(getItemKey(item, nodeLabel));
		//System.err.println(getItemKey(item, nodeLabel));
		if(res == null){
			throw new RuntimeException("Item "+getItemKey(item, nodeLabel)+" does not exist! You should use the force mode...");
		}		
		return res.createCopy();
	}

}
