/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing.mwe;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import edu.berkeley.nlp.ling.Tree;
import fr.upemlv.lgtools.parsing.BKYParser;
import fr.upemlv.lgtools.parsing.ParseTree;
import fr.upemlv.lgtools.text.Sentence;

/**
 * @author Matthieu Constant
 *
 */
public class MweSubtreeGetter extends SubtreeGetter{
	private final BKYParser parser;
	private final BKYParser standardParser;
	
	
	/**
	 * @param parser
	 */
	public MweSubtreeGetter(BKYParser parser,BKYParser standardParser ) {
		this.parser = parser;
		this.standardParser = standardParser;
	}


	@SuppressWarnings("unchecked")
	private static ParseTree createDefaultTree(List<String> item){
		Tree<String> root = new Tree<String>("NP");
		List<Tree<String>> children = new LinkedList<Tree<String>>();
		root.setChildren(children);
		for(String w:item){			
			Tree<String> c = new Tree<String>("NPP");
			children.add(c);
			c.setChildren(Arrays.asList(new Tree<String>(w)));
		}
		return new ParseTree(root);
	}
	
	@Override
	public ParseTree get(List<String> item, String nodeLabel) {
		List<ParseTree> trees = parser.parse(new Sentence<String>(item), 10);
		//System.err.println(item);
		if(trees.isEmpty()){
			System.err.println("** Warning: item "+item+" has not been recognized by MWE grammar trying to guess its syntatic structure...");
			trees = standardParser.parse(new Sentence<String>(item), 10);
			if(trees.isEmpty()){
				//System.err.println("$$");
				return createDefaultTree(item);
				//throw new RuntimeException("NO AVAILABLE MWE SUBTREE???");
			}
			
		}
		for(ParseTree tree:trees){
			Tree<String> t = tree.getTree().getChildren().get(0);
			if(t.getLabel().equals(nodeLabel)){
				return new ParseTree(t);
			}
		}
		//default tree = best-ranked parse
		return new ParseTree(trees.get(0).getTree().getChildren().get(0));
	}
	
	
}
