/**
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.parsing.mwe;

import fr.upemlv.lgtools.parsing.ParsingConfig;
import fr.upemlv.lgtools.parsing.TextItems;
import fr.upemlv.lgtools.tagging.LGTaggerConfig;
import fr.upemlv.lgtools.tagging.impl.LGTaggerPreprocessor;
import fr.upemlv.lgtools.tagging.impl.PreprocessingContext;
import fr.upemlv.lgtools.tagging.lex.InternalTagset;
import fr.upemlv.lgtools.text.MultiFeaturedElement;
import fr.upemlv.lgtools.text.MultiFeaturedText;
import fr.upemlv.lgtools.text.PropertyConstants;
import fr.upemlv.lgtools.text.RawText;
import fr.upemlv.lgtools.text.Sentence;

/**
 * Simple MWE recognizer using Shortest Path Heuristics after lexical analysis
 * @author Matthieu Constant
 *
 */

public class SimpleMweRecognizer extends AutomaticMweRecognizer{
	
	public SimpleMweRecognizer(ParsingConfig config){
		super(config);
	}
	
	
	public SimpleMweRecognizer(LGTaggerConfig config){
		super(config);
	}
	

	@Override
	protected boolean isB(MultiFeaturedElement mfe) {
		String iob = mfe.get(PropertyConstants.MWE_IOB);
		return InternalTagset.B.equals(iob);
	}

	@Override
	protected boolean isI(MultiFeaturedElement mfe) {
		String iob = mfe.get(PropertyConstants.MWE_IOB);
		return InternalTagset.I.equals(iob);
	}

	@Override
	protected boolean isO(MultiFeaturedElement mfe) {
		String iob = mfe.get(PropertyConstants.MWE_IOB);
		return PropertyConstants.DEFAULT_VALUE.equals(iob);
	}

	@Override
	public TextItems apply(RawText text) throws Exception {		
		LGTaggerConfig cfg = taggerConfig;
		PreprocessingContext context = new PreprocessingContext(cfg.getLang(), cfg.getResources(), extractors, cfg.getTagset());
		LGTaggerPreprocessor processor = new LGTaggerPreprocessor(context);
		MultiFeaturedText mtext = processor.preprocess(text);		
		TextItems items = new TextItems();
		for(Sentence<MultiFeaturedElement> sentence: mtext){
			items.addSentence(extractSentenceMWEs(sentence));
		}
		return items;
	}
	
}
