/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * Represent the configuration for token property extraction 
 * 
 * @author Matthieu Constant
 *
 */

public class ExtractConfig {
	private final String resources;
	private final String properties;
	private final String lang;
	
	public ExtractConfig(String configpath) throws IOException{
		
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(configpath), "UTF-8"));
			String s = reader.readLine();
			String[] tmp = s.split("=");
			if(tmp[0].equals("LANG")){
				this.lang = tmp[1];
			}
			else{
				throw new RuntimeException("In property extraction config, first line should be LANG\n;");
			}
			
			s = reader.readLine();
			tmp = s.split("=");
			
			if(tmp[0].equals("RESOURCES")){
				this.resources = tmp[1];
			}
			else{
				throw new RuntimeException("In property extraction config, second line should be RESOURCES\n;");
			}
			
			s = reader.readLine();
			tmp = s.split("=");
			if(tmp[0].equals("PROPERTIES")){
				this.properties = tmp[1];
			}
			else{
				throw new RuntimeException("In property extraction config, third line should be PROPERTIES\n;");
			}
			
			reader.close();
		
	}

	/**
	 * @return the resources
	 */
	public String getResources() {
		return resources;
	}

	/**
	 * @return the properties
	 */
	public String getProperties() {
		return properties;
	}

	/**
	 * @return the lang
	 */
	public String getLang() {
		return lang;
	}
	
	
	
}
