package fr.upemlv.lgtools.tagging;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import fr.upemlv.lgtools.tagging.impl.LGTaggerPreprocessor;
import fr.upemlv.lgtools.tagging.impl.PreprocessingContext;
import fr.upemlv.lgtools.tagging.impl.Utils;
import fr.upemlv.lgtools.tagging.lex.InternalTagset;
import fr.upemlv.lgtools.text.FullRawText;
import fr.upemlv.lgtools.text.MultiFeaturedText;
import fr.upemlv.lgtools.text.PropertyExtractor;
import fr.upemlv.lgtools.text.PropertyExtractorFactory;
import fr.upemlv.lgtools.text.RawText;

public class ExtractTextProperties {

	
	
	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
		if(args.length != 3){		
		  System.err.println("Usage: java "+ExtractTextProperties.class.getCanonicalName()+" <configpath> <text> <outputpath>");
		  System.err.println("<configpath> is the path of the configuration file (same as lgtagger): indicating the language and the lexical resources to be used.");
		  System.err.println("<properties> is the list of properties to be extracted (properties must be separated by a comma): e.g. \"w,MWE_IOB,MWE(POS)\"");
		  System.err.println("<text> is the text to be processed (one token per line)");		  
		  System.exit(1);
		}
		LGTaggerConfig config = new LGTaggerConfig(args[0], false);		
		String properties = args[1];
		String textFilename = args[2];
		String lang = config.getLang();
		InternalTagset tagset = config.getTagset();
		
		List<String> resources = new LinkedList<String>();
		for(String res:config.getResources()){
			if(!res.isEmpty()){
				resources.add(Utils.getAbsolutePathForLex(res, lang));
			}
		}		
		List<PropertyExtractor> extractors = new LinkedList<PropertyExtractor>();
		for(String extr:properties.split(",")){
			String[] tmp = extr.split("\\(");
			String prop = tmp[0];
			ArrayList<String> arguments = new ArrayList<String>();
			if(tmp.length == 2){
				arguments.add(tmp[1].substring(0,tmp[1].length() - 1));
			}
			extractors.addAll(PropertyExtractorFactory.createInstances(prop, arguments, false, tagset));
		}
		RawText text = new FullRawText(textFilename, "UTF-8", null, false);
		PreprocessingContext context = new PreprocessingContext(lang, resources, extractors, tagset);
		LGTaggerPreprocessor processor = new LGTaggerPreprocessor(context);
		MultiFeaturedText mtext = processor.preprocess(text);
		mtext.dump();		
	}

}
