/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging;

import java.util.Arrays;
import java.util.List;

import fr.upemlv.lgtools.tagging.impl.LGTagger;
import fr.upemlv.lgtools.tagging.impl.LGTaggerPreprocessor;
import fr.upemlv.lgtools.tagging.impl.PreprocessingContext;
import fr.upemlv.lgtools.text.MultiFeaturedText;
import fr.upemlv.lgtools.text.PropertyExtractor;
import fr.upemlv.lgtools.text.PropertyExtractorFactory;
import fr.upemlv.lgtools.text.RawText;
import fr.upemlv.lgtools.text.TaggedText;

public class LGLemmatizer {
	private final LGTagger tagger;
	private final TaggingEnvironment env;

	/**
	 * @param tagger
	 */
	public LGLemmatizer(LGTagger tagger,TaggingEnvironment env) {
		this.tagger = tagger;
		this.env = env;
	}
	
	public MultiFeaturedText perform(RawText text) throws Exception{
		TaggedText tt = tagger.perform(text);
		PropertyExtractor p1 = PropertyExtractorFactory.createTokenPropertyExtractor();
		PropertyExtractor p2 = PropertyExtractorFactory.createLemmaPropertyExtractor();
		PreprocessingContext pc = new PreprocessingContext(env);
		List<PropertyExtractor> extractors = Arrays.asList(p1,p2);		
		LGTaggerPreprocessor processor = new LGTaggerPreprocessor(new PreprocessingContext(pc.getLang(), pc.getResources(), extractors, pc.getTagset()));		
		return processor.preprocess(tt);		
	}

}
