/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging;

import java.io.IOException;

import fr.upemlv.lgtools.tagging.impl.Utils;
import fr.upemlv.lgtools.tagging.lex.InternalTagset;
import fr.upemlv.lgtools.tagging.lex.StandardTagParser;
import fr.upemlv.lgtools.tagging.lex.Tagset;
import fr.upemlv.lgtools.tagging.lex.UndefinedFlexibleTagset;
import fr.upemlv.lgtools.tagging.nlp.MultiAnnotationScore;
import fr.upemlv.lgtools.text.FullTaggedText;
import fr.upemlv.lgtools.text.TaggedText;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class LGTaggerEval {

	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		if(args.length != 2){
			System.err.print("Usage: ");
			System.err.print("java "+LGTaggerEval.class.getCanonicalName()+" ");
			System.err.println("<tagged corpus> <gold tagged corpus>");
			System.err.println("<tagged corpus> : automatically tagged corpus path (one-word\ttag-pair-per-line-format)");
			System.err.println("<gold tagged corpus> : reference tagged corpus path");
			System.exit(1);
		}
		//UndefinedFlexibleTagset
		UndefinedFlexibleTagset tagset = new UndefinedFlexibleTagset();
		TaggedText test = new FullTaggedText(args[0], "UTF-8", tagset, new StandardTagParser(), FullTaggedText.ONE_WORD_PER_LINE);
		TaggedText gold = new FullTaggedText(args[1], "UTF-8", tagset, new StandardTagParser(), FullTaggedText.ONE_WORD_PER_LINE);
		MultiAnnotationScore sc = Utils.eval(test, gold);
		System.err.println(sc);

	}

}
