/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging;

import java.io.IOException;

import fr.upemlv.lgtools.tagging.impl.LGTagger;
import fr.upemlv.lgtools.tagging.impl.LGTaggerFactory;
import fr.upemlv.lgtools.tagging.lex.StandardTagParser;
import fr.upemlv.lgtools.tagging.lex.Tagset;
import fr.upemlv.lgtools.text.FullRawText;
import fr.upemlv.lgtools.text.FullTaggedText;
import fr.upemlv.lgtools.text.RawText;

/**
 * @author Matthieu Constant
 *
 */
public class LGTaggerMain {
	
       
    private static FullTaggedText createGoldText(String textPath,Tagset tagset) throws IOException{    	
    	return new FullTaggedText(textPath, "utf-8", tagset, new StandardTagParser(),FullTaggedText.ONE_WORD_PER_LINE);    	
    }
    
    
    private static RawText createText(String textPath,TaggingEnvironment env) throws IOException, InterruptedException{    	
    	return new FullRawText(textPath, "utf-8", "French",env.isSegmented());
    }
    
    
	/**
	 * LGTagger is a part-of-speech tagger based on lexical resources and a CRF model, taking multiword units into account
	 *  
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
		
		TaggingEnvironment env = new TaggingEnvironment(new LGTaggerArguments(false,args));
		
		
		
		LGTagger tagger = LGTaggerFactory.createTagger(env,null);
		LGLemmatizer lemmatizer = null;
		if(env.isWithLemma()){
			lemmatizer = new LGLemmatizer(tagger, env);
		}
		
		for(String textPath:env.getTexts()){
			if(env.isEvalMode()){
			  FullTaggedText gold = createGoldText(textPath, env.getInternalTagset());
			  //gold.save("/tmp/t1.txt");
			  tagger.performAndEvaluate(gold,env.isTaggingWithMWSegmentation(),true,env.isDecomposed());  
			}
			else{
				RawText t = createText(textPath,env);
				if(env.isWithLemma()){
					lemmatizer.perform(t).dump();
				}
				else{
					tagger.perform(t).dump();	
				}
								
			}			
		}
		
	}

}
