package fr.upemlv.lgtools.tagging;

import java.util.Arrays;
import java.util.List;



public class LGTaggerParameters {
	
	private boolean isTraining = false;
	private boolean taggingWithMWSegmentation = false;
	private boolean isDecomposedText = false;
	private String[] texts = null;
	private boolean isSegmented = true;
	private String model = null;
	private String model2 = null;
	private String ref = null;
	private String config = null;
	private String devPath = null;
	private String algo = LGTaggerConstants.RPROP_ALGO;
	private List<String> additionalResources = null;	
	

	private boolean eval = false;		
	private int nthreads = 1;
	private double l1 = 0.5;
	private String method = LGTaggerConstants.SINGLE_MODEL_METHOD;
	private String additionalCorpusPath;
	private boolean withLemma = false;
	private boolean isTreebank = false;
	private boolean withProjection = false;
	
	public LGTaggerParameters(){}
	
	/**
	 * A default set of parameters
	 * 
	 * @param modelPath model path
	 * @param taggingWithSeg tagging is performed jointly with MW segmentation
	 * @param isTraining is training
	 */
	
	public LGTaggerParameters(String modelPath,boolean taggingWithSeg,boolean isTraining){
		this.model = modelPath;
		this.taggingWithMWSegmentation = taggingWithSeg;
		this.isTraining = isTraining;
		
	}
	
	
	/**
	 * @return the isTreebank
	 */
	public boolean isTreebank() {
		return isTreebank;
	}

	/**
	 * @param isTreebank the isTreebank to set
	 */
	public void setTreebank(boolean isTreebank) {
		this.isTreebank = isTreebank;
	}

	/**
	 * @return the withProjection
	 */
	public boolean isWithProjection() {
		return withProjection;
	}

	/**
	 * @param withProjection the withProjection to set
	 */
	public void setWithProjection(boolean withProjection) {
		this.withProjection = withProjection;
	}

	/**
	 * @return the isTraining
	 */
	public boolean isTraining() {
		return isTraining;
	}
	/**
	 * @param isTraining the isTraining to set
	 */
	public void setTraining(boolean isTraining) {
		this.isTraining = isTraining;
	}
	/**
	 * @return the taggingWithMWSegmentation
	 */
	public boolean isTaggingWithMWSegmentation() {
		return taggingWithMWSegmentation;
	}
	/**
	 * @param taggingWithMWSegmentation the taggingWithMWSegmentation to set
	 */
	public void setTaggingWithMWSegmentation(boolean taggingWithMWSegmentation) {
		this.taggingWithMWSegmentation = taggingWithMWSegmentation;
	}
	/**
	 * @return the texts
	 */
	public String[] getTexts() {
		return texts;
	}
	/**
	 * @param texts the texts to set
	 */
	public void setTexts(String[] texts) {
		this.texts = texts;
	}
	/**
	 * @return the isSegmented
	 */
	public boolean isSegmented() {
		return isSegmented;
	}
	/**
	 * @param isSegmented the isSegmented to set
	 */
	public void setSegmented(boolean isSegmented) {
		this.isSegmented = isSegmented;
	}
	/**
	 * @return the model
	 */
	public String getModel() {
		return model;
	}
	/**
	 * @param model the model to set
	 */
	public void setModel(String model) {
		this.model = model;
	}
	/**
	 * @return the ref
	 */
	public String getRef() {
		return ref;
	}
	/**
	 * @param ref the ref to set
	 */
	public void setRef(String ref) {
		this.ref = ref;
	}
	/**
	 * @return the config
	 */
	public String getConfig() {
		return config;
	}
	/**
	 * @param config the config to set
	 */
	public void setConfig(String config) {
		this.config = config;
	}
	/**
	 * @return the devPath
	 */
	public String getDevPath() {
		return devPath;
	}
	/**
	 * @param devPath the devPath to set
	 */
	public void setDevPath(String devPath) {
		this.devPath = devPath;
	}
	/**
	 * @return the algo
	 */
	public String getAlgo() {
		return algo;
	}
	/**
	 * @param algo the algo to set
	 */
	public void setAlgo(String algo) {
		this.algo = algo;
	}
	
	/**
	 * @return the eval
	 */
	public boolean isEval() {
		return eval;
	}
	/**
	 * @param eval the eval to set
	 */
	public void setEval(boolean eval) {
		this.eval = eval;
	}
	/**
	 * @return the nthreads
	 */
	public int getNthreads() {
		return nthreads;
	}
	/**
	 * @param nthreads the nthreads to set
	 */
	public void setNthreads(int nthreads) {
		this.nthreads = nthreads;
	}
	/**
	 * @return the l1
	 */
	public double getL1() {
		return l1;
	}
	/**
	 * @param l1 the l1 to set
	 */
	public void setL1(double l1) {
		this.l1 = l1;
	}

	/**
	 * @return the method
	 */
	public String getMethod() {
		return method;
	}

	/**
	 * @param method the method to set
	 */
	public void setMethod(String method) {
		this.method = method;
	}

	/**
	 * @return the model2
	 */
	public String getModel2() {
		return model2;
	}

	/**
	 * @param model2 the model2 to set
	 */
	public void setModel2(String model2) {
		this.model2 = model2;
	}

	/**
	 * @return the additionalCorpusPath
	 */
	public String getAdditionalCorpusPath() {
		return additionalCorpusPath;
	}

	/**
	 * @param additionalCorpusPath the additionalCorpusPath to set
	 */
	public void setAdditionalCorpusPath(String additionalCorpusPath) {
		this.additionalCorpusPath = additionalCorpusPath;
	}

	/**
	 * @return the isDecomposedText
	 */
	public boolean isDecomposedText() {
		return isDecomposedText;
	}

	/**
	 * @param isDecomposedText the isDecomposedText to set
	 */
	public void setDecomposedText(boolean isDecomposedText) {
		this.isDecomposedText = isDecomposedText;
	}

	/**
	 * @return the withLemma
	 */
	public boolean isWithLemma() {
		return withLemma;
	}

	/**
	 * @param withLemma the withLemma to set
	 */
	public void setWithLemma(boolean withLemma) {
		this.withLemma = withLemma;
	}

	/**
	 * @return the additionalResources
	 */
	public List<String> getAdditionalResources() {
		return additionalResources;
	}

	/**
	 * @param resources the additionalResources in the form of a string (resources separated by :)to set
	 */
	public void setAdditionalResources(String resources) {
		if(resources == null){
			this.additionalResources = null;
			return;
		}
		String[] res = resources.split(":");		
		this.additionalResources = Arrays.asList(res);
	}
	
	
}



