package fr.upemlv.lgtools.tagging;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.JSAPResult;

import fr.upemlv.lgtools.tagging.lex.ConfigurableTransform;
import fr.upemlv.lgtools.tagging.lex.LexicalEntry;
import fr.upemlv.lgtools.tagging.lex.LexicalEntryParser;
import fr.upemlv.lgtools.tagging.lex.ListDico;
import fr.upemlv.lgtools.tagging.lex.StandardTagParser;
import fr.upemlv.lgtools.tagging.lex.Tag;
import fr.upemlv.lgtools.tagging.lex.Tagset;

public class LGTaggerTransformDico {
	private final static String SRC = "src";
	private final static String DEST = "dest";
	private final static String DICO = "dic";
	private final static String TRANSFORM = "trans";
	private final static String OUTPUT = "output";
	
	
	private static JSAP createJSAP() throws JSAPException{
		JSAP jsap = new JSAP();
		
		FlaggedOption opt = new FlaggedOption(SRC)
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(true)
        .setShortFlag('s')
        .setLongFlag(SRC)        
        ;
		opt.setRequired(true);
		opt.setHelp("Source tagset");
		jsap.registerParameter(opt);
		
		opt = new FlaggedOption(DEST)
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(true)
        .setShortFlag('d')
		.setLongFlag(DEST);
		opt.setRequired(true);
		opt.setHelp("Destination tagset");
		jsap.registerParameter(opt);
		
		opt = new FlaggedOption(TRANSFORM)
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(true)
        .setShortFlag('t')
		.setLongFlag(TRANSFORM);
		opt.setRequired(true);
		opt.setHelp("Transformation file");
		jsap.registerParameter(opt);
				
		opt = new FlaggedOption(DICO)
        .setStringParser(JSAP.STRING_PARSER)        
        .setShortFlag(JSAP.NO_SHORTFLAG)
        .setShortFlag('f')
		.setLongFlag(DICO);		
		opt.setRequired(true);
		opt.setHelp("Dictionary filename (*.dic)");		
		jsap.registerParameter(opt);
		
		opt = new FlaggedOption(OUTPUT)
        .setStringParser(JSAP.STRING_PARSER)        
        .setShortFlag(JSAP.NO_SHORTFLAG)
        .setShortFlag('o')
		.setLongFlag(OUTPUT);
		opt.setRequired(true);
		opt.setHelp("Output filename (*.dic)");		
		jsap.registerParameter(opt);
		
		return jsap;
	}
	
	
	private static JSAPResult parseArguments(String[] args) throws JSAPException{
		JSAP jsap = createJSAP();
		JSAPResult config = jsap.parse(args);
		if(!config.success()){
			System.err.print("Usage: ");
			System.err.print("java "+LGTaggerMain.class.getCanonicalName()+" ");
            System.err.println(jsap.getUsage());
            System.err.print(jsap.getHelp());
            System.exit(1);
		}
		return config;
	}
	
	public static void main(String[] args) throws IOException, JSAPException, InterruptedException{
	  //dico -> d.dic 
	  
	  JSAPResult arguments = parseArguments(args);
	  Tagset srcTagset = new Tagset(arguments.getString(SRC));
	  Tagset destTagset = new Tagset(arguments.getString(DEST));
	  ConfigurableTransform transform = new ConfigurableTransform(arguments.getString(TRANSFORM), srcTagset, destTagset, new StandardTagParser(), new StandardTagParser(),true);
	  //FileReader reader = new FileReader(Utils.UNITEX_PATH+"/French/Dela/sample-dlcf.dic");
	  System.out.println("Transforming dictionary...");
	  BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(arguments.getString(DICO)), "UTF-16LE"));
	  ListDico dico = new ListDico(reader, srcTagset, LexicalEntryParser.FLEXIBLE_UNITEX_LEXICAL_ENTRY_PARSER, "French",true);  
	  reader.close();
	  BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(
			  new FileOutputStream(arguments.getString(OUTPUT)), "UTF-16LE"));
	  for(LexicalEntry e:dico){
		  for(Tag tag:transform.convert(e.getTag())){
		      writer.write(new LexicalEntry(e.getWord(),e.getLemma(),tag).toString()+"\n");
		  }
	  }
	  writer.close();
	  System.out.println("Done.");
	}
}
