/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging;

import java.io.IOException;

/**
 * Environment for training
 * 
 * @author Matthieu Constant
 *
 */
public class TrainEnvironment extends Environment {

	
	/**
	 * @param arguments
	 * @throws IOException
	 * @throws InterruptedException
	 */
	public TrainEnvironment(LGTaggerArguments arguments) throws IOException,
	InterruptedException {
		super(arguments);
	}

	public TrainEnvironment(LGTaggerParameters parameters) throws IOException,
	InterruptedException {
		super(parameters);
	}
	
	
	
	/**
	 * 
	 * @return the reference corpus pathname
	 */
	
	public String getRefCorpus(){
		return parameters.getRef();
	}
	
	/**
	 * 
	 * @return the L1 penalty for CRF training 
	 */
	
	public double getL1(){
		return parameters.getL1();
	}
    
	/**
	 * 
	 * @return the algorithm used for training 
	 */
	
	public String getAlgo(){
		return parameters.getAlgo();
	}
	
	
	
	/**
	 * 
	 * @return additional reference corpus to be added to reference corpus
	 */
	
	public String getAdditionalCorpusPath(){
	   return parameters.getAdditionalCorpusPath();	
	}
	
	/**
	 * 
	 * @return whether there is lexicon projection on the reference corpus
	 */
	
	public boolean isWithLexiconProjection(){
		return parameters.isWithProjection();
	}
	
	/**
	 * 
	 * @return whether the input reference corpus is a treebank
	 */
	
	public boolean isWithTreebank(){
		return parameters.isTreebank();
	}
	
}
