/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.impl;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.List;

import fr.upemlv.lgtools.text.DefaultTextFst;
import fr.upemlv.lgtools.text.LightMultiFeaturedText;
import fr.upemlv.lgtools.text.LightRawText;
import fr.upemlv.lgtools.text.LightTextFst;
import fr.upemlv.lgtools.text.MultiFeaturedText;
import fr.upemlv.lgtools.text.RawText;
import fr.upemlv.lgtools.text.TaggedText;
import fr.upemlv.lgtools.text.TextFst;
import fr.upemlv.lgtools.text.TextProcessingContext;

public class LGTaggerPreprocessor {	
	private final PreprocessingContext context;

	public LGTaggerPreprocessor(PreprocessingContext context){
		this.context = context;
	}
	

	 public TextFst lexicalAnalysis(RawText text,List<String> resources) throws IOException, InterruptedException{
	    	long id = new Date().getTime();
	    	String TMP = "/tmp/textForUnitex"+id+".txt";	    	
	    	String TFST = "/tmp/tfst"+id+".txt";
	    	TextFst tfst;
	    	if(!resources.isEmpty()){	    		
	    		System.err.println("Lexical analysis with Unitex ...");
	    		text.saveForUnitex(TMP, "utf-8");
	    		//Utils.copy(TMP, "/tmp/xxx.unitex.txt");
	    		context.getUnitexProcess().lookup(TMP, "utf-8", context.getLang(), resources,TFST);			
	    		tfst = new LightTextFst(TFST,context.getTagset(),text);
	    	}
	    	else{
	    		tfst = new DefaultTextFst(text, context.getTagset());
	    	}
		
			new File(TMP).deleteOnExit();
			new File(TFST).deleteOnExit();
			return tfst;	
	    }
	    
	    private  MultiFeaturedText propertyExtraction(RawText text,TextProcessingContext tContext){    	
			System.err.println("Extracting properties...");				
			return new LightMultiFeaturedText(text, tContext, context.getPropertyExtractors());
	    }
	    
	    private  MultiFeaturedText propertyExtraction(TaggedText text,TextProcessingContext tContext){
	    	System.err.println("Extracting properties...");
	    	return new LightMultiFeaturedText(text, tContext, context.getPropertyExtractors());
	    }
	
	    
	    private TextProcessingContext computeTextProcessingContext(RawText text) throws IOException, Exception{
	    	return computeTextProcessingContext(text,null);
	    }
	    
	    private TextProcessingContext computeTextProcessingContext(TaggedText text) throws IOException, Exception{
	    	return computeTextProcessingContext(new LightRawText(text),text);
	    }
	    
	private TextProcessingContext computeTextProcessingContext(RawText text,TaggedText taggedText) throws IOException, Exception{
		//System.err.println(context.getResources());
		TextFst tfst = lexicalAnalysis(text,context.getResources());
    	
    	if((taggedText == null)  && (Utils.containTagExtractor(context.getPropertyExtractors()))){
	    	taggedText = LGTaggerFactory.createDefaultTagger(context.getLang()).perform(text);
	    }
    	return new TextProcessingContext(tfst, taggedText,context.getLang());
	}
	    
	/**
	 * Preprocess a raw text
	 * 
	 * @param text a raw text
	 * @return a multi featured text (tag part is empty)
	 * @throws Exception
	 */
	
	public MultiFeaturedText preprocess(RawText text) throws Exception{		
	    return propertyExtraction(text, computeTextProcessingContext(text));
	}
	
	
	/**
	 * Preprocess a tagged text
	 * 
	 * @param text tagged text
	 * @return a multi featured text 
	 * @throws Exception
	 */
	
	public MultiFeaturedText preprocess(TaggedText text) throws Exception{		
	    return propertyExtraction(text, computeTextProcessingContext(text));
	}
	
	
	
}
