/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */
package fr.upemlv.lgtools.tagging.impl;

import java.util.List;

import fr.upemlv.lgtools.tagging.Environment;
import fr.upemlv.lgtools.tagging.lex.InternalTagset;
import fr.upemlv.lgtools.tagging.nlp.UnitexProcess;
import fr.upemlv.lgtools.text.PropertyExtractor;

/**
 * Context for preprocessing
 * 
 * @author Matthieu Constant
 *
 */

public class PreprocessingContext {
	final private String lang;
	final private UnitexProcess unitexProcess;
	final private List<String> resources;
	final private List<PropertyExtractor> extractors;
	final private InternalTagset tagset;
	
	/**
	 * 
	 * @param env environment
	 */
	
	public PreprocessingContext(Environment env){
		this.lang = env.getLang();
		//env.get
		this.resources = env.getResources();
		this.extractors = env.getPropertyExtractors();
		this.unitexProcess = env.getUnitexProcess();
		this.tagset = env.getTagset();
	}
	
	/**
	 * @param lang
	 * @param resources
	 * @param extractors
	 */
	public PreprocessingContext(String lang, List<String> resources,
			List<PropertyExtractor> extractors,InternalTagset tagset) {
		this.lang = lang;
		this.resources = resources;
		this.extractors = extractors;
		this.unitexProcess = Utils.createUnitexProcess();
		this.tagset = tagset;
	}
	
	
	/**
	 * @return the lang
	 */
	public String getLang() {
		return lang;
	}
	/**
	 * @return the unitexProcess
	 */
	public UnitexProcess getUnitexProcess() {
		return unitexProcess;
	}
	/**
	 * @return the resources
	 */
	public List<String> getResources() {
		return resources;
	}
	
	
	
	/**
	 * @return the extractors
	 */
	public List<PropertyExtractor> getPropertyExtractors() {
		return extractors;
	}

	
	/**
	 * @return the tagset
	 */
	public InternalTagset getTagset() {
		return tagset;
	}

	
}
