/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.impl;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import fr.upemlv.lgtools.tagging.LGTaggerConstants;
import fr.upemlv.lgtools.tagging.nlp.AnnotationScore;
import fr.upemlv.lgtools.tagging.nlp.CRFPPProcess;
import fr.upemlv.lgtools.tagging.nlp.MultiAnnotationScore;
import fr.upemlv.lgtools.tagging.nlp.Tokenizer;
import fr.upemlv.lgtools.tagging.nlp.UnitexProcess;
import fr.upemlv.lgtools.tagging.nlp.WapitiProcess;
import fr.upemlv.lgtools.text.AnnotatedText;
import fr.upemlv.lgtools.text.FullRawText;
import fr.upemlv.lgtools.text.LinguisticElement;
import fr.upemlv.lgtools.text.PropertyConstants;
import fr.upemlv.lgtools.text.PropertyExtractor;
import fr.upemlv.lgtools.text.Sentence;
import fr.upemlv.lgtools.text.TaggedElement;
import fr.upemlv.lgtools.text.TaggedText;

/**
 * @author Matthieu Constant
 *
 */
public class Utils {

	/**
	 * temporary directory path 
	 */

	public final static String TMP = "/tmp"; 


	/*	
	public final static String DATA_PREFIX=System.getenv("DATA_DIR");
	public final static String RESEARCH_PREFIX=System.getenv("RESEARCH_DIR");

	public final static String TAGGING_DIR_PATH = RESEARCH_PREFIX 
	                                          + "/experiments/tagging"; 

	public final static String CORPUS_DIR_PATH = DATA_PREFIX 
    + "/gen/corpus/chunks";
	 */


	/**
	 * provides the path of Unitex directory in the system
	 */
	final static public String UNITEX_PATH=System.getenv("UNITEX_PATH");


	/**
	 * provides the path of directory of Unitex programs in the system
	 */
	final static public String UNITEX_APP_PATH=UNITEX_PATH+"/App";

	/**
	 * Merge separator
	 */

	final static public String MERGE_SEPARATOR="_";

	/**
	 * Tag separator
	 */

	final static public String TAG_SEPARATOR="_";

	/**
	 * Inside Tag separator
	 */

	final static public String INSIDE_TAG_SEPARATOR="+";

	/**
	 * File separator (depending on operating system)
	 */

	final static public String FILE_SEPARATOR = System.getProperty("file.separator");

	/**
	 * UTF-8 encoding
	 */

	final static public String UTF8 = "UTF-8";


	final static private int MINUS_LOG_EPSILON = 50;

	
	/*
	public static String concatenateTags(Collection<LexicalEntry> tags,TagsetTransform transform){
		String res = "";
		HashSet<String> set = new HashSet<String>();

		for(LexicalEntry tag:tags){
			set.add(transform.convert(tag));			
		}
		LinkedList<String> list = new LinkedList<String>(set);
		Collections.sort(list);
		for(String tag:list){
			res += Utils.TAG_SEPARATOR + tag;
		}

		return res.substring(1);
	}
	 */

	
	/**
	 * Normalize a word (e.g. a multiword unit is merged with _)
	 */
	
	public static String normalizeWord(String word,String lang){
		if(word.length() == 1 || word.equals(">>")|| word.equals("<<")){
			return word;
		}
		Tokenizer tokenizer = new Tokenizer(lang);
		List<String> tokens = tokenizer.perform(word);
		String w = "";
		for(String tok:tokens){
		  w += LGTaggerConstants.TOKEN_SEPARATOR+tok;	
		}
		return w.substring(1);
	}
	
	
	
	/**
	 * 
	 * @param s a string
	 * @return whether the string is only composed of letters
	 */


	public static boolean containsOnlyLetters(String s){
		for(char c:s.toCharArray()){
			if(!Character.isLetter(c)){
				return false;
			}
		}
		return true;
	}

	/**
	 * copy a file in another 
	 * 
	 * @param path path of the source file
	 * @param newPath path of the destination file
	 * @throws IOException 
	 */

	public static void copy(String path,String newPath) throws IOException{
		InputStream in = new FileInputStream(path);
		OutputStream out = new FileOutputStream(newPath);

		byte[] buf = new byte[1024];
		int len;
		while ((len = in.read(buf)) > 0) {
			out.write(buf, 0, len);
		}
		in.close();
		out.close();
	}

	/*
	public static LexicalContainer getTextDico(String textFilename,String encoding,String lang,LexicalResourceGroup resource,Tagset tagset) throws IOException, InterruptedException{
		Reader r = UnitexProcess.lookup(textFilename,encoding,lang, resource);
		return new TreeDico(r,tagset,LexicalEntryParser.UNITEX_LEXICAL_ENTRY_PARSER,lang,resource.getId());		
	}
	 */

	static public double logsumexp(double x,double y,boolean flg){
		if (flg) return y;  // init mode
		double vmin = Math.min(x, y);
		double vmax = Math.max(x, y);
		if (vmax > vmin + MINUS_LOG_EPSILON) {
			return vmax;
		} else {
			return vmax + Math.log(Math.exp(vmin - vmax) + 1.0);
		}
	}

	public static<K,V> V get(Map<K,V> map,K key,V val){
		V v = map.get(key);
		if(v == null){
			v = val;
		}
		return v;
	}

	/**
	 * Convert a list of tags into a string (tags are sorted)
	 * 
	 * @param tags list of tags
	 * @return string representation of the list of tags
	 */

	public static String listOfTags2String(List<String> tags){
		Collections.sort(tags);
		String res = "";
		for(String c:tags){
			res += Utils.TAG_SEPARATOR + c;
		}
		return res.substring(1);
	}

	/**
	 * 
	 * @param filename file path
	 * @return whether the file exists
	 */

	public static boolean fileExists(String filename){
		return new File(filename).exists();
	}

	public static UnitexProcess createUnitexProcess(){		
		return new UnitexProcess(LGTaggerConstants.LGTAGGER_UNITEX_PATH);
	}

	public static CRFPPProcess createCRFPPProcess(){		
		return new CRFPPProcess(LGTaggerConstants.LGTAGGER_CRFPP_PATH);
	}

	public static WapitiProcess createWapitiProcess(){		
		return new WapitiProcess(LGTaggerConstants.LGTAGGER_WAPITI_PATH);
	}


	public static <T extends LinguisticElement> FullRawText getDecomposedRawText(AnnotatedText<T> text){
		FullRawText t = new FullRawText();
		for(Sentence<T> s:text){
			Sentence<String> sentence = new Sentence<String>();
			t.addSentence(sentence);
		    for(T e: s){
		    	for(String w:e.getLexicalValue().split(LGTaggerConstants.TOKEN_SEPARATOR)){
			       sentence.add(w);
		    	}
		    }	
		}
		return t;
	}
	
	/**
	 * Counts the number of multiword units in an annotated text
	 * @param text annotated text 
	 */

	public static <T extends LinguisticElement> int multiWordCount(AnnotatedText<T> text){
		int cnt = 0;
		for(Sentence<T> s:text){
			for(T e:s){
				if(e.getLexicalValue().contains(LGTaggerConstants.TOKEN_SEPARATOR)){
					cnt++;
				}
			}
		}
		return cnt;
	}

	
	private static int wordSize(String word){	
		return word.split(LGTaggerConstants.TOKEN_SEPARATOR).length;
	}
	

	/**
	 * Evaluates a tagged text by comparing it with a gold version
	 * 
	 * @param test automatically annotated text
	 * @param gold golden annotated text
	 */

	public static <T extends LinguisticElement> MultiAnnotationScore eval(AnnotatedText<T> test,AnnotatedText<T> gold){
		MultiAnnotationScore score = new MultiAnnotationScore();
		AnnotationScore segScore = new AnnotationScore(gold.size(), test.size());
		AnnotationScore tagScore = new AnnotationScore(gold.size(), test.size());
		AnnotationScore mwSegScore = new AnnotationScore(multiWordCount(gold),multiWordCount(test));
		AnnotationScore mwTagScore = new AnnotationScore(multiWordCount(gold),multiWordCount(test));
		score.addScore("SEG", segScore);
		score.addScore("TAG", tagScore);
		score.addScore("MW_SEG", mwSegScore);
		score.addScore("MW_TAG", mwTagScore);

		Iterator<Sentence<T>> sit = gold.iterator();
		for(Sentence<T> sentence1:test){
			Sentence<T> sentence2 = sit.next();
			int n1 = 0, n2 = 0;			
			Iterator<T> it1 = sentence1.iterator();
			Iterator<T> it2 = sentence2.iterator();
			LinguisticElement e1 = null;
			LinguisticElement e2 = null;
			while(it1.hasNext() && it2.hasNext()){
				if(n1 <= n2){
					e1 = it1.next();					
				}				
				if(n2 <= n1){
					e2 = it2.next();
				}
				int s1 = wordSize(e1.getLexicalValue());
				int s2 = wordSize(e2.getLexicalValue());
				//System.err.println("T1:"+e1.getLexicalValue()+" "+n1+"==>"+s1);
				//System.err.println("T2:"+e2.getLexicalValue()+" "+n2+"=="+s2);
				if(n1 == n2){ // same segment beginning
					if(s1 == s2){ //same segment end
						segScore.incrementCorrect();
						if(s1 > 1){
							//System.err.println(e1.getLexicalValue());
							mwSegScore.incrementCorrect();
						}
						if(e1.getTag().equals(e2.getTag())){ // same tag 
							tagScore.incrementCorrect();
							if(s1 > 1){
								mwTagScore.incrementCorrect();
							}
						}
					}					
				}
				if(n1 == n2){
				  n1 += s1;
				  n2 += s2;
				  if((s1 > 1) || (s2 > 1)){
					  //mwTagScore.incrementTest();
					  //mwSegScore.incrementTest();
				      //mwTagScore.incrementGold();
					  //mwSegScore.incrementGold();
				  }
				}
				else{
					if(n1 < n2){
						n1 += s1;
						//mwTagScore.incrementTest();
						//mwSegScore.incrementTest();
					}
					else{
						n2 += s2;
						//mwTagScore.incrementGold();
						 //mwSegScore.incrementGold();
					}
				}
			}


		}		
		return score;
	}


	
	
	public static Set<String> constructLexiconFromTaggedText(TaggedText t,String codeSuf,UnitexProcess unitex,String dicoPath,boolean onlyMultiword) throws IOException, InterruptedException{
		String DICO = "/tmp/dic-"+new Date().getTime();
		HashSet<String> dico = new HashSet<String>();
		for(Sentence<TaggedElement> s:t){
			for(TaggedElement e:s){
				String word = e.getLexicalValue();
				if(!isMultiword(word) && onlyMultiword){
					continue;
				}
				String w = FullRawText.tokens2string(Arrays.asList(word));
				if(!",".equals(w) && !".".equals(w)&& !"-".equals(w)&& !"'".equals(w)){
				   //dico.add(w+",."+e.getTag().getValue());					
					w = w.replace(",", "\\,");
					w = w.replace("- ", "-");
				   dico.add(w+",."+e.getTag().getValue()+codeSuf);
				  // System.err.println(w+",."+e.getTag().getValue()+"+"+InternalTagset.INT);
				}
			}			
		}
		if(dico.isEmpty()){
			return dico;
		}
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(DICO+".dic"),"UTF-8"));
		for(String s:dico){			
			writer.write(s+"\n");
		}
		writer.close();		
		unitex.convert(DICO+".dic", "UTF-8", "LITTLE-ENDIAN");		
		unitex.compress(DICO+".dic");
		Utils.copy(DICO+".dic", dicoPath+".dic");
		Utils.copy(DICO+".bin", dicoPath+".bin");
		Utils.copy(DICO+".inf", dicoPath+".inf");
		new File(DICO+".dic").delete();
		new File(DICO+".bin").delete();
		new File(DICO+".inf").delete();
		
		return dico;
	}
	
	
	
	
	/**
	 * 
	 */
	
	public static List<String> formatResourcePaths(List<String> resourcesTmp,String lang){    	
    	String SEP = File.separator;
    	List<String> resources = new LinkedList<String>();
    	for(String name:resourcesTmp){
    		File f = new File(name);    		
    		if(f.isAbsolute()){
    			resources.add(name);
    		}
    		else{
    			resources.add(LGTaggerConstants.LGTAGGER_PATH+SEP+lang+SEP+"lex"+SEP+name);
    		}
    	}
    	return resources;
    }
	
	
	public static boolean containTagExtractor(List<PropertyExtractor> extractors){
		for(PropertyExtractor extractor:extractors){
			if(PropertyConstants.TAG.equals(extractor.getPropertyName())){
				return true;
			}		
		}
		return false;
	}
	
	/**
	 * 
	 * @param modelPath model path
	 * @return the path of the config file associated with the model
	 */
	
    public static String getConfigPath(String modelPath){
    	return modelPath+".cfg";
    }
   
	
    public static String getAbsolutePath(String name,String lang, String dir){
    	String SEP = File.separator;
    	File f = new File(name);
    	if(!f.isAbsolute()){
    		name = LGTaggerConstants.LGTAGGER_PATH+SEP+lang+SEP+dir+SEP+name;
    	}
    	return name;
    }
    
    public static String getAbsolutePathForLex(String name,String lang){
    	return getAbsolutePath(name, lang, "lex");
    }
    
    /**
     * 
     * @param word word
     * @return whether the word is a multiword unit
     */
    
    public static boolean isMultiword(String word){
    	return word.contains(LGTaggerConstants.TOKEN_SEPARATOR);
    	
    }
    
}
