/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.lex;

import java.util.Collection;

import fr.upemlv.lgtools.tagging.impl.Utils;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class ConfigurableTransformRule {
	private final Tag source ;
	private final Collection<Tag> destination;
	private final static String IS_CONVERTED_IN = "=";
	
	public ConfigurableTransformRule(String rule,Tagset sourceTagset,Tagset destinationTagset,TagParser sourceParser,TagParser destParser){
		String[] tab = rule.split(IS_CONVERTED_IN);
		if(tab.length != 2){
			throw new RuntimeException("Error: line "+rule+" has wrong format in transform configuration file!");
		}		
		source = TagFactory.createTag(tab[0], sourceTagset, sourceParser);		
		destination = TagFactory.createTags(tab[1], destinationTagset, Utils.TAG_SEPARATOR, destParser);    	 
	}

	/**
	 * apply the rule on a tag
	 * 
	 * @param tag tag to be converted
	 * @return tags corresponding to the tag if the source tag is included in the tag
	 */
	
	public Collection<Tag> apply(Tag tag){    	 
		return tag.contains(source)?destination:null;
	}

}
