/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.lex;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

/**
 * Class implementing a tagset transform by filtering tags by the types of categories
 * 
 * @author Matthieu Constant
 *
 */
public class FilterTagsetTransform implements TagsetTransform {
    final private List<String> types = new LinkedList<String>(); 	
	
	public FilterTagsetTransform(String type,String ...types){
		this.types.add(type);
		for(String t:types){
			this.types.add(t);
		}		
	}
	
	
	/* (non-Javadoc)
	 * @see fr.upemlv.tagging.lex.TagsetTransform#convert(fr.upemlv.tagging.lex.LexicalEntry)
	 */
	@Override
	public Collection<Tag> convert(Tag tag) {
		HashSet<String> set = new HashSet<String>();
		for(String type:types){
		  set.addAll(tag.getCategories(type));
		}
		if(set.isEmpty()){
		   throw new RuntimeException("Tag "+tag+" cannot be converted with respect to defined transform!");	
		} 
		Tag t = new Tag(tag.getTagset());
		for(String c:set){
			t.addCat(c);
		}
		Collection<Tag> coll = new LinkedList<Tag>();
		coll.add(t);
		return coll;
	}

}
