/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.lex;

import java.util.HashSet;
import java.util.Set;

/**
 * Tagset with specific categories (for internal computation)
 * 
 * @author Matthieu Constant
 *
 */


public class InternalTagset extends Tagset {
	/**
	 * Empty lexical unit
	 */
	
	final public static String EMPTY = "$EMPTY$";
		
	
	/**
	 * Beginning of lexical unit
	 */
	final public static String B = "$B$";
	
	/**
	 * inside lexical unit
	 */
	
	final public static String I = "$I$";
	
	/**
	 * Outside lexical unit
	 */
	
	final public static String O = "$O$";
	
	/**
	 * IOB type 
	 */
	
	final public static String IOB = "$IOB$";
	
	

	/**
	 * the tag category indicating that the lexical unit is encoded in the training lexicon
	 */
	
	final public static String INT = "LGINTERNAL";
	
	/**
	 * type of categories about training lexicon 
	 */
	
	final public static String INTERN = "$INTERN$";
	
	
	
	final public static String INSIDE ="INSIDE";
	
	/** category that indicate that a token belongs to a MW */
	final public static String IN = "$IN$";
	
	/**
	 * Constructs an internal tagset from another tagset, that will be augmented with IOB information and information whether a tag found in training lexicon
	 * @param tagset
	 */


	public InternalTagset(Tagset tagset){
		super();
		this.map.putAll(tagset.map);
		this.cats.putAll(tagset.cats);
		this.mandatories.putAll(tagset.mandatories);
		this.mandatoryTypes.addAll(tagset.mandatoryTypes);
		this.print.putAll(tagset.print);
		
		
		//integration of EMPTY tag
		Set<String> set = this.map.get(POS);
		set.add(EMPTY);
		this.cats.put(EMPTY, POS);
		
		set = new HashSet<String>();
		set.add(I);
		set.add(O);
		set.add(B);
		this.cats.put(I, IOB);
		this.cats.put(O, IOB);
		this.cats.put(B, IOB);
		this.mandatories.put(IOB, false);
		this.map.put(IOB,set);
		
	/*set = new HashSet<String>();
		set.add(IS);
		set.add(OS);
		set.add(BS);
		this.cats.put(IS, IOB_SEG);
		this.cats.put(OS, IOB_SEG);
		this.cats.put(BS, IOB_SEG);
		this.mandatories.put(IOB_SEG, false);
		this.map.put(IOB_SEG,set);
		*/
		set = new HashSet<String>();
		set.add(INTERN);		
		this.cats.put(INTERN, INT);
		this.mandatories.put(INT, false);
		this.map.put(INT,set);
		
		set = new HashSet<String>();
		set.add(IN);		
		this.cats.put(IN, INSIDE);
		this.mandatories.put(INSIDE, false);
		this.map.put(INSIDE,set);
		
		
	}
	
	
}
