/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.lex;

import java.util.LinkedList;
import java.util.List;

/**
 * Abstract class representing a lexical container like dictionaries
 * 
 * @author Matthieu Constant
 *
 */
public abstract class LexicalContainer {
	
	
	
	/**
	 * find all lexical entries in a sequence of tokens starting at a given position with a given maximum length
	 * 
	 * @param position position
	 * @param tokens sequence of tokens
	 * @param maxLength maximum length of lexical entries in terms of tokens; if -1, all lengths are permitted
	 * @return all matches starting at position
	 * 
	 */
	public abstract List<LookupMatch> lookup(int position, List<String> tokens,int maxLength);
	
	
	/**
	 * find all lexical entries in a sequence of tokens starting at a given position
	 * 
	 * @param position position
	 * @param tokens sequence of tokens
	 * @return all matches starting at position
	 */
	public List<LookupMatch> lookup(int position, List<String> tokens){
		return lookup(position, tokens, -1);
	}
	
	
	/**
	 * find all matches in a sequence of tokens 
	 * 
	 * @param tokens sequence of tokens
	 * @return all matches
	 */
	
	
	public List<List<LookupMatch>> lookup(List<String> tokens){
		List<List<LookupMatch>> list = new LinkedList<List<LookupMatch>>();
		for(int i = 0 ; i < tokens.size() ; i++){
			list.add(lookup(i,tokens));
		}
		return list;
	}
	
		
	
}
