/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */


package fr.upemlv.lgtools.tagging.lex;


/**
 * class that represents a word tag in the form of a feature structure
 * 
 * 
 * @author Matthieu Constant
 *
 */

public class LexicalEntry implements Comparable<LexicalEntry>{
	private final String word;
	private final String lemma;
	private final Tag tag;

	/**
	 * Constructs a lexical entry from a word, a lemma and a tag
	 * 
	 * @param word word 
	 * @param lemma lemma
	 * @param tag tag
	 */

	public LexicalEntry(String word,String lemma,Tag tag){
		this.word = word;
		this.lemma = lemma;
		this.tag = tag;
	}		

	

	/**
	 * @return the tag
	 */
	public Tag getTag() {
		return tag;
	}



	/**
	 * 
	 * @return lemma
	 */

	public String getLemma(){
		return lemma;
	}




	/**
	 * @return the word
	 */
	public String getWord() {
		return word;
	}


	/**
	 * 
	 * @return an exact copy of the tag
	 */

	public LexicalEntry copy(){
		Tag tag = this.getTag().copy();		
		return new LexicalEntry(getWord(),getLemma(),tag);		
	}
	


	public void addTagFeature(String cat){
		tag.addCat(cat);
	}
	

	@Override
	public String toString(){
		StringBuilder res = new StringBuilder();
		res.append(word+",");
		res.append(lemma+".");
		res.append(tag);
		//res.append("\n");
		return res.toString();
	}



	@Override
	public int compareTo(LexicalEntry o) {		
		return getWord().compareTo(o.getWord());
	}   
}
