package fr.upemlv.lgtools.tagging.lex;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import fr.upemlv.lgtools.tagging.impl.Utils;
import fr.upemlv.lgtools.tagging.nlp.Tokenizer;

public class ListDico extends Dico implements Iterable<LexicalEntry>{
	private final LinkedList<LexicalEntry> list;	
	
    public ListDico(){
    	list = new LinkedList<LexicalEntry>();
    }
	
    /**
     * 
     * @param filename unitex dico (UTF-16LE encoded)
     * @param tagset
     * @param parser
     * @param lang
     * @param hasFlexibleAddition
     * @throws IOException
     * @throws InterruptedException 
     */
    
    public ListDico(String filename,Tagset tagset,LexicalEntryParser parser,String lang,boolean hasFlexibleAddition) throws IOException, InterruptedException{
    	list = new LinkedList<LexicalEntry>();
    	String tmpDico = "/tmp/dico.dic";
    	Utils.copy(filename, tmpDico);
    	Utils.createUnitexProcess().convert(tmpDico, "LITTLE-ENDIAN","UTF-8");
    	InputStreamReader reader = new InputStreamReader(new FileInputStream(tmpDico), "UTF-8");
    	loadDico(reader,tagset,parser,lang,hasFlexibleAddition);
    }
    
	public ListDico(Reader reader,Tagset tagset,LexicalEntryParser parser,String lang,boolean hasFlexibleAddition) throws IOException{	
		list = new LinkedList<LexicalEntry>();
		loadDico(reader,tagset,parser,lang,hasFlexibleAddition);
	}
    
	@Override
	protected void addLexicalEntry(LexicalEntry entry, Tokenizer tokenizer) {		
		list.add(entry);
	}

	@Override
	public List<LookupMatch> lookup(int position, List<String> tokens,
			int maxLength) {
		throw new IllegalArgumentException("Lookup not implemented in ListDico!");
	}

	@Override
	public Iterator<LexicalEntry> iterator() {		
		return list.iterator();
	}

}
