/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.lex;

import java.util.Collection;
import java.util.LinkedList;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class TagFactory {
   
	/**
	 * Creates a tag from a textual value in a tagset
	 * 
	 * @param text textual value
	 * @param tagset tagset
	 * @return the tag
	 */
	
	public static Tag createTag(String text,Tagset tagset,TagParser parser){
	   return parser.parse(text, tagset);
   }
   
   /**
    * Creates a collection of tags from a text
    * 
    * @param text text
    * @param tagset tagset
    * @param tagSep the string separator of tags
    * @param insideTagSep the string separator of tag categories
    * @return tags
    */
	
   public static Collection<Tag> createTags(String text,Tagset tagset,String tagSep,TagParser parser){
	   Collection<Tag> tags = new LinkedList<Tag>();
	   for(String tagValue:text.split(tagSep)){
		   tags.add(createTag(tagValue,tagset,parser));
	   }
	   return tags;
   }
   
}
