/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.lex;

/**
 * @author Matthieu Constant
 *
 * Interface to filter tags
 *
 */
public interface TagFilter {
	
	/**
	 * A default tag filter accepting all tags of lexical entries coming from externel lexicons (i.e. no internal lexicon) 
	 */
	
	public static TagFilter DEFAULT = new TagFilter() {
		
		@Override
		public boolean accept(Tag tag) {
			return tag.getValue(InternalTagset.INT).isEmpty();
		}
	};
	
	
	public static TagFilter SIMPLE = new TagFilter() {
		
		@Override
		public boolean accept(Tag t) {
			//System.err.println(t);
			return DEFAULT.accept(t) && t.getValue(InternalTagset.INSIDE).isEmpty();
		}
	};
	
	public static TagFilter MW = new TagFilter() {

		@Override
		public boolean accept(Tag t) {
			//System.err.println(t);
			//System.err.println("=>"+t.getValue(InternalTagset.INSIDE));
			//System.err.println("=>"+(DEFAULT.accept(t) && !t.getValue(InternalTagset.INSIDE).isEmpty()));
			return DEFAULT.accept(t) && !t.getValue(InternalTagset.INSIDE).isEmpty();
		}
	};

	
	
	/**
	 * 
	 * @param t tag
	 * @return whether the tag is accepted by the filter
	 */
   public boolean accept(Tag t);	
}
