/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.lex;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.List;

import fr.upemlv.lgtools.tagging.impl.Utils;
import fr.upemlv.lgtools.tagging.nlp.Tokenizer;
import fr.upemlv.lgtools.tree.LexicalTree;

/**
 * Implements a dico in the form of a lexical tree
 * 
 * @author Matthieu Constant
 *
 */


public class TreeDico extends Dico {

	private final LexicalTree tree;
	
	 public TreeDico(String filename,Tagset tagset,LexicalEntryParser parser,String lang,boolean hasFlexibleAddition) throws IOException, InterruptedException{
		    tree = new LexicalTree();
	    	String tmpDico = "/tmp/dico.dic";
	    	Utils.copy(filename, tmpDico);
	    	Utils.createUnitexProcess().convert(tmpDico, "LITTLE-ENDIAN","UTF-8");
	    	InputStreamReader reader = new InputStreamReader(new FileInputStream(tmpDico), "UTF-8");
	    	loadDico(reader,tagset,parser,lang,hasFlexibleAddition);
	    }
	
	public TreeDico(Reader reader,Tagset tagset,LexicalEntryParser parser,String lang,String id) throws IOException{
		tree = new LexicalTree();
		loadDico(reader, tagset, parser, lang,false);
	}

	
	@Override
	public List<LookupMatch> lookup(int position,List<String> tokens, int maxLength){
		return tree.lookup(position, tokens,maxLength);
	}


	@Override
	protected void addLexicalEntry(LexicalEntry entry, Tokenizer tokenizer) {
		List<String> wordTokens = tokenizer.perform(entry.getWord());
		tree.addLexicalEntry(wordTokens, entry);
	}
	
}
