/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */


package fr.upemlv.lgtools.tagging.nlp;

import java.text.DecimalFormat;

/**
 * Represents Score
 * 
 * 
 * @author Matthieu Constant
 *
 */

public class AnnotationScore {
    private int goldSize;
    private int testSize;
    private int ok = 0;
    
	/**
	 * @param goldSize
	 * @param testSize
	 */
	public AnnotationScore(int goldSize, int testSize) {
		this.goldSize = goldSize;
		this.testSize = testSize;
	}
    
	/**
	 * 
	 */
	public AnnotationScore() {
		this.goldSize = 0;
		this.testSize = 0;
	}
	
	/**
	 * Increment the count of correct annotations
	 */
	
    public void incrementCorrect(){
    	ok++;
    }
    
    /**
     * increment the count of relevant annotations in gold annotated text
     */
    
    public void incrementGold(){
    	goldSize++;
    }
    
    
    public void updateScore(AnnotationScore sc){
    	this.ok += sc.ok;
    	this.goldSize += sc.goldSize;
    	this.testSize += sc.testSize;
    }
    
    /**
     * increment the count of relevant annotations in test annotated text
     */
    
    public void incrementTest(){
    	testSize++;
    }
    
    
    
    public double getRecall(){
    	return ok/(double)goldSize;
    }
    
    public double getPrecision(){
    	return ok/(double)testSize;
    }
    
    public double fscore(){
    	double recall = getRecall();
    	double precision = getPrecision();
    	return 2*recall*precision/(recall+precision);
    }
	
    /**
     * 
     * @return test size
     */
    
    public int getGoldSize(){
    	return goldSize;
    }
    
    /**
     * 
     * @return gold size
     */
    
    public int getTestSize(){
    	return testSize;
    }
    
    
    @Override
    public String toString(){
    	DecimalFormat df = new DecimalFormat("00.00");
    	String res = "p="+df.format(getPrecision()*100)+";";
    	res += "r="+df.format(getRecall()*100)+";";
    	res += "f="+df.format(fscore()*100)+";";
    	res += "gsize="+getGoldSize()+";";
    	res += "tsize="+getTestSize();
    	return res;
    }
    
}
