/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.nlp;

import java.io.File;
import java.io.IOException;

import fr.upemlv.lgtools.tagging.TrainEnvironment;

/**
 * Use of CRF++ to learn CRF models
 * 
 * @author Matthieu Constant
 *
 */
public class CRFPPProcess extends CRFProcess {
	private final String crfLearn;
	private final String crfTest;
	
	
	public CRFPPProcess(String path){		
		this.crfLearn = path+File.separator+"crf_learn";
		this.crfTest = path+File.separator+"crf_test";
	}
	
  
	@Override
	public ExecInfo learn(String templateFilePath,String trainFilePath,String modelFilePath,TrainEnvironment env,String devPath) throws IOException, InterruptedException{
		//crf_learn -t -f 2 template_file train_file model_file
		String[] command = new String[]{
				crfLearn
				, "-t","2"
				,"-p",""+env.getNthreads()
				,templateFilePath
				,trainFilePath
				,modelFilePath
		};
		return exec(command,false);
	}
  
	@Override
	public void label(String modelFilePath,String textFilePath,String outfilePath) throws IOException, InterruptedException{		
		String[] command = new String[]{
				crfTest
				,"-o",outfilePath
				, "-m",""+modelFilePath
				,textFilePath
		};
		exec(command,false);
	}


	@Override
	public String getToolName() {		
		return "CRF++";
	}

}
