/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.nlp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import fr.upemlv.lgtools.tagging.TrainEnvironment;

/**
 * Process to train a CRF model
 * And use it for labelling word sequences
 * 
 * @author Matthieu Constant
 *
 */
public abstract class CRFProcess {

	public abstract String getToolName();
	public abstract ExecInfo learn(String templateFilePath,String trainFilePath,String modelFilePath,TrainEnvironment env,String devPath) throws IOException, InterruptedException;
	public abstract void label(String modelFilePath,String textFilePath,String outfilePath) throws IOException, InterruptedException;

	
	private double getTime(double time,String line){
		Pattern pattern = Pattern.compile("time=[^/]+/([^s]+)s");
		Matcher m = pattern.matcher(line);
		if(m.find()){
			return Double.parseDouble(m.group(1));
		}
		return time;
	}
	
	
	private int getLabelCount(int labelCount,String line){
		Pattern pattern = Pattern.compile("nb labels:[^0-9]+([0-9]+)");
		Matcher m = pattern.matcher(line);
		if(m.find()){
			return Integer.parseInt(m.group(1));
		}
		return labelCount;
	}
	
	private int getFeatureCount(int featureCount,String line){
		Pattern pattern = Pattern.compile("nb features:[^0-9]+([0-9]+)");
		Matcher m = pattern.matcher(line);
		if(m.find()){
			return Integer.parseInt(m.group(1));
		}
		return featureCount;
	}
	
	
	protected ExecInfo exec(String[] command,boolean withErrorStream) throws IOException, InterruptedException{
		double time = 0; int labelCount = 0,featureCount = 0;
		Process process = Runtime.getRuntime().exec(command);
		BufferedReader stdErr;
		if(!withErrorStream){
		  stdErr = new BufferedReader(new InputStreamReader(process.getInputStream()));
		}
		else{
			stdErr = new BufferedReader(new InputStreamReader(process.getErrorStream()));
		}
		String line;
        while ((line = stdErr.readLine()) != null) {
        	time = getTime(time,line);
        	labelCount = getLabelCount(labelCount,line);
        	featureCount = getFeatureCount(featureCount, line);
           System.err.println(line);
        }
        stdErr.close();	
			
		int exit = process.waitFor();
		
		if(exit != 0){
			throw new RuntimeException(getToolName()+" process "+command[0]+" finished with unexpected error!");
		}
		return new ExecInfo(time, featureCount, labelCount);
	}
}
