/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.nlp;

import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 
 * Represents posttokenizers included in tokenizer if regexp not enough
 * 
 * @author Matthieu Constant
 *
 */
public enum PostTokenizer {
	
	
    WESTERN_LANGUAGE{

    	final private Pattern pattern = Pattern.compile("\\p{P}+");    	
    	
		@Override
		public List<String> perform(String tok) {
			List<String> res = new LinkedList<String>();
			Matcher matcher = pattern.matcher(tok);
			if(matcher.matches() && !tok.equals("...")){
			   for(char c:tok.toCharArray()){
				   res.add(c+"");				  
			   }	
			}
			else{
			  res.add(tok);
			}
			return res;
		}
    	
    },
    DEFAULT{

		@Override
		public List<String> perform(String tok) {
			List<String> res = new LinkedList<String>();
			res.add(tok);
			return res;
		}
    	
    }
    
    ;
	
    
    
	public abstract List<String> perform(String tok); 
}
