/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.nlp;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import fr.upemlv.lgtools.tagging.TrainEnvironment;

/**
 *  Use of Wapiti to learn CRF models
 * 
 * @author Matthieu Constant
 *
 */

public class WapitiProcess extends CRFProcess{
    private final String wapiti;
    
    public WapitiProcess(String dirPath){
    	this.wapiti = dirPath+File.separator+"wapiti";
    }

	@Override
	public String getToolName() {
		return "Wapiti";
	}

	@Override
	public void label(String modelFilePath, String textFilePath,
			String outfilePath) throws IOException, InterruptedException {
		String[] command = new String[]{
				wapiti
				,"label"
				,"-m",modelFilePath
				,textFilePath
				,outfilePath
		};
		exec(command,true);
		
	}

	@Override
	public ExecInfo learn(String templateFilePath, String trainFilePath,			
			String modelFilePath, TrainEnvironment env,String devPath) throws IOException,
			InterruptedException {
		String algo = env.getAlgo();
		if(!"l-bfgs".equals(algo)){
			algo = "rprop";
		}
		String[] command = new String[]{
				wapiti
				,"train"
				,"-1",""+env.getL1()
				//,"-i",""+30
				,"--algo",algo
				//,"--nthread",""+nthreads				
				,"-p",templateFilePath				
				
		};
		List<String> tmp = new LinkedList<String>(Arrays.asList(command));
		if(devPath != null){		   
		   tmp.add("--devel");
		   tmp.add(devPath);
		}
		tmp.add(trainFilePath);
		tmp.add(modelFilePath);
		command = tmp.toArray(new String[]{});
		return exec(command,true);
				
	}
	
    
}
