/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.validation;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.List;

import fr.upemlv.lgtools.tagging.LGTaggerParameters;
import fr.upemlv.lgtools.tagging.TrainEnvironment;
import fr.upemlv.lgtools.tagging.impl.LGTaggerLearner;
import fr.upemlv.lgtools.tagging.impl.LGTaggerPreprocessor;
import fr.upemlv.lgtools.tagging.impl.PreprocessingContext;
import fr.upemlv.lgtools.tagging.impl.Utils;
import fr.upemlv.lgtools.tagging.nlp.ExecInfo;

/**
 * @author Matthieu Constant
 *
 */
public class ValidationModel {
	private final String name;
	private final String algo;
	private final String additionnalCorpus;
	private final List<String> templates;
	private final List<String> resources;
	private final LGTaggerValidateConfig config;
	private ExecInfo trainingInfo;
	private final boolean isSeg;
	
	/**
	 * @param name
	 * @param algo
	 * @param resources
	 * @param isSeg
	 * @param templates
	 */
	public ValidationModel(String name, String algo,
			boolean isSeg, List<String> resources, List<String> templates,String additionalCorpus,LGTaggerValidateConfig config) {
		this.name = name;
		this.algo = algo;
		this.isSeg = isSeg;
		this.templates = templates;
		this.resources = resources;
		this.config = config;
		this.additionnalCorpus = additionalCorpus;
	}

	
		
	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}



	
	
	public void generateConfig(String configPath) throws IOException{
		OutputStreamWriter out = new OutputStreamWriter(new FileOutputStream(configPath),"UTF-8");
		out.write("LANG:\n"+config.getLanguage()+"\n\n");
		out.write("TAGSET:\n"+config.getTagsetFilename()+"\n\n");
		out.write("RESOURCES:\n");
		for(String r:resources){
			out.write(r+"\n");
		}
		out.write("\nTEMPLATES:\n");
		for(String t:templates){
			out.write(t+"\n");
		}
		out.close();
	}

	public LGTaggerLearner createLearner(String configPath,String modelPath,String addCorpus) throws IOException, InterruptedException{
		LGTaggerParameters p = new LGTaggerParameters();
		p.setAlgo(algo);
		p.setTaggingWithMWSegmentation(isSeg);
		p.setConfig(configPath);
		p.setModel(modelPath);
		p.setAdditionalCorpusPath(addCorpus);
		TrainEnvironment env = new TrainEnvironment(p);
		LGTaggerPreprocessor preprocessor = new LGTaggerPreprocessor(new PreprocessingContext(env));
		return new LGTaggerLearner(env, preprocessor);
	}
	
	
	public String getPath(String dirPath,int part){
		return dirPath+File.separator+name+File.separator+name+"-"+part;
	}
	
	public void setInfo(ExecInfo info){
		this.trainingInfo = info;
	}
	
	public ExecInfo getInfo(){
		return trainingInfo;
	}



	/**
	 * @return the additionnalCorpus
	 */
	public String getAdditionnalCorpus() {		
		return additionnalCorpus == null?null:Utils.getAbsolutePath(additionnalCorpus, config.getLanguage(), "corpus");		
	}
	
	
	
}
