/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tagging.validation;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import fr.upemlv.lgtools.tagging.LGTaggerConstants;
import fr.upemlv.lgtools.tagging.LGTaggerParameters;
import fr.upemlv.lgtools.tagging.TaggingEnvironment;
import fr.upemlv.lgtools.tagging.impl.LGTagger;
import fr.upemlv.lgtools.tagging.impl.LGTaggerFactory;

/**
 * @author Matthieu Constant
 *
 */
public class ValidationTest {
	private final List<String> resources;
	private final String name;
	private final String method;
	private final ValidationModel segmodel;
	private final ValidationModel tagmodel;
	private final boolean isSeg;
	
	/**
	 * @param name
	 * @param method
	 * @param segmodel
	 * @param tagmodel
	 * @param isSeg
	 */
	public ValidationTest(String name, String method, ValidationModel segmodel,
			ValidationModel tagmodel, boolean isSeg,List<String> resources) {
		this.name = name;
		this.method = method;
		this.segmodel = segmodel;
		this.tagmodel = tagmodel;
		this.isSeg = isSeg;
		this.resources = resources != null?resources:new LinkedList<String>();
	}
	
	
	public LGTagger createTestTagger(String dir,int i) throws IOException, InterruptedException{
		
		LGTaggerParameters p = null;
		if(method.equals(LGTaggerConstants.BASELINE_METHOD)){
			p = new LGTaggerParameters(tagmodel.getPath(dir,i), isSeg, false);
			p.setMethod(LGTaggerConstants.BASELINE_METHOD);
			if(resources.isEmpty()){ // by default, training lexicon
				resources.add(tagmodel.getPath(dir, i)+".bin");
			}
		}
		if(method.equals(LGTaggerConstants.SINGLE_MODEL_METHOD)){			
			p = new LGTaggerParameters(segmodel.getPath(dir,i), isSeg, false);
			p.setMethod(LGTaggerConstants.SINGLE_MODEL_METHOD);
		}
		if(method.equals(LGTaggerConstants.DOUBLE_MODEL_METHOD)){
			p = new LGTaggerParameters(segmodel.getPath(dir,i), isSeg, false);
			p.setMethod(LGTaggerConstants.DOUBLE_MODEL_METHOD);
			p.setModel2(tagmodel.getPath(dir,i));
		}
		return LGTaggerFactory.createTagger(new TaggingEnvironment(p),resources);	
	}


	/**
	 * @return the isSeg
	 */
	public boolean isSeg() {
		return isSeg;
	}


	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	
	
	
}
