/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.util.LinkedList;
import java.util.List;


/**
 * Describes a feature template as a list of template feature 
 * 
 * @author Matthieu Constant
 *
 */
public class FeatureTemplate {
	private final List<TemplateFeature> features = new LinkedList<TemplateFeature>();
	
    
	/**
	 * Add feature in the template
	 * 
	 * @param f feature
	 * @return true if success; false otherwise
	 */
	
	public boolean addFeature(TemplateFeature f){
		return features.add(f);
	}


	/**
	 * @return the features
	 */
	public List<TemplateFeature> getFeatures() {
		return features;
	}

	public FeatureTemplate replace(String type, String newF){
		FeatureTemplate template = new FeatureTemplate();
		boolean done = false;
        for(TemplateFeature f:getFeatures()){
        	String prop = f.getPropertyType();
        	if(!done && type.equals(prop)){
        		prop = newF;
        		done = true;
        	}
        	TemplateFeature nf = new TemplateFeature(prop, f.getArguments(), f.getRelativePosition());
        	template.addFeature(nf);
        }
				
		return template;
	}
	
	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "FeatureTemplate [features=" + features + "]";
	}
	
	
	
	
}
