/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.LinkedList;
import java.util.List;

import fr.upemlv.lgtools.tagging.LGTaggerConstants;
import fr.upemlv.lgtools.tagging.nlp.Tokenizer;
import fr.upemlv.lgtools.tagging.nlp.UnitexProcess;

/**
 * Represents a raw text segmented in sentences. 
 * Each sentence is a sequence of tokens.
 * 
 * @author Matthieu Constant
 *
 */
public class FullRawText extends RawText{
	
	//final private List<List<String>> sentences = new LinkedList<List<String>>();
	
	/**
	 * Constructs an empty raw text
	 */
	public FullRawText(){		
	}
	
	
	/**
	 * 
	 * Constructs a Text from a filename
	 * 
	 * @param textFilename text filename
	 * @param encoding text encoding
	 * @param lang text language
	 * @param withSegmentation indicates whether the text has to be segmented in sentences and tokens
	 * @throws InterruptedException 
	 * @throws IOException 
	 */
	public FullRawText(String textFilename,String encoding,String lang,boolean withSegmentation) throws IOException, InterruptedException{
		if(withSegmentation){
			loadTextWithSegmentation(textFilename, encoding, lang);
		}
		else{
			loadTextAlreadySegmented(textFilename, encoding);
		}
	}
	
	private void loadTextWithSegmentation(String textFilename,String encoding,String lang) throws IOException, InterruptedException{
		Tokenizer tokenizer = new Tokenizer(lang);
		UnitexProcess unitex = new UnitexProcess(LGTaggerConstants.LGTAGGER_UNITEX_PATH);		
		for(String sentence:loadSentences(unitex.sentenceSegmenter(textFilename, encoding, lang))){
			List<String> s = tokenizer.perform(sentence);						
			if(s.size() != 0){
				//System.err.println(s);
		     addSentence(new Sentence<String>(s));
			}
		}
	}
	
	/*
	 * load a text already segmented. A token per line; an empty line is the separator between two sentences 
	 */
	
	private void loadTextAlreadySegmented(String textFilename,String encoding) throws IOException{
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(textFilename), encoding));
		String line;
		List<String> currentSentence = new LinkedList<String>();
		boolean inside = false;
		while ((line = reader.readLine()) != null) {
			if (line.equals("")){
				addSentence(new Sentence<String>(currentSentence));
				currentSentence = new LinkedList<String>();
				inside = false;
				continue;
			}
			currentSentence.add(line);
			inside = true;
		}
		if(inside){
			addSentence(new Sentence<String>(currentSentence));
		}
		reader.close();
	}
	
	private List<String> loadSentences(Reader reader) throws IOException{
		List<String> list = new LinkedList<String>();
		BufferedReader breader = new BufferedReader(reader);
		String line;
		while((line = breader.readLine()) != null){
			String sent[] =line.split("\\{S\\}");
			for(String s:sent){
				if(!s.equals("")){
					list.add(s.trim());	
				}
			}
		}
		breader.close();
		return list;
	}

	
}
