/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import fr.upemlv.lgtools.tagging.lex.InternalTagset;

/**
 * Represents a full text fst (very heavy) entirely read from a stream
 * 
 * @author Matthieu Constant
 *
 */

public class FullTextFst extends TextFst {

    private final List<SentenceFst> sentences;	 
	private final int wordCount;
    
    public FullTextFst(Reader reader,InternalTagset tagset,RawText text) throws IOException{
    	sentences = new LinkedList<SentenceFst>();
    	BufferedReader r = new BufferedReader(reader);
		r.read();
		//int sentenceCount = 
		Integer.parseInt(r.readLine()); // sentence count
		//System.out.println(sentenceCount);
		for(Sentence<String> sentence:text){			
			addSentence(new SentenceFst(r,tagset,sentence));
		}
		wordCount = countWords();
    }
    
    private int countWords(){
    	int i = 0;
    	for(SentenceFst s:sentences){
    		i += s.getTokens().size();
    	}
    	return i;
    }
    
        
    
	@Override
	public Iterator<SentenceFst> iterator() {
		return sentences.iterator();
	}
	
	public void addSentence(SentenceFst sentence){
		sentences.add(sentence);
	}




	@Override
	int getSentenceCount() {
		return sentences.size();
	}




	@Override
	int getWordCount() {
		return wordCount;
	}

}
