/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.util.Collection;
import java.util.Iterator;

/**
 * Light multi featured Text (not stored in memory)
 * 
 * @author Matthieu Constant
 *
 */

public class LightMultiFeaturedText extends MultiFeaturedText{
  private final RawText text;
  private final TaggedText ttext;
  private final TextProcessingContext tContext;
  private final int size;
  private final int sentenceCnt;
  private final Collection<PropertyExtractor> propertyExtractors;
  
  
  /** constructs a multi-featured text from a raw text by applying a collection of propertyExtractors
   * with an empty tag for each token
   * 
   * @param t raw text
   * @param propertyExtractors collection of property extractors
   */
  
  public LightMultiFeaturedText(RawText t, TextProcessingContext tContext,Collection<PropertyExtractor> propertyExtractors){
	  addFeatureNames(propertyExtractors);
	  this.text = t;
	  this.tContext = tContext;
	  this.ttext = null;
	  this.sentenceCnt = t.sentenceCount();
	  this.size = t.size();
	  this.propertyExtractors = propertyExtractors;
	  /*
	  for(Sentence<String> sentence:t){
		  SentenceFst sfst = iterator.next();
		  addSentence(computeRawSentence(sentence,sfst,propertyExtractors));
	  }
	  */
  }
  
  /**
   * constructs a multi-featured text from a tagged text by applying a collection of propertyExtractors
   * 
   * @param t tagged text
   * @param propertyExtractors collection of property extractors
   */
  
  public LightMultiFeaturedText(TaggedText t,TextProcessingContext tContext, Collection<PropertyExtractor> propertyExtractors){
	  addFeatureNames(propertyExtractors);	  
	  this.text = null;
	  this.tContext = tContext;
	  this.ttext = t;
	  this.sentenceCnt = t.sentenceCount();
	  this.size = t.size();
	  this.propertyExtractors = propertyExtractors;
	  /*
	  Iterator<SentenceFst> iterator = tfst.iterator();
	  for(Sentence<TaggedElement> sentence:t){		 
		  SentenceFst sfst = iterator.next();		 
		  addSentence(computeTaggedSentence(sentence,sfst,propertyExtractors));
	  }*/
  }

/* (non-Javadoc)
 * @see fr.upemlv.tagging.text.Text#iterator()
 */
@Override
public Iterator<Sentence<MultiFeaturedElement>> iterator() {
	return new Iterator<Sentence<MultiFeaturedElement>>() {
		Iterator<SentenceProcessingContext> iterator = tContext.iterator();
		Iterator<Sentence<String>> rit = text == null?null:text.iterator();
		Iterator<Sentence<TaggedElement>> tit = ttext == null?null:ttext.iterator();
		
		@Override
		public boolean hasNext() {
			return iterator.hasNext();
		}

		@Override
		public Sentence<MultiFeaturedElement> next() {
			//System.err.println(iterator);
			SentenceProcessingContext sContext = iterator.next();
			
			if(tit == null){ // case: RawText				
				Sentence<String> sentence = rit.next();
				//System.err.println(sentence.getSequence());
				return computeRawSentence(sentence,sContext,propertyExtractors);
			}
			else{ //case: TaggedText
				Sentence<TaggedElement> sentence = tit.next();
				return computeTaggedSentence(sentence,sContext,propertyExtractors);
			}			
		}

		@Override
		public void remove() {		
		}
		
	};
}

/* (non-Javadoc)
 * @see fr.upemlv.tagging.text.Text#sentenceCount()
 */
@Override
public int sentenceCount() {
	return sentenceCnt;
}

/* (non-Javadoc)
 * @see fr.upemlv.tagging.text.Text#size()
 */
@Override
public int size() {
	return size;
}
  
  
  
  
 
}
