/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.util.Iterator;
import java.util.LinkedList;

import fr.upemlv.lgtools.tagging.lex.LexicalEntry;
import fr.upemlv.lgtools.tagging.lex.Tagset;
import fr.upemlv.lgtools.wfst.Path;
import fr.upemlv.lgtools.wfst.Transition;

public class LightRawTextFromTFST extends RawText{
	final TextFst text;
	final Tagset tagset;
	
	
	public LightRawTextFromTFST(TextFst text,Tagset tagset){
		this.text = text;
		this.tagset = tagset;
	}
	
	/* (non-Javadoc)
	 * @see fr.upemlv.tagging.text.Text#iterator()
	 */
	@Override
	public Iterator<Sentence<String>> iterator() {
		return  new Iterator<Sentence<String>>() {  
			Iterator<SentenceFst> it = text.iterator();
			@Override
			public boolean hasNext() {				
				return it.hasNext();
			}

			@Override
			public Sentence<String> next() {
				SentenceFst sfst = it.next();
				
				/*for(Transition<LexicalEntry> t : sfst.getTransitions()){
					System.err.println(t);
				}*/
				
				Path<LexicalEntry> p = sfst.getShortestPath();
				LinkedList<TaggedElement> sequence = new LinkedList<TaggedElement>();
				Sentence<TaggedElement> sentence = new Sentence<TaggedElement>(sequence);
				for(Transition<LexicalEntry> t:p.getTransitions()){
					//System.err.println("==>"+t.getInput().getWord());
					sequence.add(new TaggedElement(t.getInput().getWord(), t.getInput().getTag()));
				}
				sentence = TaggedTextFactory.recomposeTaggedTextSentence(sentence, tagset,false,false);
				LinkedList<String> seq = new LinkedList<String>();
				for(TaggedElement e:sentence){
					seq.add(e.getLexicalValue());
				}
				return new Sentence<String>(seq);
			}

			@Override
			public void remove() {
			}
			
		};
	}

	
	@Override
	public int sentenceCount() {
		return text.getSentenceCount();
	}

	
	@Override
	public int size() {
		return text.getWordCount();
	}

	
	
}
