/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import fr.upemlv.lgtools.parsing.ParseTree;
import fr.upemlv.lgtools.parsing.Treebank;
import fr.upemlv.lgtools.tagging.lex.Tag;
import fr.upemlv.lgtools.tagging.lex.Tagset;

/**
 * Represents a tagged text loaded from a treebank
 * 
 * @author Matthieu Constant
 *
 */

public class LightTaggedText extends TaggedText{
	final private Treebank treebank;

	/**
	 * @param tagset
	 * @param treebank
	 */
	public LightTaggedText(Tagset tagset, Treebank treebank) {
		super(tagset);
		this.treebank = treebank;
	}

	/* (non-Javadoc)
	 * @see fr.upemlv.lgtools.text.Text#iterator()
	 */
	@Override
	public Iterator<Sentence<TaggedElement>> iterator() {
		return new Iterator<Sentence<TaggedElement>>() {
			Iterator<ParseTree> it = treebank.iterator();
			
			@Override
			public boolean hasNext() {				
				return it.hasNext();
			}

			@Override
			public Sentence<TaggedElement> next() {
				ParseTree tree = it.next();
				List<TaggedElement> seq = new LinkedList<TaggedElement>();
				List<String> words = tree.getTerminalSymbols();
				List<String> tags = tree.getPreTerminalSymbols();
				Iterator<String> it = tags.iterator();
				for(String w:words){
					String cat = it.next();
					seq.add(new TaggedElement(w, new Tag(cat, tagset)));
				}
				return new Sentence<TaggedElement>(seq);
			}

			@Override
			public void remove() {				
			}
			
		};
	}
	
	
	
}
