/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Iterator;

import fr.upemlv.lgtools.tagging.lex.InternalTagset;

/**
 * Represents a light text fst (not stored in memory) read from a stream 
 * 
 * @author Matthieu Constant
 *
 */

public class LightTextFst extends TextFst {
    private final String tfstPath;
    private final RawText text;	
    private final InternalTagset tagset;
	
    public LightTextFst(String tfstPath,InternalTagset tagset,RawText text) throws IOException{
    	this.tfstPath = tfstPath;		
		this.text = text;		
		this.tagset = tagset;		
    }
    
    
    
    
    
	@Override
	public Iterator<SentenceFst> iterator() {		
		return new Iterator<SentenceFst>() {
			
			Iterator<Sentence<String>> it = text.iterator();
			BufferedReader r = null;
			
			@Override
			public void remove() {				
			}
			
			@Override
			public SentenceFst next() {				
				try {
					if(r == null){
						r = new BufferedReader(new InputStreamReader(new FileInputStream(tfstPath),"UTF-16LE"));
						r.read();		
						Integer.parseInt(r.readLine()); // sentence count	
					}
					Sentence<String> s = it.next();
					//System.err.println(s.getSequence());
					if(s.size() == 0){
						throw new RuntimeException("There is an empty sentence!!");
					}
					return new SentenceFst(r,tagset,s);
				} catch (IOException e) {
					throw new RuntimeException(e);
				}
			}
			
			@Override
			public boolean hasNext() {
				if(!it.hasNext() && r!=null){
					try {
						r.close();
					} catch (IOException e) {
						throw new RuntimeException(e);
					}
					r = null;
				}
				return it.hasNext();
			}
		};
	}





	@Override
	int getSentenceCount() {
		return text.sentenceCount();
	}



	@Override
	int getWordCount() {
		return text.size();
	}
	
	

}
