/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.util.List;
import java.util.Map;

import fr.upemlv.lgtools.tagging.lex.Tag;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class MultiFeaturedElement implements LinguisticElement{

	private Tag tag;
	private final Map<String,Integer> featureNameIds;
	private final List<String> features;
	
	public MultiFeaturedElement(Map<String,Integer> featureNameIds,List<String> features){
		this.featureNameIds = featureNameIds;
		this.features = features;
	}

	
	
	
	/**
	 * 
	 * @param featureName feature name
	 * @return the value of the feature
	 */
	
	public String get(String featureName){
		Integer id = featureNameIds.get(featureName);
		if(id == null){
			throw new RuntimeException("Error: feature "+featureName+" does not exist!");
		}
		return features.get(id);
	} 
	
	
	
	/**
	 * @param tag the tag to set
	 */
	public void setTag(Tag tag) {
		this.tag = tag;
	}

	
	@Override
	public String getLexicalValue() {
		return get(PropertyConstants.TOKEN);
	}

	@Override
	public Tag getTag() {
		return tag;
	}
    
}
