/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * 
 * @author Matthieu Constant
 *
 */

public class MultiFeaturedText extends AnnotatedText<MultiFeaturedElement>{
  protected final Map<String,Integer> featureNameIds = new HashMap<String, Integer>();
  protected final List<String> featureNames = new LinkedList<String>();
  
  

  
  protected Sentence<MultiFeaturedElement> computeRawSentence(Sentence<String> sentence,SentenceProcessingContext sContext,Collection<PropertyExtractor> propertyExtractors){

	  List<MultiFeaturedElement> sequence = new ArrayList<MultiFeaturedElement>(sentence.size());	 
	  for(int pos = 0 ; pos < sentence.size() ; pos++){
		  List<String> featureValues = new ArrayList<String>(propertyExtractors.size());
		  for(PropertyExtractor extractor:propertyExtractors){
			featureValues.add(extractor.perform(sContext, pos));
		  }
		  sequence.add(new MultiFeaturedElement(featureNameIds,featureValues));
	  }	  
	  return new Sentence<MultiFeaturedElement>(sequence);  
  }
    
  
  protected Sentence<MultiFeaturedElement> computeTaggedSentence(Sentence<TaggedElement> sentence,SentenceProcessingContext sContext,Collection<PropertyExtractor> propertyExtractors){
	  List<MultiFeaturedElement> sequence = new ArrayList<MultiFeaturedElement>(sentence.size());
	  //Sentence<String> tokenSentence = AnnotatedText.taggedToRawSentence(sentence);	  
	  for(int pos = 0 ; pos < sentence.size() ; pos++){
		  List<String> featureValues = new ArrayList<String>(propertyExtractors.size());		  
		  for(PropertyExtractor extractor:propertyExtractors){
			//System.err.print("##"+extractor.perform(sfst, pos));
			featureValues.add(extractor.perform(sContext, pos));
		  }
		 
		  MultiFeaturedElement mfe = new MultiFeaturedElement(featureNameIds,featureValues);
		  mfe.setTag(sentence.get(pos).getTag());
		  //System.err.println("##"+sentence.get(pos).getTag());
		  sequence.add(mfe);
	  }	  
	  return new Sentence<MultiFeaturedElement>(sequence);  
  }
  
  
  private void addFeatureName(String name){
	  Integer id = featureNameIds.get(name);
	  if(id == null){
		  featureNameIds.put(name,featureNameIds.size());
		  featureNames.add(name);
	  }
  }
  
  protected void addFeatureNames(Collection<PropertyExtractor> propertyExtractors){
	for(PropertyExtractor extractor:propertyExtractors){
		addFeatureName(extractor.getPropertyName());
	}  
  }
  
  public int featureCount(){
	  return featureNames.size() + 1;
  }
  

@Override  
  public void save(String filename,String encoding) throws IOException{
	  OutputStreamWriter out = new OutputStreamWriter(new FileOutputStream(filename),encoding);
	  write(out);
	  out.close();
	  
  }

/**
 * Print multi-featured text on standard output in UTF-8
 */

private void write(OutputStreamWriter out) throws IOException{		
	  for(Sentence<MultiFeaturedElement> sentence:this){
			for(MultiFeaturedElement mfe:sentence){
			    for(String featureName:this.featureNames){
			    	out.append(mfe.get(featureName)+"\t");
			    }
			    if(mfe.getTag() != null){
			      out.append(mfe.getTag()+"\n");
			    }
			    else{
			    	out.append("\n");
			    }			    
			}
			out.append("\n");
			sentence = null;
			out.flush();
		}	  
}

  public void dump() throws UnsupportedEncodingException, IOException{
	  dump("UTF-8");
  }

  
  /**
   * Print multi-featured text on standard output
   * @param encoding text encoding
 * @throws IOException 
 * @throws UnsupportedEncodingException 
   */
  public void dump(String encoding) throws UnsupportedEncodingException, IOException{
	  write(new OutputStreamWriter(System.out,encoding));	  
  }
 
}
