/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */


package fr.upemlv.lgtools.text;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.List;

import fr.upemlv.lgtools.tagging.LGTaggerConstants;

/**
 *  
 *   Abstract raw text
 *   
 * @author Matthieu Constant
 *
 */

public abstract class RawText extends Text<String>{
	
	/**
	 * transform a sequence of tokens in a sequence of characters
	 * 
	 * @param tokens sequence of tokens
	 * @return corresponding string
	 */
	
	
	public static String tokens2string(List<String> tokens){
		String s = "";
		for(String token:tokens){			
			token = token.replaceAll(LGTaggerConstants.TOKEN_SEPARATOR, " ");							
			s+= " " + token;
		}
		s = s.trim();
		s = s.replaceAll(" - ", "-");
		s = s.replaceAll(" \\.", ".");
		s = s.replaceAll(" ,", ",");
		s = s.replaceAll(", ([0-9])", ",$1");
		s = s.replaceAll("' ", "'");
		//String r = "0, 5";
		//r = r.replaceAll(",([0-9])", ",$1");
		return s;
	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		for(Sentence<String> sentence:this){
			for(String w:sentence.getSequence()){
			    sb.append(w+"\n");	
			}
			sb.append("\n");
		}
		
		return sb.toString();
	}

	@Override
	public void save(String filename, String encoding) throws IOException {
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),encoding));
		for(Sentence<String> sentence:this){			
			writer.append(tokens2string(sentence.getSequence()));
			writer.newLine();
		}
		writer.close();		
	}	
	
	public void saveAsIs(String filename) throws IOException{
		saveAsIs(filename, "UTF-8");
	}
	public void saveAsIs(String filename,String encoding) throws IOException{
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),encoding));
		for(Sentence<String> sentence:this){
			for(String w:sentence){
				writer.append(w);
				writer.newLine();
			}
			writer.newLine();			
		}
		writer.close();
	}
	
	public void saveSegmented(String filename) throws IOException{
		saveSegmented(filename,"UTF-8");
	}
	
	public void saveSegmented(String filename,String encoding) throws IOException{
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),encoding));
		for(Sentence<String> sentence:this){
			String s = "";
			for(String w:sentence){
				s += " "+w;				
			}
			if(!s.isEmpty()){
				s = s.substring(1);
			}
			writer.write(s);
			writer.newLine();			
		}
		writer.close();
	}
	
	public void saveForUnitex(String filename, String encoding) throws IOException {
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),encoding));
		for(Sentence<String> sentence:this){			
			writer.append(tokens2string(sentence.getSequence()));
			writer.append("{S}");
			writer.newLine();
		}
		writer.close();
		
	}
}
