/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * A Sentence<T> is a sequence of T 
 * 
 * 
 * @author Matthieu Constant
 *
 */


public class Sentence<T> implements Iterable<T>{
	private final List<T> sequence;

	/**
	 * Constructs an empty sentence
	 */
	
	public Sentence(){
		this.sequence = new ArrayList<T>();
	}
	
	/**
	 * Constructs a sentence with a sequence of elements
	 * @param sequence sequence of elements
	 */
	
	public Sentence(List<T> sequence){
		this.sequence = sequence;
	}
	
	
	
	@Override
	public Iterator<T> iterator() {
		return new Iterator<T>() {
            private final Iterator<T> iterator = sequence.iterator(); 
			@Override
			public boolean hasNext() {
				return iterator.hasNext();
			}

			@Override
			public T next() {
				return iterator.next();
			}

			@Override
			public void remove() {
				iterator.remove();
			}
			
		};
	}
	
	
	
	/**
	 * Add an element in the sentence
	 * 
	 * @param e element to add
	 */
	
	
	public void add(T e){
		sequence.add(e);
	}

	
	/**
	 * Adds an element at the first position in the sentence
	 * @param e element to be inserted
	 */
	
	
	public void addFirst(T e){
		sequence.add(0,e);
	}
	
	
	/**
	 * 
	 * @return the sequence of elements composing the sentence
	 */
	
	public List<T> getSequence(){
		return sequence;
	}
	
	/**
	 * 
	 * @param pos position in the sequence
	 * @return element at the given position 
	 */
	
	public T get(int pos){
		return sequence.get(pos);
	}
	

	/**
	 * 
	 * @return size of sentence
	 */
	
	public int size(){
		return sequence.size();
	}	
}
