/**
 * 
 *
 * Software: LGTools
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import fr.upemlv.lgtools.tagging.lex.Tagset;

/**
 * An abstract tagged text
 * 
 */

public class TaggedText extends AnnotatedText<TaggedElement>{
   public final static String TAGGED_TEXT_SEP = "\t";
   protected final Tagset tagset;
   
   protected TaggedText(Tagset tagset){
	   this.tagset = tagset;
   }
   
	
	
	@Override
	public void save(String filename,String encoding) throws IOException{	
		OutputStreamWriter out = new OutputStreamWriter(new FileOutputStream(filename),encoding); 
		write(out);
		out.close();		  
	}
	
	private void write(OutputStreamWriter out) throws IOException{		
		for(Sentence<TaggedElement> sentence:this){
			for(TaggedElement e:sentence){				    
			   out.append(e.getLexicalValue());
			   out.append(TAGGED_TEXT_SEP);
			   out.append(tagset.getPrintable(e.getTag().getValue()));
			   out.append("\n");
			}
			out.append("\n");
			out.flush();
		}
	}
	
	
	/**
	 * dump the tagged text on the standard output (in utf-8)
	 * @throws IOException
	 */
	
	public void dump() throws IOException{
		dump("utf-8");
	}
	
	/**
	 * dump the tagged text on the standard output (in a specified encoding)
	 * 
	 * @param encoding encoding
	 * @throws IOException
	 */
	
	
	public void dump(String encoding) throws IOException{
		
	   	write(new OutputStreamWriter(System.out,encoding));	   	
	}
	
	public FullRawText toRawText(){
		FullRawText t = new FullRawText();
		for(Sentence<TaggedElement> sentence:this){
			Sentence<String> s = new Sentence<String>();
			for(TaggedElement e:sentence){
				s.add(e.getLexicalValue());
			}
			t.addSentence(s);
		}
		return t;
	}



	/**
	 * @return the tagset
	 */
	public Tagset getTagset() {
		return tagset;
	}
	
}
