/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.text;

import java.util.List;

/**
 * Describes a template feature 
 * 
 * @author Matthieu Constant
 *
 */

public class TemplateFeature {
	private final String propertyType;
	private final List<String> arguments;
	private final int relativePosition;

	
	/**
	 * A template feature is constructed with a property type, its arguments and a relative position
	 * 
	 * @param propertyType property type
	 * @param arguments property arguments
	 * @param relativePosition relative position
	 */
	
	public TemplateFeature(String propertyType, List<String> arguments,
			int relativePosition) {
		this.propertyType = propertyType;
		this.arguments = arguments;
		this.relativePosition = relativePosition;
	}


	/**
	 * @return the propertyType
	 */
	public String getPropertyType() {
		return propertyType;
	}


	/**
	 * @return the arguments
	 */
	public List<String> getArguments() {
		return arguments;
	}


	/**
	 * @return the relativePosition
	 */
	public int getRelativePosition() {
		return relativePosition;
	}
	
	/**
	 * 
	 * @return the property name composed of its type and arguments
	 */
	
	public String getPropertyName(){
	  	String res = this.propertyType;
	  	if(this.arguments.isEmpty()){
	  		return res;
	  	}
	  	res += "(";
	  	for(String arg:this.arguments){	  		
	  		res += "," + arg;
	  	}
	  	res += ")";
	  	res = res.replace("(,", "(");
	  	return res;
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "(arguments=" + arguments + ", propertyType="
				+ propertyType + ", relativePosition=" + relativePosition + ")";
	}
	
	
	
}
