/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */
package fr.upemlv.lgtools.text;

import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import fr.upemlv.lgtools.tagging.impl.Utils;

/**
 * A Text is a sequence of sentences (Sentence)
 * 
 * 
 * @author Matthieu Constant
 *
 */

public abstract class Text<T> implements Iterable<Sentence<T>>{
     final private List<Sentence<T>> sentences = new LinkedList<Sentence<T>>();

     
	@Override
	public Iterator<Sentence<T>> iterator() {
		return new Iterator<Sentence<T>>() {
			Iterator<Sentence<T>> iterator = sentences.iterator();
			@Override
			public boolean hasNext() {
				return iterator.hasNext();
			}

			@Override
			public Sentence<T> next() {
				return iterator.next();
			}

			@Override
			public void remove() {
				iterator.remove();
			}			
		};
	}
     
	/**
	 * add a sentence in the text
	 * 
	 * @param sentence sentence
	 */
	
     public void addSentence(Sentence<T> sentence){
    	 sentences.add(sentence);
     }
     
     /**
      * 
      * @return the number of sentences in the text
      */
     
     public int sentenceCount(){
    	 return sentences.size();
     }
     
     /**
      * 
      * @return the number of tokens in the text
      */
     
     public int size(){
    	 int cnt = 0;
    	for(Sentence<T> s:sentences){
    		cnt += s.size();
    	}
    	return cnt++;
     }
     
     
     /**
      * Save text in a file encoded in UTF-8
      * 
      * @param filename filename
      * @throws IOException
      */
     
     public void save(String filename) throws IOException{
   	  save(filename, Utils.UTF8);
     }
     
     /**
      * Save text in a file
      * 
      * @param filename filename
      * @param encoding encoding
      * @throws IOException
      */
     
     abstract public void save(String filename,String encoding) throws IOException;     
     
}
