package fr.upemlv.lgtools.text;

import java.util.Iterator;

public class TextProcessingContext implements Iterable<SentenceProcessingContext>{
	private final TextFst textFst;
	private final TaggedText taggedText;
	private final String lang;
	
	/**
	 * @param textFst test fst after lexical analysis
	 * @param taggedText tagged text with a tagger
	 */
	public TextProcessingContext(TextFst textFst, TaggedText taggedText,String lang) {
		this.textFst = textFst;
		this.taggedText = taggedText;
		this.lang = lang;
	}

	@Override
	public Iterator<SentenceProcessingContext> iterator() {
		return new Iterator<SentenceProcessingContext>() {
			final Iterator<SentenceFst> itFst = textFst.iterator();
			//TAGGED TEXT
			final Iterator<Sentence<TaggedElement>> itTagged = taggedText == null?null:taggedText.iterator();
			@Override
			public boolean hasNext() {
				if(itTagged == null){
					return itFst.hasNext();
				}
				return itFst.hasNext() && itTagged.hasNext();
			}

			@Override
			public SentenceProcessingContext next() {
				if(itTagged == null){
					return new SentenceProcessingContext(itFst.next(), null,lang);
				}
				return new SentenceProcessingContext(itFst.next(), itTagged.next(),lang);
			}

			@Override
			public void remove() {			
			}
		};
	}
	
	
	
}
