/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.tree;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import fr.upemlv.lgtools.tagging.lex.LexicalEntry;

/**
 * Represents a node of a lexical tree
 * 
 * @author Matthieu Constant
 *
 */

public class LexicalNode {
	final private char value;
	final private List<LexicalNode> children = new LinkedList<LexicalNode>();
	private Set<LexicalEntry> tags; 
	
	public LexicalNode(char value){
		this.value = value;
	}

	/**
	 * @return the value
	 */
	public char getValue() {
		return value;
	}

	/**
	 * @return the children
	 */
	public List<LexicalNode> getChildren() {
		return children;
	}

	/**
	 * @return the tags
	 */
	public Set<LexicalEntry> getTags() {
		return tags;
	}
	
	private void addChild(LexicalNode n){
		children.add(n);
	}
	
	/**
	 * Finds the node of a certain value in the child nodes 
	 * 
	 * @param c value
	 * @param construct indicates if it is tree construction
	 * @return the node in the children with this value; null if not found and not at construction time 
	 */
	
	public LexicalNode findNode(char c,boolean construct){
		for(LexicalNode n:getChildren()){
			if(n.getValue() == c || n.getValue() == Character.toLowerCase(c)){
				return n;
			}
		}
		if(construct){
			LexicalNode n = new LexicalNode(c);
			addChild(n);
			return n;
		}
		return null;
	}
	
	public void addTag(LexicalEntry tag){
		if(tags == null){
			tags = new HashSet<LexicalEntry>();
		}
		tags.add(tag);
	}
}
