/**
 * 
 *
 * Software: LGTagger
 *
 * Copyright (C) 2010-2011 Université Paris-Est Marne-la-Vallée
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.upemlv.lgtools.utils;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.LinkedList;
import java.util.List;

/**
 * File Utilities
 * 
 * @author Matthieu Constant
 *
 */

public class FileUtilities {

	/**
	 * copy a file in another 
	 * 
	 * @param path path of the source file
	 * @param newPath path of the destination file
	 * @throws IOException 
	 */

	public static void copy(String path,String newPath) throws IOException{
		InputStream in = new FileInputStream(path);
		OutputStream out = new FileOutputStream(newPath);

		byte[] buf = new byte[1024];
		int len;
		while ((len = in.read(buf)) > 0) {
			out.write(buf, 0, len);
		}
		in.close();
		out.close();
	}

	
	
	/**
	 * 
	 * @param filename file name
	 * @return the list of elements (lines) in UTF-8 file
	 * @throws IOException
	 */
	
	
	public static List<String>  loadList(String filename) throws IOException{
		return loadList(filename, "UTF-8");
	}
	
	
	/**
	 * 
	 * @param filename filename
	 * @param encoding encoding
	 * @return the list of elements (lines) in file filename 
	 * @throws IOException
	 */
	
	public static List<String>  loadList(String filename,String encoding) throws IOException{
		List<String> list = new LinkedList<String>();
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(filename), encoding));
		String line;
		while((line = reader.readLine()) != null){
			line = line.trim();
			if(!line.isEmpty() && !line.startsWith("#")){ //# est un commentaire
				list.add(line);
			}	
		}
		reader.close();

		return list;
	}
	
	
	public static boolean createDir(String path){
		return new File(path).mkdirs();
	}
	
}
